@@ -20,6 +20,7 @@
  * should be used to speed up BSON encoding and decoding.
  */
 
+#define PY_SSIZE_T_CLEAN
 #include "Python.h"
 #include "datetime.h"
 
@@ -77,6 +78,7 @@ static struct module_state _state;
 #define STANDARD 4
 #define JAVA_LEGACY   5
 #define CSHARP_LEGACY 6
+#define UNSPECIFIED 0
 
 #define BSON_MAX_SIZE 2147483647
 /* The smallest possible BSON document, i.e. "{}" */
@@ -119,7 +121,14 @@ static PyObject* elements_to_dict(PyObject* self, const char* string,
 static int _write_element_to_buffer(PyObject* self, buffer_t buffer,
                                     int type_byte, PyObject* value,
                                     unsigned char check_keys,
-                                    const codec_options_t* options);
+                                    const codec_options_t* options,
+                                    unsigned char in_custom_call,
+                                    unsigned char in_fallback_call);
+
+/* Write a RawBSONDocument to the buffer.
+ * Returns the number of bytes written or 0 on failure.
+ */
+static int write_raw_doc(buffer_t buffer, PyObject* raw);
 
 /* Date stuff */
 static PyObject* datetime_from_millis(long long millis) {
@@ -185,7 +194,6 @@ static long long millis_from_datetime(PyObject* datetime) {
 /* Just make this compatible w/ the old API. */
 int buffer_write_bytes(buffer_t buffer, const char* data, int size) {
     if (buffer_write(buffer, data, size)) {
-        PyErr_NoMemory();
         return 0;
     }
     return 1;
@@ -345,9 +353,9 @@ static int _load_object(PyObject** object, char* module_name, char* object_name)
  *
  * Returns non-zero on failure. */
 static int _load_python_objects(PyObject* module) {
-    PyObject* empty_string;
-    PyObject* re_compile;
-    PyObject* compiled;
+    PyObject* empty_string = NULL;
+    PyObject* re_compile = NULL;
+    PyObject* compiled = NULL;
     struct module_state *state = GETSTATE(module);
 
     if (_load_object(&state->Binary, "bson.binary", "Binary") ||
@@ -447,6 +455,46 @@ static long _type_marker(PyObject* object) {
     return type;
 }
 
+/* Fill out a type_registry_t* from a TypeRegistry object.
+ *
+ * Return 1 on success. options->document_class is a new reference.
+ * Return 0 on failure.
+ */
+int convert_type_registry(PyObject* registry_obj, type_registry_t* registry) {
+    registry->encoder_map = NULL;
+    registry->decoder_map = NULL;
+    registry->fallback_encoder = NULL;
+    registry->registry_obj = NULL;
+
+    registry->encoder_map = PyObject_GetAttrString(registry_obj, "_encoder_map");
+    if (registry->encoder_map == NULL) {
+        goto fail;
+    }
+    registry->is_encoder_empty = (PyDict_Size(registry->encoder_map) == 0);
+
+    registry->decoder_map = PyObject_GetAttrString(registry_obj, "_decoder_map");
+    if (registry->decoder_map == NULL) {
+        goto fail;
+    }
+    registry->is_decoder_empty = (PyDict_Size(registry->decoder_map) == 0);
+
+    registry->fallback_encoder = PyObject_GetAttrString(registry_obj, "_fallback_encoder");
+    if (registry->fallback_encoder == NULL) {
+        goto fail;
+    }
+    registry->has_fallback_encoder = (registry->fallback_encoder != Py_None);
+
+    registry->registry_obj = registry_obj;
+    Py_INCREF(registry->registry_obj);
+    return 1;
+
+fail:
+    Py_XDECREF(registry->encoder_map);
+    Py_XDECREF(registry->decoder_map);
+    Py_XDECREF(registry->fallback_encoder);
+    return 0;
+}
+
 /* Fill out a codec_options_t* from a CodecOptions object. Use with the "O&"
  * format spec in PyArg_ParseTuple.
  *
@@ -455,25 +503,37 @@ static long _type_marker(PyObject* object) {
  */
 int convert_codec_options(PyObject* options_obj, void* p) {
     codec_options_t* options = (codec_options_t*)p;
+    PyObject* type_registry_obj = NULL;
     long type_marker;
+
     options->unicode_decode_error_handler = NULL;
-    if (!PyArg_ParseTuple(options_obj, "ObbzO",
+
+    if (!PyArg_ParseTuple(options_obj, "ObbzOO",
                           &options->document_class,
                           &options->tz_aware,
                           &options->uuid_rep,
                           &options->unicode_decode_error_handler,
-                          &options->tzinfo)) {
+                          &options->tzinfo,
+                          &type_registry_obj))
+        return 0;
+
+    type_marker = _type_marker(options->document_class);
+    if (type_marker < 0) {
         return 0;
     }
 
-    type_marker = _type_marker(options->document_class);
-    if (type_marker < 0) return 0;
+    if (!convert_type_registry(type_registry_obj,
+                               &options->type_registry)) {
+        return 0;
+    }
 
+    options->is_raw_bson = (101 == type_marker);
+    options->options_obj = options_obj;
+
+    Py_INCREF(options->options_obj);
     Py_INCREF(options->document_class);
     Py_INCREF(options->tzinfo);
-    options->options_obj = options_obj;
-    Py_INCREF(options->options_obj);
-    options->is_raw_bson = (101 == type_marker);
+
     return 1;
 }
 
@@ -501,67 +561,78 @@ void destroy_codec_options(codec_options_t* options) {
     Py_CLEAR(options->document_class);
     Py_CLEAR(options->tzinfo);
     Py_CLEAR(options->options_obj);
+    Py_CLEAR(options->type_registry.registry_obj);
+    Py_CLEAR(options->type_registry.encoder_map);
+    Py_CLEAR(options->type_registry.decoder_map);
+    Py_CLEAR(options->type_registry.fallback_encoder);
 }
 
 static int write_element_to_buffer(PyObject* self, buffer_t buffer,
                                    int type_byte, PyObject* value,
                                    unsigned char check_keys,
-                                   const codec_options_t* options) {
-    int result;
-    if(Py_EnterRecursiveCall(" while encoding an object to BSON "))
+                                   const codec_options_t* options,
+                                   unsigned char in_custom_call,
+                                   unsigned char in_fallback_call) {
+    int result = 0;
+    if(Py_EnterRecursiveCall(" while encoding an object to BSON ")) {
         return 0;
+    }
     result = _write_element_to_buffer(self, buffer, type_byte,
-                                      value, check_keys, options);
+                                      value, check_keys, options,
+                                      in_custom_call, in_fallback_call);
     Py_LeaveRecursiveCall();
     return result;
 }
 
-static void
-_fix_java(const char* in, char* out) {
-    int i, j;
-    for (i = 0, j = 7; i < j; i++, j--) {
-        out[i] = in[j];
-        out[j] = in[i];
-    }
-    for (i = 8, j = 15; i < j; i++, j--) {
-        out[i] = in[j];
-        out[j] = in[i];
-    }
-}
-
 static void
 _set_cannot_encode(PyObject* value) {
+    PyObject* type = NULL;
     PyObject* InvalidDocument = _error("InvalidDocument");
-    if (InvalidDocument) {
-        PyObject* repr = PyObject_Repr(value);
-        if (repr) {
-#if PY_MAJOR_VERSION >= 3
-            PyObject* errmsg = PyUnicode_FromString("Cannot encode object: ");
-#else
-            PyObject* errmsg = PyString_FromString("Cannot encode object: ");
-#endif
-            if (errmsg) {
-#if PY_MAJOR_VERSION >= 3
-                PyObject* error = PyUnicode_Concat(errmsg, repr);
-                if (error) {
-                    PyErr_SetObject(InvalidDocument, error);
-                    Py_DECREF(error);
-                }
-                Py_DECREF(errmsg);
-                Py_DECREF(repr);
-#else
-                PyString_ConcatAndDel(&errmsg, repr);
-                if (errmsg) {
-                    PyErr_SetObject(InvalidDocument, errmsg);
-                    Py_DECREF(errmsg);
-                }
-#endif
-            } else {
-                Py_DECREF(repr);
-            }
-        }
-        Py_DECREF(InvalidDocument);
+    if (InvalidDocument == NULL) {
+        goto error;
     }
+
+    type = PyObject_Type(value);
+    if (type == NULL) {
+        goto error;
+    }
+#if PY_MAJOR_VERSION >= 3
+    PyErr_Format(InvalidDocument, "cannot encode object: %R, of type: %R",
+                 value, type);
+#else
+     else {
+        PyObject* value_repr = NULL;
+        PyObject* type_repr = NULL;
+        char* value_str = NULL;
+        char* type_str = NULL;
+
+        value_repr = PyObject_Repr(value);
+        if (value_repr == NULL) {
+            goto py2error;
+        }
+        value_str = PyString_AsString(value_repr);
+        if (value_str == NULL) {
+            goto py2error;
+        }
+        type_repr = PyObject_Repr(type);
+        if (type_repr == NULL) {
+            goto py2error;
+        }
+        type_str = PyString_AsString(type_repr);
+        if (type_str == NULL) {
+            goto py2error;
+        }
+
+        PyErr_Format(InvalidDocument, "cannot encode object: %s, of type: %s",
+                     value_str, type_str);
+py2error:
+        Py_XDECREF(type_repr);
+        Py_XDECREF(value_repr);
+    }
+#endif
+error:
+    Py_XDECREF(type);
+    Py_XDECREF(InvalidDocument);
 }
 
 /*
@@ -697,9 +768,13 @@ static int _write_regex_to_buffer(
 static int _write_element_to_buffer(PyObject* self, buffer_t buffer,
                                     int type_byte, PyObject* value,
                                     unsigned char check_keys,
-                                    const codec_options_t* options) {
+                                    const codec_options_t* options,
+                                    unsigned char in_custom_call,
+                                    unsigned char in_fallback_call) {
     struct module_state *state = GETSTATE(self);
     PyObject* mapping_type;
+    PyObject* new_value = NULL;
+    int retval;
     PyObject* uuid_type;
     /*
      * Don't use PyObject_IsInstance for our custom types. It causes
@@ -785,7 +860,7 @@ static int _write_element_to_buffer(PyObject* self, buffer_t buffer,
         {
             /* ObjectId */
             const char* data;
-            PyObject* pystring = PyObject_GetAttrString(value, "_ObjectId__id");
+            PyObject* pystring = PyObject_GetAttrString(value, "binary");
             if (!pystring) {
                 return 0;
             }
@@ -835,7 +910,6 @@ static int _write_element_to_buffer(PyObject* self, buffer_t buffer,
             /* save space for length */
             length_location = buffer_save_space(buffer, 4);
             if (length_location == -1) {
-                PyErr_NoMemory();
                 Py_DECREF(scope);
                 return 0;
             }
@@ -949,39 +1023,10 @@ static int _write_element_to_buffer(PyObject* self, buffer_t buffer,
     case 101:
         {
             /* RawBSONDocument */
-            char* raw_bson_document_bytes;
-            Py_ssize_t raw_bson_document_bytes_len;
-            int raw_bson_document_bytes_len_int;
-            PyObject* raw_bson_document_bytes_obj = PyObject_GetAttrString(value, "raw");
-            if (!raw_bson_document_bytes_obj) {
-                return 0;
-            }
-
-#if PY_MAJOR_VERSION >= 3
-            if (-1 == PyBytes_AsStringAndSize(raw_bson_document_bytes_obj,
-                                              &raw_bson_document_bytes,
-                                              &raw_bson_document_bytes_len)) {
-#else
-            if (-1 == PyString_AsStringAndSize(raw_bson_document_bytes_obj,
-                                               &raw_bson_document_bytes,
-                                               &raw_bson_document_bytes_len)) {
-#endif
-                Py_DECREF(raw_bson_document_bytes_obj);
-                return 0;
-            }
-            raw_bson_document_bytes_len_int = _downcast_and_check(
-                raw_bson_document_bytes_len, 0);
-            if (-1 == raw_bson_document_bytes_len_int) {
-                Py_DECREF(raw_bson_document_bytes_obj);
-                return 0;
-            }
-            if(!buffer_write_bytes(buffer, raw_bson_document_bytes,
-                                   raw_bson_document_bytes_len_int)) {
-                Py_DECREF(raw_bson_document_bytes_obj);
+            if (!write_raw_doc(buffer, value)) {
                 return 0;
             }
             *(buffer_get_buffer(buffer) + type_byte) = 0x03;
-            Py_DECREF(raw_bson_document_bytes_obj);
             return 1;
         }
     case 255:
@@ -1062,7 +1107,6 @@ static int _write_element_to_buffer(PyObject* self, buffer_t buffer,
         /* save space for length */
         length_location = buffer_save_space(buffer, 4);
         if (length_location == -1) {
-            PyErr_NoMemory();
             return 0;
         }
 
@@ -1081,7 +1125,6 @@ static int _write_element_to_buffer(PyObject* self, buffer_t buffer,
             PyObject* item_value;
 
             if (list_type_byte == -1) {
-                PyErr_NoMemory();
                 return 0;
             }
             INT2STRING(name, (int)i);
@@ -1092,7 +1135,8 @@ static int _write_element_to_buffer(PyObject* self, buffer_t buffer,
             if (!(item_value = PySequence_GetItem(value, i)))
                 return 0;
             if (!write_element_to_buffer(self, buffer, list_type_byte,
-                                         item_value, check_keys, options)) {
+                                         item_value, check_keys, options,
+                                         0, 0)) {
                 Py_DECREF(item_value);
                 return 0;
             }
@@ -1220,14 +1264,9 @@ static int _write_element_to_buffer(PyObject* self, buffer_t buffer,
 
     uuid_type = _get_object(state->UUID, "uuid", "UUID");
     if (uuid_type && PyObject_IsInstance(value, uuid_type)) {
-        /* Just a special case of Binary above, but
-         * simpler to do as a separate case. */
-        PyObject* bytes;
-        /* Could be bytes, bytearray, str... */
-        const char* data;
-        /* UUID is always 16 bytes */
-        int size = 16;
-        char subtype;
+        PyObject* binary_type = NULL;
+        PyObject* binary_value = NULL;
+        int result;
 
         Py_DECREF(uuid_type);
         /* PyObject_IsInstance returns -1 on error */
@@ -1235,61 +1274,69 @@ static int _write_element_to_buffer(PyObject* self, buffer_t buffer,
             return 0;
         }
 
-        if (options->uuid_rep == JAVA_LEGACY
-                || options->uuid_rep == CSHARP_LEGACY) {
-            subtype = 3;
-        }
-        else {
-            subtype = options->uuid_rep;
-        }
-
-        *(buffer_get_buffer(buffer) + type_byte) = 0x05;
-        if (!buffer_write_int32(buffer, (int32_t)size)) {
-            return 0;
-        }
-        if (!buffer_write_bytes(buffer, &subtype, 1)) {
+        binary_type = _get_object(state->Binary, "bson", "Binary");
+        if (binary_type == NULL) {
             return 0;
         }
 
-        if (options->uuid_rep == CSHARP_LEGACY) {
-           /* Legacy C# byte order */
-            bytes = PyObject_GetAttrString(value, "bytes_le");
-        }
-        else {
-            bytes = PyObject_GetAttrString(value, "bytes");
-        }
-        if (!bytes) {
+        binary_value = PyObject_CallMethod(binary_type, "from_uuid", "(Oi)", value, options->uuid_rep);
+        if (binary_value == NULL) {
+            Py_DECREF(binary_type);
             return 0;
         }
-#if PY_MAJOR_VERSION >= 3
-        data = PyBytes_AsString(bytes);
-#else
-        data = PyString_AsString(bytes);
-#endif
-        if (data == NULL) {
-            Py_DECREF(bytes);
-            return 0;
-        }
-        if (options->uuid_rep == JAVA_LEGACY) {
-            /* Store in legacy java byte order. */
-            char as_legacy_java[16];
-            _fix_java(data, as_legacy_java);
-            if (!buffer_write_bytes(buffer, as_legacy_java, size)) {
-                Py_DECREF(bytes);
-                return 0;
-            }
-        }
-        else {
-            if (!buffer_write_bytes(buffer, data, size)) {
-                Py_DECREF(bytes);
-                return 0;
-            }
-        }
-        Py_DECREF(bytes);
-        return 1;
+
+        result = _write_element_to_buffer(self, buffer,
+                                          type_byte, binary_value,
+                                          check_keys, options,
+                                          in_custom_call,
+                                          in_fallback_call);
+        Py_DECREF(binary_type);
+        Py_DECREF(binary_value);
+        return result;
     }
     Py_XDECREF(mapping_type);
     Py_XDECREF(uuid_type);
+
+    /* Try a custom encoder if one is provided and we have not already
+     * attempted to use a type encoder. */
+    if (!in_custom_call && !options->type_registry.is_encoder_empty) {
+        PyObject* value_type = NULL;
+        PyObject* converter = NULL;
+        value_type = PyObject_Type(value);
+        if (value_type == NULL) {
+            return 0;
+        }
+        converter = PyDict_GetItem(options->type_registry.encoder_map, value_type);
+        Py_XDECREF(value_type);
+        if (converter != NULL) {
+            /* Transform types that have a registered converter.
+             * A new reference is created upon transformation. */
+            new_value = PyObject_CallFunctionObjArgs(converter, value, NULL);
+            if (new_value == NULL) {
+                return 0;
+            }
+            retval = write_element_to_buffer(self, buffer, type_byte, new_value,
+                                             check_keys, options, 1, 0);
+            Py_XDECREF(new_value);
+            return retval;
+        }
+    }
+
+    /* Try the fallback encoder if one is provided and we have not already
+     * attempted to use the fallback encoder. */
+    if (!in_fallback_call && options->type_registry.has_fallback_encoder) {
+        new_value = PyObject_CallFunctionObjArgs(
+            options->type_registry.fallback_encoder, value, NULL);
+        if (new_value == NULL) {
+            // propagate any exception raised by the callback
+            return 0;
+        }
+        retval = write_element_to_buffer(self, buffer, type_byte, new_value,
+                                         check_keys, options, 0, 1);
+        Py_XDECREF(new_value);
+        return retval;
+    }
+
     /* We can't determine value's type. Fail. */
     _set_cannot_encode(value);
     return 0;
@@ -1353,7 +1400,6 @@ int write_pair(PyObject* self, buffer_t buffer, const char* name, int name_lengt
 
     type_byte = buffer_save_space(buffer, 1);
     if (type_byte == -1) {
-        PyErr_NoMemory();
         return 0;
     }
     if (check_keys && !check_key_name(name, name_length)) {
@@ -1363,7 +1409,7 @@ int write_pair(PyObject* self, buffer_t buffer, const char* name, int name_lengt
         return 0;
     }
     if (!write_element_to_buffer(self, buffer, type_byte,
-                                 value, check_keys, options)) {
+                                 value, check_keys, options, 0, 0)) {
         return 0;
     }
     return 1;
@@ -1495,7 +1541,39 @@ int decode_and_write_pair(PyObject* self, buffer_t buffer,
     return 1;
 }
 
-/* returns 0 on failure */
+
+/* Write a RawBSONDocument to the buffer.
+ * Returns the number of bytes written or 0 on failure.
+ */
+static int write_raw_doc(buffer_t buffer, PyObject* raw) {
+    char* bytes;
+    Py_ssize_t len;
+    int len_int;
+    int bytes_written = 0;
+    PyObject* bytes_obj = NULL;
+
+    bytes_obj = PyObject_GetAttrString(raw, "raw");
+    if (!bytes_obj) {
+        goto fail;
+    }
+
+    if (-1 == PyBytes_AsStringAndSize(bytes_obj, &bytes, &len)) {
+        goto fail;
+    }
+    len_int = _downcast_and_check(len, 0);
+    if (-1 == len_int) {
+        goto fail;
+    }
+    if (!buffer_write_bytes(buffer, bytes, len_int)) {
+        goto fail;
+    }
+    bytes_written = len_int;
+fail:
+    Py_XDECREF(bytes_obj);
+    return bytes_written;
+}
+
+/* returns the number of bytes written or 0 on failure */
 int write_dict(PyObject* self, buffer_t buffer,
                PyObject* dict, unsigned char check_keys,
                const codec_options_t* options, unsigned char top_level) {
@@ -1505,12 +1583,23 @@ int write_dict(PyObject* self, buffer_t buffer,
     int length;
     int length_location;
     struct module_state *state = GETSTATE(self);
+    PyObject* mapping_type;
+    long type_marker;
+
+    /* check for RawBSONDocument */
+    type_marker = _type_marker(dict);
+    if (type_marker < 0) {
+        return 0;
+    }
+
+    if (101 == type_marker) {
+        return write_raw_doc(buffer, dict);
+    }
+
 #if PY_MAJOR_VERSION >= 3
-    PyObject* mapping_type = _get_object(state->Mapping,
-                                         "collections.abc", "Mapping");
+    mapping_type = _get_object(state->Mapping, "collections.abc", "Mapping");
 #else
-    PyObject* mapping_type = _get_object(state->Mapping,
-                                         "collections", "Mapping");
+    mapping_type = _get_object(state->Mapping, "collections", "Mapping");
 #endif
 
     if (mapping_type) {
@@ -1560,7 +1649,6 @@ int write_dict(PyObject* self, buffer_t buffer,
 
     length_location = buffer_save_space(buffer, 4);
     if (length_location == -1) {
-        PyErr_NoMemory();
         return 0;
     }
 
@@ -1629,7 +1717,7 @@ int write_dict(PyObject* self, buffer_t buffer,
     length = buffer_get_position(buffer) - length_location;
     buffer_write_int32_at_position(
         buffer, length_location, (int32_t)length);
-    return 1;
+    return length;
 }
 
 static PyObject* _cbson_dict_to_bson(PyObject* self, PyObject* args) {
@@ -1664,7 +1752,6 @@ static PyObject* _cbson_dict_to_bson(PyObject* self, PyObject* args) {
     buffer = buffer_new();
     if (!buffer) {
         destroy_codec_options(&options);
-        PyErr_NoMemory();
         return NULL;
     }
 
@@ -1675,13 +1762,8 @@ static PyObject* _cbson_dict_to_bson(PyObject* self, PyObject* args) {
     }
 
     /* objectify buffer */
-#if PY_MAJOR_VERSION >= 3
-    result = Py_BuildValue("y#", buffer_get_buffer(buffer),
-                           buffer_get_position(buffer));
-#else
-    result = Py_BuildValue("s#", buffer_get_buffer(buffer),
-                           buffer_get_position(buffer));
-#endif
+    result = Py_BuildValue(BYTES_FORMAT_STRING, buffer_get_buffer(buffer),
+                           (Py_ssize_t)buffer_get_position(buffer));
     destroy_codec_options(&options);
     buffer_free(buffer);
     return result;
@@ -1691,7 +1773,6 @@ static PyObject* get_value(PyObject* self, PyObject* name, const char* buffer,
                            unsigned* position, unsigned char type,
                            unsigned max, const codec_options_t* options) {
     struct module_state *state = GETSTATE(self);
-
     PyObject* value = NULL;
     switch (type) {
     case 1:
@@ -1753,7 +1834,7 @@ static PyObject* get_value(PyObject* self, PyObject* name, const char* buffer,
             if (options->is_raw_bson) {
                 value = PyObject_CallFunction(
                     options->document_class, BYTES_FORMAT_STRING "O",
-                    buffer + *position, size, options->options_obj);
+                    buffer + *position, (Py_ssize_t)size, options->options_obj);
                 if (!value) {
                     goto invalid;
                 }
@@ -1931,70 +2012,49 @@ static PyObject* get_value(PyObject* self, PyObject* name, const char* buffer,
             if (!data) {
                 goto invalid;
             }
-            /* Encode as UUID, not Binary */
+            /* Encode as UUID or Binary based on options->uuid_rep
+             * TODO: PYTHON-2245 Decoding should follow UUID spec in PyMongo 4.0  */
             if (subtype == 3 || subtype == 4) {
-                PyObject* kwargs;
-                PyObject* args = PyTuple_New(0);
+                PyObject* binary_type = NULL;
+                PyObject* binary_value = NULL;
+                char uuid_rep = options->uuid_rep;
+
                 /* UUID should always be 16 bytes */
-                if (!args || length != 16) {
-                    Py_DECREF(data);
-                    goto invalid;
-                }
-                kwargs = PyDict_New();
-                if (!kwargs) {
-                    Py_DECREF(data);
-                    Py_DECREF(args);
-                    goto invalid;
+                if (length != 16) {
+                    goto uuiderror;
                 }
 
-                /*
-                 * From this point, we hold refs to args, kwargs, and data.
-                 * If anything fails, goto uuiderror to clean them up.
-                 */
-                if (subtype == 3 && options->uuid_rep == CSHARP_LEGACY) {
-                    /* Legacy C# byte order */
-                    if ((PyDict_SetItemString(kwargs, "bytes_le", data)) == -1)
-                        goto uuiderror;
+                binary_type = _get_object(state->Binary, "bson", "Binary");
+                if (binary_type == NULL) {
+                    goto uuiderror;
                 }
-                else {
-                    if (subtype == 3 && options->uuid_rep == JAVA_LEGACY) {
-                        /* Convert from legacy java byte order */
-                        char big_endian[16];
-                        _fix_java(buffer + *position, big_endian);
-                        /* Free the previously created PyString object */
-                        Py_DECREF(data);
-#if PY_MAJOR_VERSION >= 3
-                        data = PyBytes_FromStringAndSize(big_endian, length);
-#else
-                        data = PyString_FromStringAndSize(big_endian, length);
-#endif
-                        if (data == NULL)
-                            goto uuiderror;
+
+                binary_value = PyObject_CallFunction(binary_type, "(Oi)", data, subtype);
+                if (binary_value == NULL) {
+                    goto uuiderror;
+                }
+
+                if (uuid_rep == UNSPECIFIED) {
+                    value = binary_value;
+                    Py_INCREF(value);
+                } else {
+                    if (subtype == 4) {
+                        uuid_rep = STANDARD;
+                    } else if (uuid_rep == STANDARD) {
+                        uuid_rep = PYTHON_LEGACY;
                     }
-                    if ((PyDict_SetItemString(kwargs, "bytes", data)) == -1)
-                        goto uuiderror;
-
-                }
-                if ((type_to_create = _get_object(state->UUID, "uuid", "UUID"))) {
-                    value = PyObject_Call(type_to_create, args, kwargs);
-                    Py_DECREF(type_to_create);
+                    value = PyObject_CallMethod(binary_value, "as_uuid", "(i)", uuid_rep);
                 }
 
-                Py_DECREF(args);
-                Py_DECREF(kwargs);
+            uuiderror:
+                Py_XDECREF(binary_type);
+                Py_XDECREF(binary_value);
                 Py_DECREF(data);
                 if (!value) {
                     goto invalid;
                 }
-
                 *position += length;
                 break;
-
-            uuiderror:
-                Py_DECREF(args);
-                Py_DECREF(kwargs);
-                Py_XDECREF(data);
-                goto invalid;
             }
 
 #if PY_MAJOR_VERSION >= 3
@@ -2032,11 +2092,8 @@ static PyObject* get_value(PyObject* self, PyObject* name, const char* buffer,
                 goto invalid;
             }
             if ((objectid_type = _get_object(state->ObjectId, "bson.objectid", "ObjectId"))) {
-#if PY_MAJOR_VERSION >= 3
-                value = PyObject_CallFunction(objectid_type, "y#", buffer + *position, 12);
-#else
-                value = PyObject_CallFunction(objectid_type, "s#", buffer + *position, 12);
-#endif
+                value = PyObject_CallFunction(objectid_type, BYTES_FORMAT_STRING,
+                                              buffer + *position, (Py_ssize_t)12);
                 Py_DECREF(objectid_type);
             }
             *position += 12;
@@ -2120,6 +2177,7 @@ static PyObject* get_value(PyObject* self, PyObject* name, const char* buffer,
             /* convert to local time */
             if (options->tzinfo != Py_None) {
                 astimezone = PyObject_GetAttrString(value, "astimezone");
+                Py_DECREF(value);
                 if (!astimezone) {
                     Py_DECREF(replace);
                     Py_DECREF(args);
@@ -2221,11 +2279,8 @@ static PyObject* get_value(PyObject* self, PyObject* name, const char* buffer,
             *position += coll_length;
 
             if ((objectid_type = _get_object(state->ObjectId, "bson.objectid", "ObjectId"))) {
-#if PY_MAJOR_VERSION >= 3
-                id = PyObject_CallFunction(objectid_type, "y#", buffer + *position, 12);
-#else
-                id = PyObject_CallFunction(objectid_type, "s#", buffer + *position, 12);
-#endif
+                id = PyObject_CallFunction(objectid_type, BYTES_FORMAT_STRING,
+                                           buffer + *position, (Py_ssize_t)12);
                 Py_DECREF(objectid_type);
             }
             if (!id) {
@@ -2412,13 +2467,9 @@ static PyObject* get_value(PyObject* self, PyObject* name, const char* buffer,
                                       "Decimal128"))) {
                 value = PyObject_CallMethod(dec128,
                                             "from_bid",
-#if PY_MAJOR_VERSION >= 3
-                                            "y#",
-#else
-                                            "s#",
-#endif
+                                            BYTES_FORMAT_STRING,
                                             buffer + *position,
-                                            16);
+                                            (Py_ssize_t)16);
                 Py_DECREF(dec128);
             }
             *position += 16;
@@ -2482,6 +2533,24 @@ static PyObject* get_value(PyObject* self, PyObject* name, const char* buffer,
     }
 
     if (value) {
+        if (!options->type_registry.is_decoder_empty) {
+            PyObject* value_type = NULL;
+            PyObject* converter = NULL;
+            value_type = PyObject_Type(value);
+            if (value_type == NULL) {
+                goto invalid;
+            }
+            converter = PyDict_GetItem(options->type_registry.decoder_map, value_type);
+            if (converter != NULL) {
+                PyObject* new_value = PyObject_CallFunctionObjArgs(converter, value, NULL);
+                Py_DECREF(value_type);
+                Py_DECREF(value);
+                return new_value;
+            } else {
+                Py_DECREF(value_type);
+                return value;
+            }
+        }
         return value;
     }
 
@@ -2595,6 +2664,7 @@ static int _element_to_dict(PyObject* self, const char* string,
 }
 
 static PyObject* _cbson_element_to_dict(PyObject* self, PyObject* args) {
+    /* TODO: Support buffer protocol */
     char* string;
     PyObject* bson;
     codec_options_t options;
@@ -2686,35 +2756,52 @@ static PyObject* elements_to_dict(PyObject* self, const char* string,
     return result;
 }
 
+static int _get_buffer(PyObject *exporter, Py_buffer *view) {
+    if (PyObject_GetBuffer(exporter, view, PyBUF_SIMPLE) == -1) {
+        return 0;
+    }
+    if (!PyBuffer_IsContiguous(view, 'C')) {
+        PyErr_SetString(PyExc_ValueError,
+                        "must be a contiguous buffer");
+        goto fail;
+    }
+    if (!view->buf || view->len < 0) {
+        PyErr_SetString(PyExc_ValueError, "invalid buffer");
+        goto fail;
+    }
+    if (view->itemsize != 1) {
+        PyErr_SetString(PyExc_ValueError,
+                        "buffer data must be ascii or utf8");
+        goto fail;
+    }
+    return 1;
+fail:
+    PyBuffer_Release(view);
+    return 0;
+}
+
 static PyObject* _cbson_bson_to_dict(PyObject* self, PyObject* args) {
     int32_t size;
     Py_ssize_t total_size;
     const char* string;
     PyObject* bson;
     codec_options_t options;
-    PyObject* result;
+    PyObject* result = NULL;
     PyObject* options_obj;
+    Py_buffer view;
 
     if (! (PyArg_ParseTuple(args, "OO", &bson, &options_obj) &&
             convert_codec_options(options_obj, &options))) {
-        return NULL;
+        return result;
     }
 
-#if PY_MAJOR_VERSION >= 3
-    if (!PyBytes_Check(bson)) {
-        PyErr_SetString(PyExc_TypeError, "argument to _bson_to_dict must be a bytes object");
-#else
-    if (!PyString_Check(bson)) {
-        PyErr_SetString(PyExc_TypeError, "argument to _bson_to_dict must be a string");
-#endif
+    if (!_get_buffer(bson, &view)) {
         destroy_codec_options(&options);
-        return NULL;
+        return result;
     }
-#if PY_MAJOR_VERSION >= 3
-    total_size = PyBytes_Size(bson);
-#else
-    total_size = PyString_Size(bson);
-#endif
+
+    total_size = view.len;
+
     if (total_size < BSON_MIN_SIZE) {
         PyObject* InvalidBSON = _error("InvalidBSON");
         if (InvalidBSON) {
@@ -2722,19 +2809,10 @@ static PyObject* _cbson_bson_to_dict(PyObject* self, PyObject* args) {
                             "not enough data for a BSON document");
             Py_DECREF(InvalidBSON);
         }
-        destroy_codec_options(&options);
-        return NULL;
+        goto done;;
     }
 
-#if PY_MAJOR_VERSION >= 3
-    string = PyBytes_AsString(bson);
-#else
-    string = PyString_AsString(bson);
-#endif
-    if (!string) {
-        destroy_codec_options(&options);
-        return NULL;
-    }
+    string = (char*)view.buf;
 
     memcpy(&size, string, 4);
     size = (int32_t)BSON_UINT32_FROM_LE(size);
@@ -2744,8 +2822,7 @@ static PyObject* _cbson_bson_to_dict(PyObject* self, PyObject* args) {
             PyErr_SetString(InvalidBSON, "invalid message size");
             Py_DECREF(InvalidBSON);
         }
-        destroy_codec_options(&options);
-        return NULL;
+        goto done;
     }
 
     if (total_size < size || total_size > BSON_MAX_SIZE) {
@@ -2754,8 +2831,7 @@ static PyObject* _cbson_bson_to_dict(PyObject* self, PyObject* args) {
             PyErr_SetString(InvalidBSON, "objsize too large");
             Py_DECREF(InvalidBSON);
         }
-        destroy_codec_options(&options);
-        return NULL;
+        goto done;
     }
 
     if (size != total_size || string[size - 1]) {
@@ -2764,18 +2840,20 @@ static PyObject* _cbson_bson_to_dict(PyObject* self, PyObject* args) {
             PyErr_SetString(InvalidBSON, "bad eoo");
             Py_DECREF(InvalidBSON);
         }
-        destroy_codec_options(&options);
-        return NULL;
+        goto done;
     }
 
     /* No need to decode fields if using RawBSONDocument */
     if (options.is_raw_bson) {
-        return PyObject_CallFunction(
-            options.document_class, BYTES_FORMAT_STRING "O", string, size,
+        result = PyObject_CallFunction(
+            options.document_class, BYTES_FORMAT_STRING "O", string, (Py_ssize_t)size,
             options_obj);
     }
-
-    result = elements_to_dict(self, string + 4, (unsigned)size - 5, &options);
+    else {
+        result = elements_to_dict(self, string + 4, (unsigned)size - 5, &options);
+    }
+done:
+    PyBuffer_Release(&view);
     destroy_codec_options(&options);
     return result;
 }
@@ -2786,9 +2864,10 @@ static PyObject* _cbson_decode_all(PyObject* self, PyObject* args) {
     const char* string;
     PyObject* bson;
     PyObject* dict;
-    PyObject* result;
+    PyObject* result = NULL;
     codec_options_t options;
     PyObject* options_obj;
+    Py_buffer view;
 
     if (!PyArg_ParseTuple(args, "O|O", &bson, &options_obj)) {
         return NULL;
@@ -2801,31 +2880,15 @@ static PyObject* _cbson_decode_all(PyObject* self, PyObject* args) {
         return NULL;
     }
 
-#if PY_MAJOR_VERSION >= 3
-    if (!PyBytes_Check(bson)) {
-        PyErr_SetString(PyExc_TypeError, "argument to decode_all must be a bytes object");
-#else
-    if (!PyString_Check(bson)) {
-        PyErr_SetString(PyExc_TypeError, "argument to decode_all must be a string");
-#endif
-        destroy_codec_options(&options);
-        return NULL;
-    }
-#if PY_MAJOR_VERSION >= 3
-    total_size = PyBytes_Size(bson);
-    string = PyBytes_AsString(bson);
-#else
-    total_size = PyString_Size(bson);
-    string = PyString_AsString(bson);
-#endif
-    if (!string) {
+    if (!_get_buffer(bson, &view)) {
         destroy_codec_options(&options);
         return NULL;
     }
+    total_size = view.len;
+    string = (char*)view.buf;
 
     if (!(result = PyList_New(0))) {
-        destroy_codec_options(&options);
-        return NULL;
+        goto fail;
     }
 
     while (total_size > 0) {
@@ -2836,9 +2899,8 @@ static PyObject* _cbson_decode_all(PyObject* self, PyObject* args) {
                                 "not enough data for a BSON document");
                 Py_DECREF(InvalidBSON);
             }
-            destroy_codec_options(&options);
             Py_DECREF(result);
-            return NULL;
+            goto fail;
         }
 
         memcpy(&size, string, 4);
@@ -2849,9 +2911,8 @@ static PyObject* _cbson_decode_all(PyObject* self, PyObject* args) {
                 PyErr_SetString(InvalidBSON, "invalid message size");
                 Py_DECREF(InvalidBSON);
             }
-            destroy_codec_options(&options);
             Py_DECREF(result);
-            return NULL;
+            goto fail;
         }
 
         if (total_size < size) {
@@ -2860,9 +2921,8 @@ static PyObject* _cbson_decode_all(PyObject* self, PyObject* args) {
                 PyErr_SetString(InvalidBSON, "objsize too large");
                 Py_DECREF(InvalidBSON);
             }
-            destroy_codec_options(&options);
             Py_DECREF(result);
-            return NULL;
+            goto fail;
         }
 
         if (string[size - 1]) {
@@ -2871,35 +2931,36 @@ static PyObject* _cbson_decode_all(PyObject* self, PyObject* args) {
                 PyErr_SetString(InvalidBSON, "bad eoo");
                 Py_DECREF(InvalidBSON);
             }
-            destroy_codec_options(&options);
             Py_DECREF(result);
-            return NULL;
+            goto fail;
         }
 
         /* No need to decode fields if using RawBSONDocument. */
         if (options.is_raw_bson) {
             dict = PyObject_CallFunction(
-                options.document_class, BYTES_FORMAT_STRING "O", string, size,
+                options.document_class, BYTES_FORMAT_STRING "O", string, (Py_ssize_t)size,
                 options_obj);
         } else {
             dict = elements_to_dict(self, string + 4, (unsigned)size - 5, &options);
         }
         if (!dict) {
             Py_DECREF(result);
-            destroy_codec_options(&options);
-            return NULL;
+            goto fail;
         }
         if (PyList_Append(result, dict) < 0) {
             Py_DECREF(dict);
             Py_DECREF(result);
-            destroy_codec_options(&options);
-            return NULL;
+            goto fail;
         }
         Py_DECREF(dict);
         string += size;
         total_size -= size;
     }
-
+    goto done;
+fail:
+    result = NULL;
+done:
+    PyBuffer_Release(&view);
     destroy_codec_options(&options);
     return result;
 }
@@ -2989,6 +3050,7 @@ init_cbson(void)
     _cbson_API[_cbson_buffer_write_int64_INDEX] = (void *) buffer_write_int64;
     _cbson_API[_cbson_buffer_write_int32_at_position_INDEX] =
         (void *) buffer_write_int32_at_position;
+    _cbson_API[_cbson_downcast_and_check_INDEX] = (void *) _downcast_and_check;
 
 #if PY_VERSION_HEX >= 0x03010000
     /* PyCapsule is new in python 3.1 */
