@@ -1,5 +1,5 @@
 /*
- * Copyright 2009-2015 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,15 +19,6 @@
 #ifndef _CBSONMODULE_H
 #define _CBSONMODULE_H
 
-/* Py_ssize_t was new in python 2.5. See conversion
- * guidlines in http://www.python.org/dev/peps/pep-0353
- * */
-#if PY_VERSION_HEX < 0x02050000 && !defined(PY_SSIZE_T_MIN)
-typedef int Py_ssize_t;
-#define PY_SSIZE_T_MAX INT_MAX
-#define PY_SSIZE_T_MIN INT_MIN
-#endif
-
 #if defined(WIN32) || defined(_MSC_VER)
 /*
  * This macro is basically an implementation of asprintf for win32
@@ -60,12 +51,23 @@ typedef int Py_ssize_t;
 #define BYTES_FORMAT_STRING "s#"
 #endif
 
+typedef struct type_registry_t {
+    PyObject* encoder_map;
+    PyObject* decoder_map;
+    PyObject* fallback_encoder;
+    PyObject* registry_obj;
+    unsigned char is_encoder_empty;
+    unsigned char is_decoder_empty;
+    unsigned char has_fallback_encoder;
+} type_registry_t;
+
 typedef struct codec_options_t {
     PyObject* document_class;
     unsigned char tz_aware;
     unsigned char uuid_rep;
     char* unicode_decode_error_handler;
     PyObject* tzinfo;
+    type_registry_t type_registry;
     PyObject* options_obj;
     unsigned char is_raw_bson;
 } codec_options_t;
@@ -111,8 +113,12 @@ typedef struct codec_options_t {
 #define _cbson_buffer_write_int32_at_position_RETURN void
 #define _cbson_buffer_write_int32_at_position_PROTO (buffer_t buffer, int position, int32_t data)
 
+#define _cbson_downcast_and_check_INDEX 10
+#define _cbson_downcast_and_check_RETURN int
+#define _cbson_downcast_and_check_PROTO (Py_ssize_t size, uint8_t extra)
+
 /* Total number of C API pointers */
-#define _cbson_API_POINTER_COUNT 10
+#define _cbson_API_POINTER_COUNT 11
 
 #ifdef _CBSON_MODULE
 /* This section is used when compiling _cbsonmodule */
@@ -137,6 +143,8 @@ static _cbson_buffer_write_int64_RETURN buffer_write_int64 _cbson_buffer_write_i
 
 static _cbson_buffer_write_int32_at_position_RETURN buffer_write_int32_at_position _cbson_buffer_write_int32_at_position_PROTO;
 
+static _cbson_downcast_and_check_RETURN _downcast_and_check _cbson_downcast_and_check_PROTO;
+
 #else
 /* This section is used in modules that use _cbsonmodule's API */
 
@@ -162,6 +170,8 @@ static void **_cbson_API;
 
 #define buffer_write_int32_at_position (*(_cbson_buffer_write_int32_at_position_RETURN (*)_cbson_buffer_write_int32_at_position_PROTO) _cbson_API[_cbson_buffer_write_int32_at_position_INDEX])
 
+#define _downcast_and_check (*(_cbson_downcast_and_check_RETURN (*)_cbson_downcast_and_check_PROTO) _cbson_API[_cbson_downcast_and_check_INDEX])
+
 #define _cbson_IMPORT _cbson_API = (void **)PyCapsule_Import("_cbson._C_API", 0)
 
 #endif
