@@ -1,5 +1,5 @@
 /*
- * Copyright 2009-2014 MongoDB, Inc.
+ * Copyright 2009-2015 MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,6 +14,10 @@
  * limitations under the License.
  */
 
+/* Include Python.h so we can set Python's error indicator. */
+#define PY_SSIZE_T_CLEAN
+#include "Python.h"
+
 #include <stdlib.h>
 #include <string.h>
 
@@ -27,12 +31,19 @@ struct buffer {
     int position;
 };
 
+/* Set Python's error indicator to MemoryError.
+ * Called after allocation failures. */
+static void set_memory_error(void) {
+    PyErr_NoMemory();
+}
+
 /* Allocate and return a new buffer.
- * Return NULL on allocation failure. */
+ * Return NULL and sets MemoryError on allocation failure. */
 buffer_t buffer_new(void) {
     buffer_t buffer;
     buffer = (buffer_t)malloc(sizeof(struct buffer));
     if (buffer == NULL) {
+        set_memory_error();
         return NULL;
     }
 
@@ -41,6 +52,7 @@ buffer_t buffer_new(void) {
     buffer->buffer = (char*)malloc(sizeof(char) * INITIAL_BUFFER_SIZE);
     if (buffer->buffer == NULL) {
         free(buffer);
+        set_memory_error();
         return NULL;
     }
 
@@ -53,13 +65,16 @@ int buffer_free(buffer_t buffer) {
     if (buffer == NULL) {
         return 1;
     }
-    free(buffer->buffer);
+    /* Buffer will be NULL when buffer_grow fails. */
+    if (buffer->buffer != NULL) {
+        free(buffer->buffer);
+    }
     free(buffer);
     return 0;
 }
 
 /* Grow `buffer` to at least `min_length`.
- * Return non-zero on allocation failure. */
+ * Return non-zero and sets MemoryError on allocation failure. */
 static int buffer_grow(buffer_t buffer, int min_length) {
     int old_size = 0;
     int size = buffer->size;
@@ -79,7 +94,7 @@ static int buffer_grow(buffer_t buffer, int min_length) {
     buffer->buffer = (char*)realloc(buffer->buffer, sizeof(char) * size);
     if (buffer->buffer == NULL) {
         free(old_buffer);
-        free(buffer);
+        set_memory_error();
         return 1;
     }
     buffer->size = size;
@@ -87,16 +102,26 @@ static int buffer_grow(buffer_t buffer, int min_length) {
 }
 
 /* Assure that `buffer` has at least `size` free bytes (and grow if needed).
- * Return non-zero on allocation failure. */
+ * Return non-zero and sets MemoryError on allocation failure.
+ * Return non-zero and sets ValueError if `size` would exceed 2GiB. */
 static int buffer_assure_space(buffer_t buffer, int size) {
-    if (buffer->position + size <= buffer->size) {
+    int new_size = buffer->position + size;
+    /* Check for overflow. */
+    if (new_size < buffer->position) {
+        PyErr_SetString(PyExc_ValueError,
+                        "Document would overflow BSON size limit");
+        return 1;
+    }
+
+    if (new_size <= buffer->size) {
         return 0;
     }
-    return buffer_grow(buffer, buffer->position + size);
+    return buffer_grow(buffer, new_size);
 }
 
 /* Save `size` bytes from the current position in `buffer` (and grow if needed).
- * Return offset for writing, or -1 on allocation failure. */
+ * Return offset for writing, or -1 on failure.
+ * Sets MemoryError or ValueError on failure. */
 buffer_position buffer_save_space(buffer_t buffer, int size) {
     int position = buffer->position;
     if (buffer_assure_space(buffer, size) != 0) {
@@ -107,7 +132,8 @@ buffer_position buffer_save_space(buffer_t buffer, int size) {
 }
 
 /* Write `size` bytes from `data` to `buffer` (and grow if needed).
- * Return non-zero on allocation failure. */
+ * Return non-zero on failure.
+ * Sets MemoryError or ValueError on failure. */
 int buffer_write(buffer_t buffer, const char* data, int size) {
     if (buffer_assure_space(buffer, size) != 0) {
         return 1;
@@ -118,21 +144,6 @@ int buffer_write(buffer_t buffer, const char* data, int size) {
     return 0;
 }
 
-/* Write `size` bytes from `data` to `buffer` at position `position`.
- * Does not change the internal position of `buffer`.
- * Return non-zero if buffer isn't large enough for write. */
-int buffer_write_at_position(buffer_t buffer, buffer_position position,
-                             const char* data, int size) {
-    if (position + size > buffer->size) {
-        buffer_free(buffer);
-        return 1;
-    }
-
-    memcpy(buffer->buffer + position, data, size);
-    return 0;
-}
-
-
 int buffer_get_position(buffer_t buffer) {
     return buffer->position;
 }
