@@ -1,4 +1,4 @@
-# Copyright 2009-2014 MongoDB, Inc.
+# Copyright 2009-2015 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,58 +19,44 @@
 import binascii
 import calendar
 import datetime
-try:
-    import hashlib
-    _md5func = hashlib.md5
-except ImportError:  # for Python < 2.5
-    import md5
-    _md5func = md5.new
 import os
-import random
-import socket
 import struct
 import threading
 import time
 
+from random import SystemRandom
+
 from bson.errors import InvalidId
-from bson.py3compat import (PY3, b, binary_type, text_type,
-                            bytes_from_hex, string_types)
+from bson.py3compat import PY3, bytes_from_hex, string_type, text_type
 from bson.tz_util import utc
 
-EMPTY = b("")
-ZERO  = b("\x00")
 
-def _machine_bytes():
-    """Get the machine portion of an ObjectId.
-    """
-    machine_hash = _md5func()
-    if PY3:
-        # gethostname() returns a unicode string in python 3.x
-        # while update() requires a byte string.
-        machine_hash.update(socket.gethostname().encode())
-    else:
-        # Calling encode() here will fail with non-ascii hostnames
-        machine_hash.update(socket.gethostname())
-    return machine_hash.digest()[0:3]
+_MAX_COUNTER_VALUE = 0xFFFFFF
 
 
 def _raise_invalid_id(oid):
     raise InvalidId(
         "%r is not a valid ObjectId, it must be a 12-byte input"
-        " of type %r or a 24-character hex string" % (
-            oid, binary_type.__name__))
+        " or a 24-character hex string" % oid)
+
+
+def _random_bytes():
+    """Get the 5-byte random field of an ObjectId."""
+    return os.urandom(5)
 
 
 class ObjectId(object):
     """A MongoDB ObjectId.
     """
 
-    _inc = random.randint(0, 0xFFFFFF)
+    _pid = os.getpid()
+
+    _inc = SystemRandom().randint(0, _MAX_COUNTER_VALUE)
     _inc_lock = threading.Lock()
 
-    _machine_bytes = _machine_bytes()
+    __random = _random_bytes()
 
-    __slots__ = ('__id')
+    __slots__ = ('__id',)
 
     _type_marker = 7
 
@@ -80,8 +66,7 @@ class ObjectId(object):
         An ObjectId is a 12-byte unique identifier consisting of:
 
           - a 4-byte value representing the seconds since the Unix epoch,
-          - a 3-byte machine identifier,
-          - a 2-byte process id, and
+          - a 5-byte random value,
           - a 3-byte counter, starting with a random value.
 
         By default, ``ObjectId()`` creates a new unique identifier. The
@@ -109,14 +94,18 @@ class ObjectId(object):
         :Parameters:
           - `oid` (optional): a valid ObjectId.
 
-        .. versionadded:: 1.2.1
-           The `oid` parameter can be a ``unicode`` instance (that contains
-           24 hexadecimal digits).
-
         .. mongodoc:: objectids
+
+        .. versionchanged:: 3.8
+           :class:`~bson.objectid.ObjectId` now implements the `ObjectID
+           specification version 0.2
+           <https://github.com/mongodb/specifications/blob/master/source/
+           objectid.rst>`_.
         """
         if oid is None:
             self.__generate()
+        elif isinstance(oid, bytes) and len(oid) == 12:
+            self.__id = oid
         else:
             self.__validate(oid)
 
@@ -147,17 +136,12 @@ class ObjectId(object):
         :Parameters:
           - `generation_time`: :class:`~datetime.datetime` to be used
             as the generation time for the resulting ObjectId.
-
-        .. versionchanged:: 1.8
-           Properly handle timezone aware values for
-           `generation_time`.
-
-        .. versionadded:: 1.6
         """
         if generation_time.utcoffset() is not None:
             generation_time = generation_time - generation_time.utcoffset()
-        ts = calendar.timegm(generation_time.timetuple())
-        oid = struct.pack(">i", int(ts)) + ZERO * 8
+        timestamp = calendar.timegm(generation_time.timetuple())
+        oid = struct.pack(
+            ">I", int(timestamp)) + b"\x00\x00\x00\x00\x00\x00\x00\x00"
         return cls(oid)
 
     @classmethod
@@ -178,25 +162,30 @@ class ObjectId(object):
         except (InvalidId, TypeError):
             return False
 
+    @classmethod
+    def _random(cls):
+        """Generate a 5-byte random number once per process.
+        """
+        pid = os.getpid()
+        if pid != cls._pid:
+            cls._pid = pid
+            cls.__random = _random_bytes()
+        return cls.__random
+
     def __generate(self):
         """Generate a new value for this ObjectId.
         """
-        oid = EMPTY
 
         # 4 bytes current time
-        oid += struct.pack(">i", int(time.time()))
+        oid = struct.pack(">I", int(time.time()))
 
-        # 3 bytes machine
-        oid += ObjectId._machine_bytes
-
-        # 2 bytes pid
-        oid += struct.pack(">H", os.getpid() % 0xFFFF)
+        # 5 bytes random
+        oid += ObjectId._random()
 
         # 3 bytes inc
-        ObjectId._inc_lock.acquire()
-        oid += struct.pack(">i", ObjectId._inc)[1:4]
-        ObjectId._inc = (ObjectId._inc + 1) % 0xFFFFFF
-        ObjectId._inc_lock.release()
+        with ObjectId._inc_lock:
+            oid += struct.pack(">I", ObjectId._inc)[1:4]
+            ObjectId._inc = (ObjectId._inc + 1) % (_MAX_COUNTER_VALUE + 1)
 
         self.__id = oid
 
@@ -212,14 +201,10 @@ class ObjectId(object):
           - `oid`: a valid ObjectId
         """
         if isinstance(oid, ObjectId):
-            self.__id = oid.__id
-        elif isinstance(oid, string_types):
-            if len(oid) == 12:
-                if isinstance(oid, binary_type):
-                    self.__id = oid
-                else:
-                    _raise_invalid_id(oid)
-            elif len(oid) == 24:
+            self.__id = oid.binary
+        # bytes or unicode in python 2, str in python 3
+        elif isinstance(oid, string_type):
+            if len(oid) == 24:
                 try:
                     self.__id = bytes_from_hex(oid)
                 except (TypeError, ValueError):
@@ -227,9 +212,8 @@ class ObjectId(object):
             else:
                 _raise_invalid_id(oid)
         else:
-            raise TypeError("id must be an instance of (%s, %s, ObjectId), "
-                            "not %s" % (binary_type.__name__,
-                                        text_type.__name__, type(oid)))
+            raise TypeError("id must be an instance of (bytes, %s, ObjectId), "
+                            "not %s" % (text_type.__name__, type(oid)))
 
     @property
     def binary(self):
@@ -245,14 +229,9 @@ class ObjectId(object):
         The :class:`datetime.datetime` is timezone aware, and
         represents the generation time in UTC. It is precise to the
         second.
-
-        .. versionchanged:: 1.8
-           Now return an aware datetime instead of a naive one.
-
-        .. versionadded:: 1.2
         """
-        t = struct.unpack(">i", self.__id[0:4])[0]
-        return datetime.datetime.fromtimestamp(t, utc)
+        timestamp = struct.unpack(">I", self.__id[0:4])[0]
+        return datetime.datetime.fromtimestamp(timestamp, utc)
 
     def __getstate__(self):
         """return value of object for pickling.
@@ -287,37 +266,34 @@ class ObjectId(object):
 
     def __eq__(self, other):
         if isinstance(other, ObjectId):
-            return self.__id == other.__id
+            return self.__id == other.binary
         return NotImplemented
 
     def __ne__(self, other):
         if isinstance(other, ObjectId):
-            return self.__id != other.__id
+            return self.__id != other.binary
         return NotImplemented
 
     def __lt__(self, other):
         if isinstance(other, ObjectId):
-            return self.__id < other.__id
+            return self.__id < other.binary
         return NotImplemented
 
     def __le__(self, other):
         if isinstance(other, ObjectId):
-            return self.__id <= other.__id
+            return self.__id <= other.binary
         return NotImplemented
 
     def __gt__(self, other):
         if isinstance(other, ObjectId):
-            return self.__id > other.__id
+            return self.__id > other.binary
         return NotImplemented
 
     def __ge__(self, other):
         if isinstance(other, ObjectId):
-            return self.__id >= other.__id
+            return self.__id >= other.binary
         return NotImplemented
 
     def __hash__(self):
-        """Get a hash value for this :class:`ObjectId`.
-
-        .. versionadded:: 1.1
-        """
+        """Get a hash value for this :class:`ObjectId`."""
         return hash(self.__id)
