@@ -26,9 +26,15 @@ GEO2D = "2d"
 """
 
 GEOHAYSTACK = "geoHaystack"
-"""Index specifier for a 2-dimensional `haystack index`_.
+"""**DEPRECATED** - Index specifier for a 2-dimensional `haystack index`_.
 
-.. versionadded:: 2.1
+**DEPRECATED** - :attr:`GEOHAYSTACK` is deprecated and will be removed in
+PyMongo 4.0. geoHaystack indexes (and the geoSearch command) were deprecated
+in MongoDB 4.4. Instead, create a 2d index and use $geoNear or $geoWithin.
+See https://dochub.mongodb.org/core/4.4-deprecate-geoHaystack.
+
+.. versionchanged:: 3.11
+   Deprecated.
 
 .. _haystack index: http://docs.mongodb.org/manual/core/geohaystack/
 """
@@ -52,6 +58,10 @@ HASHED = "hashed"
 TEXT = "text"
 """Index specifier for a `text index`_.
 
+.. seealso:: MongoDB's `Atlas Search
+   <https://docs.atlas.mongodb.com/atlas-search/>`_ which offers more advanced
+   text search functionality.
+
 .. versionadded:: 2.7.1
 
 .. _text index: http://docs.mongodb.org/manual/core/index-text/
@@ -64,7 +74,7 @@ SLOW_ONLY = 1
 ALL = 2
 """Profile all operations."""
 
-version_tuple = (3, 10, 1)
+version_tuple = (3, 11, 0)
 
 def get_version_string():
     if isinstance(version_tuple[-1], str):
