@@ -88,7 +88,6 @@ static int add_last_error(PyObject* self, buffer_t buffer,
 
     message_start = buffer_save_space(buffer, 4);
     if (message_start == -1) {
-        PyErr_NoMemory();
         return 0;
     }
     if (!buffer_write_int32(buffer, (int32_t)request_id) ||
@@ -109,7 +108,6 @@ static int add_last_error(PyObject* self, buffer_t buffer,
     /* save space for length */
     document_start = buffer_save_space(buffer, 4);
     if (document_start == -1) {
-        PyErr_NoMemory();
         return 0;
     }
 
@@ -154,7 +152,6 @@ static int init_insert_buffer(buffer_t buffer, int request_id, int options,
         /* Save space for message length */
         int length_location = buffer_save_space(buffer, 4);
         if (length_location == -1) {
-            PyErr_NoMemory();
             return length_location;
         }
         if (!buffer_write_int32(buffer, (int32_t)request_id) ||
@@ -193,9 +190,9 @@ static PyObject* _cbson_insert_message(PyObject* self, PyObject* args) {
     unsigned char continue_on_error;
     codec_options_t options;
     PyObject* last_error_args;
-    buffer_t buffer;
+    buffer_t buffer = NULL;
     int length_location, message_length;
-    PyObject* result;
+    PyObject* result = NULL;
 
     if (!PyArg_ParseTuple(args, "et#ObbObO&",
                           "utf-8",
@@ -212,10 +209,7 @@ static PyObject* _cbson_insert_message(PyObject* self, PyObject* args) {
     }
     buffer = buffer_new();
     if (!buffer) {
-        PyErr_NoMemory();
-        destroy_codec_options(&options);
-        PyMem_Free(collection_name);
-        return NULL;
+        goto fail;
     }
 
     length_location = init_insert_buffer(buffer,
@@ -225,10 +219,7 @@ static PyObject* _cbson_insert_message(PyObject* self, PyObject* args) {
                                          collection_name_length,
                                          0);
     if (length_location == -1) {
-        destroy_codec_options(&options);
-        PyMem_Free(collection_name);
-        buffer_free(buffer);
-        return NULL;
+        goto fail;
     }
 
     iterator = PyObject_GetIter(docs);
@@ -238,10 +229,7 @@ static PyObject* _cbson_insert_message(PyObject* self, PyObject* args) {
             PyErr_SetString(InvalidOperation, "input is not iterable");
             Py_DECREF(InvalidOperation);
         }
-        destroy_codec_options(&options);
-        buffer_free(buffer);
-        PyMem_Free(collection_name);
-        return NULL;
+        goto fail;
     }
     while ((doc = PyIter_Next(iterator)) != NULL) {
         before = buffer_get_position(buffer);
@@ -249,10 +237,7 @@ static PyObject* _cbson_insert_message(PyObject* self, PyObject* args) {
                         &options, 1)) {
             Py_DECREF(doc);
             Py_DECREF(iterator);
-            destroy_codec_options(&options);
-            buffer_free(buffer);
-            PyMem_Free(collection_name);
-            return NULL;
+            goto fail;
         }
         Py_DECREF(doc);
         cur_size = buffer_get_position(buffer) - before;
@@ -261,10 +246,7 @@ static PyObject* _cbson_insert_message(PyObject* self, PyObject* args) {
     Py_DECREF(iterator);
 
     if (PyErr_Occurred()) {
-        destroy_codec_options(&options);
-        buffer_free(buffer);
-        PyMem_Free(collection_name);
-        return NULL;
+        goto fail;
     }
 
     if (!max_size) {
@@ -273,10 +255,7 @@ static PyObject* _cbson_insert_message(PyObject* self, PyObject* args) {
             PyErr_SetString(InvalidOperation, "cannot do an empty bulk insert");
             Py_DECREF(InvalidOperation);
         }
-        destroy_codec_options(&options);
-        buffer_free(buffer);
-        PyMem_Free(collection_name);
-        return NULL;
+        goto fail;
     }
 
     message_length = buffer_get_position(buffer) - length_location;
@@ -286,22 +265,21 @@ static PyObject* _cbson_insert_message(PyObject* self, PyObject* args) {
     if (safe) {
         if (!add_last_error(self, buffer, request_id, collection_name,
                             collection_name_length, &options, last_error_args)) {
-            destroy_codec_options(&options);
-            buffer_free(buffer);
-            PyMem_Free(collection_name);
-            return NULL;
+            goto fail;
         }
     }
 
-    PyMem_Free(collection_name);
-
     /* objectify buffer */
     result = Py_BuildValue("i" BYTES_FORMAT_STRING "i", request_id,
                            buffer_get_buffer(buffer),
                            (Py_ssize_t)buffer_get_position(buffer),
                            max_size);
+fail:
+    PyMem_Free(collection_name);
     destroy_codec_options(&options);
-    buffer_free(buffer);
+    if (buffer) {
+        buffer_free(buffer);
+    }
     return result;
 }
 
@@ -322,9 +300,9 @@ static PyObject* _cbson_update_message(PyObject* self, PyObject* args) {
     codec_options_t options;
     PyObject* last_error_args;
     int flags;
-    buffer_t buffer;
+    buffer_t buffer = NULL;
     int length_location, message_length;
-    PyObject* result;
+    PyObject* result = NULL;
 
     if (!PyArg_ParseTuple(args, "et#bbOObObO&",
                           "utf-8",
@@ -345,19 +323,13 @@ static PyObject* _cbson_update_message(PyObject* self, PyObject* args) {
     }
     buffer = buffer_new();
     if (!buffer) {
-        destroy_codec_options(&options);
-        PyErr_NoMemory();
-        PyMem_Free(collection_name);
-        return NULL;
+        goto fail;
     }
 
     // save space for message length
     length_location = buffer_save_space(buffer, 4);
     if (length_location == -1) {
-        destroy_codec_options(&options);
-        PyMem_Free(collection_name);
-        PyErr_NoMemory();
-        return NULL;
+        goto fail;
     }
     if (!buffer_write_int32(buffer, (int32_t)request_id) ||
         !buffer_write_bytes(buffer,
@@ -369,28 +341,19 @@ static PyObject* _cbson_update_message(PyObject* self, PyObject* args) {
                                     collection_name,
                                     collection_name_length + 1) ||
         !buffer_write_int32(buffer, (int32_t)flags)) {
-        destroy_codec_options(&options);
-        buffer_free(buffer);
-        PyMem_Free(collection_name);
-        return NULL;
+        goto fail;
     }
 
     before = buffer_get_position(buffer);
     if (!write_dict(state->_cbson, buffer, spec, 0, &options, 1)) {
-        destroy_codec_options(&options);
-        buffer_free(buffer);
-        PyMem_Free(collection_name);
-        return NULL;
+        goto fail;
     }
     max_size = buffer_get_position(buffer) - before;
 
     before = buffer_get_position(buffer);
     if (!write_dict(state->_cbson, buffer, doc, check_keys,
                     &options, 1)) {
-        destroy_codec_options(&options);
-        buffer_free(buffer);
-        PyMem_Free(collection_name);
-        return NULL;
+        goto fail;
     }
     cur_size = buffer_get_position(buffer) - before;
     max_size = (cur_size > max_size) ? cur_size : max_size;
@@ -402,22 +365,21 @@ static PyObject* _cbson_update_message(PyObject* self, PyObject* args) {
     if (safe) {
         if (!add_last_error(self, buffer, request_id, collection_name,
                             collection_name_length, &options, last_error_args)) {
-            destroy_codec_options(&options);
-            buffer_free(buffer);
-            PyMem_Free(collection_name);
-            return NULL;
+            goto fail;
         }
     }
 
-    PyMem_Free(collection_name);
-
     /* objectify buffer */
     result = Py_BuildValue("i" BYTES_FORMAT_STRING "i", request_id,
                            buffer_get_buffer(buffer),
                            (Py_ssize_t)buffer_get_position(buffer),
                            max_size);
+fail:
+    PyMem_Free(collection_name);
     destroy_codec_options(&options);
-    buffer_free(buffer);
+    if (buffer) {
+        buffer_free(buffer);
+    }
     return result;
 }
 
@@ -436,7 +398,7 @@ static PyObject* _cbson_query_message(PyObject* self, PyObject* args) {
     PyObject* query;
     PyObject* field_selector;
     codec_options_t options;
-    buffer_t buffer;
+    buffer_t buffer = NULL;
     int length_location, message_length;
     unsigned char check_keys = 0;
     PyObject* result = NULL;
@@ -454,19 +416,13 @@ static PyObject* _cbson_query_message(PyObject* self, PyObject* args) {
     }
     buffer = buffer_new();
     if (!buffer) {
-        PyErr_NoMemory();
-        destroy_codec_options(&options);
-        PyMem_Free(collection_name);
-        return NULL;
+        goto fail;
     }
 
     // save space for message length
     length_location = buffer_save_space(buffer, 4);
     if (length_location == -1) {
-        destroy_codec_options(&options);
-        PyMem_Free(collection_name);
-        PyErr_NoMemory();
-        return NULL;
+        goto fail;
     }
 
     /* Pop $clusterTime from dict and write it at the end, avoiding an error
@@ -555,11 +511,12 @@ static PyObject* _cbson_query_message(PyObject* self, PyObject* args) {
                            buffer_get_buffer(buffer),
                            (Py_ssize_t)buffer_get_position(buffer),
                            max_size);
-
 fail:
     PyMem_Free(collection_name);
     destroy_codec_options(&options);
-    buffer_free(buffer);
+    if (buffer) {
+        buffer_free(buffer);
+    }
     Py_XDECREF(cluster_time);
     return result;
 }
@@ -571,9 +528,9 @@ static PyObject* _cbson_get_more_message(PyObject* self, PyObject* args) {
     Py_ssize_t collection_name_length;
     int num_to_return;
     long long cursor_id;
-    buffer_t buffer;
+    buffer_t buffer = NULL;
     int length_location, message_length;
-    PyObject* result;
+    PyObject* result = NULL;
 
     if (!PyArg_ParseTuple(args, "et#iL",
                           "utf-8",
@@ -585,17 +542,13 @@ static PyObject* _cbson_get_more_message(PyObject* self, PyObject* args) {
     }
     buffer = buffer_new();
     if (!buffer) {
-        PyErr_NoMemory();
-        PyMem_Free(collection_name);
-        return NULL;
+        goto fail;
     }
 
     // save space for message length
     length_location = buffer_save_space(buffer, 4);
     if (length_location == -1) {
-        PyMem_Free(collection_name);
-        PyErr_NoMemory();
-        return NULL;
+        goto fail;
     }
     if (!buffer_write_int32(buffer, (int32_t)request_id) ||
         !buffer_write_bytes(buffer,
@@ -607,13 +560,9 @@ static PyObject* _cbson_get_more_message(PyObject* self, PyObject* args) {
                                     collection_name_length + 1) ||
         !buffer_write_int32(buffer, (int32_t)num_to_return) ||
         !buffer_write_int64(buffer, (int64_t)cursor_id)) {
-        buffer_free(buffer);
-        PyMem_Free(collection_name);
-        return NULL;
+        goto fail;
     }
 
-    PyMem_Free(collection_name);
-
     message_length = buffer_get_position(buffer) - length_location;
     buffer_write_int32_at_position(
         buffer, length_location, (int32_t)message_length);
@@ -622,7 +571,11 @@ static PyObject* _cbson_get_more_message(PyObject* self, PyObject* args) {
     result = Py_BuildValue("i" BYTES_FORMAT_STRING, request_id,
                            buffer_get_buffer(buffer),
                            (Py_ssize_t)buffer_get_position(buffer));
-    buffer_free(buffer);
+fail:
+    PyMem_Free(collection_name);
+    if (buffer) {
+        buffer_free(buffer);
+    }
     return result;
 }
 
@@ -644,7 +597,7 @@ static PyObject* _cbson_op_msg(PyObject* self, PyObject* args) {
     PyObject* doc;
     unsigned char check_keys = 0;
     codec_options_t options;
-    buffer_t buffer;
+    buffer_t buffer = NULL;
     int length_location, message_length;
     int total_size = 0;
     int max_doc_size = 0;
@@ -665,48 +618,46 @@ static PyObject* _cbson_op_msg(PyObject* self, PyObject* args) {
     }
     buffer = buffer_new();
     if (!buffer) {
-        PyErr_NoMemory();
-        goto bufferfail;
+        goto fail;
     }
 
     // save space for message length
     length_location = buffer_save_space(buffer, 4);
     if (length_location == -1) {
-        PyErr_NoMemory();
-        goto bufferfail;
+        goto fail;
     }
     if (!buffer_write_int32(buffer, (int32_t)request_id) ||
         !buffer_write_bytes(buffer,
                             "\x00\x00\x00\x00" /* responseTo */
                             "\xdd\x07\x00\x00" /* 2013 */, 8)) {
-        goto encodefail;
+        goto fail;
     }
 
     if (!buffer_write_int32(buffer, (int32_t)flags) ||
         !buffer_write_bytes(buffer, "\x00", 1) /* Payload type 0 */) {
-        goto encodefail;
+        goto fail;
     }
     total_size = write_dict(state->_cbson, buffer, command, 0,
                           &options, 1);
     if (!total_size) {
-        goto encodefail;
+        goto fail;
     }
 
     if (identifier_length) {
         int payload_one_length_location, payload_length;
         /* Payload type 1 */
         if (!buffer_write_bytes(buffer, "\x01", 1)) {
-            goto encodefail;
+            goto fail;
         }
         /* save space for payload 0 length */
         payload_one_length_location = buffer_save_space(buffer, 4);
         /* C string identifier */
         if (!buffer_write_bytes_ssize_t(buffer, identifier, identifier_length + 1)) {
-            goto encodefail;
+            goto fail;
         }
         iterator = PyObject_GetIter(docs);
         if (iterator == NULL) {
-            goto encodefail;
+            goto fail;
         }
         while ((doc = PyIter_Next(iterator)) != NULL) {
             int encoded_doc_size = write_dict(
@@ -714,7 +665,7 @@ static PyObject* _cbson_op_msg(PyObject* self, PyObject* args) {
                       &options, 1);
             if (!encoded_doc_size) {
                 Py_CLEAR(doc);
-                goto encodefail;
+                goto fail;
             }
             if (encoded_doc_size > max_doc_size) {
                 max_doc_size = encoded_doc_size;
@@ -738,10 +689,11 @@ static PyObject* _cbson_op_msg(PyObject* self, PyObject* args) {
                            (Py_ssize_t)buffer_get_position(buffer),
                            total_size,
                            max_doc_size);
-encodefail:
+fail:
     Py_XDECREF(iterator);
-    buffer_free(buffer);
-bufferfail:
+    if (buffer) {
+        buffer_free(buffer);
+    }
     PyMem_Free(identifier);
     destroy_codec_options(&options);
     return result;
@@ -879,7 +831,6 @@ static PyObject* _cbson_do_batched_insert(PyObject* self, PyObject* args) {
     buffer = buffer_new();
     if (!buffer) {
         destroy_codec_options(&options);
-        PyErr_NoMemory();
         PyMem_Free(collection_name);
         return NULL;
     }
@@ -944,7 +895,6 @@ static PyObject* _cbson_do_batched_insert(PyObject* self, PyObject* args) {
             int message_start;
             buffer_t new_buffer = buffer_new();
             if (!new_buffer) {
-                PyErr_NoMemory();
                 goto iterfail;
             }
             message_start = init_insert_buffer(new_buffer,
@@ -1181,7 +1131,6 @@ _batched_op_msg(
     /* Save space for size */
     size_location = buffer_save_space(buffer, 4);
     if (size_location == -1) {
-        PyErr_NoMemory();
         return 0;
     }
 
@@ -1325,7 +1274,6 @@ _cbson_encode_batched_op_msg(PyObject* self, PyObject* args) {
         return NULL;
     }
     if (!(buffer = buffer_new())) {
-        PyErr_NoMemory();
         destroy_codec_options(&options);
         return NULL;
     }
@@ -1381,13 +1329,11 @@ _cbson_batched_op_msg(PyObject* self, PyObject* args) {
         return NULL;
     }
     if (!(buffer = buffer_new())) {
-        PyErr_NoMemory();
         destroy_codec_options(&options);
         return NULL;
     }
     /* Save space for message length and request id */
     if ((buffer_save_space(buffer, 8)) == -1) {
-        PyErr_NoMemory();
         goto fail;
     }
     if (!buffer_write_bytes(buffer,
@@ -1552,7 +1498,6 @@ _batched_write_command(
     /* Save space for list document */
     lst_len_loc = buffer_save_space(buffer, 4);
     if (lst_len_loc == -1) {
-        PyErr_NoMemory();
         return 0;
     }
 
@@ -1672,7 +1617,6 @@ _cbson_encode_batched_write_command(PyObject* self, PyObject* args) {
         return NULL;
     }
     if (!(buffer = buffer_new())) {
-        PyErr_NoMemory();
         PyMem_Free(ns);
         destroy_codec_options(&options);
         return NULL;
@@ -1732,14 +1676,12 @@ _cbson_batched_write_command(PyObject* self, PyObject* args) {
         return NULL;
     }
     if (!(buffer = buffer_new())) {
-        PyErr_NoMemory();
         PyMem_Free(ns);
         destroy_codec_options(&options);
         return NULL;
     }
     /* Save space for message length and request id */
     if ((buffer_save_space(buffer, 8)) == -1) {
-        PyErr_NoMemory();
         goto fail;
     }
     if (!buffer_write_bytes(buffer,
