@@ -65,10 +65,13 @@ transaction automatically calls :meth:`ClientSession.commit_transaction`.
 If the block exits with an exception, the transaction automatically calls
 :meth:`ClientSession.abort_transaction`.
 
-For multi-document transactions, you can only specify read/write (CRUD)
-operations on existing collections. For example, a multi-document transaction
-cannot include a create or drop collection/index operations, including an
+In general, multi-document transactions only support read/write (CRUD)
+operations on existing collections. However, MongoDB 4.4 adds support for
+creating collections and indexes with some limitations, including an
 insert operation that would result in the creation of a new collection.
+For a complete description of all the supported and unsupported operations
+see the `MongoDB server's documentation for transactions
+<http://dochub.mongodb.org/core/transactions>`_.
 
 A session may only have a single active transaction at a time, multiple
 transactions on the same session can be executed in sequence.
@@ -95,13 +98,11 @@ Classes
 """
 
 import collections
-import os
-import sys
 import uuid
 
 from bson.binary import Binary
 from bson.int64 import Int64
-from bson.py3compat import abc, integer_types, reraise_instance
+from bson.py3compat import abc, integer_types
 from bson.son import SON
 from bson.timestamp import Timestamp
 
@@ -111,7 +112,6 @@ from pymongo.errors import (ConfigurationError,
                             InvalidOperation,
                             OperationFailure,
                             PyMongoError,
-                            ServerSelectionTimeoutError,
                             WTimeoutError)
 from pymongo.helpers import _RETRYABLE_ERROR_CODES
 from pymongo.read_concern import ReadConcern
@@ -292,6 +292,7 @@ class _Transaction(object):
         self.sharded = False
         self.pinned_address = None
         self.recovery_token = None
+        self.attempt = 0
 
     def active(self):
         return self.state in (_TxnState.STARTING, _TxnState.IN_PROGRESS)
@@ -301,12 +302,13 @@ class _Transaction(object):
         self.sharded = False
         self.pinned_address = None
         self.recovery_token = None
+        self.attempt = 0
 
 
 def _reraise_with_unknown_commit(exc):
     """Re-raise an exception with the UnknownTransactionCommitResult label."""
     exc._add_error_label("UnknownTransactionCommitResult")
-    reraise_instance(exc, trace=sys.exc_info()[2])
+    raise
 
 
 def _max_time_expired_error(exc):
@@ -334,7 +336,17 @@ def _within_time_limit(start_time):
 
 
 class ClientSession(object):
-    """A session for ordering sequential operations."""
+    """A session for ordering sequential operations.
+
+    :class:`ClientSession` instances are **not thread-safe or fork-safe**.
+    They can only be used by one thread or process at a time. A single
+    :class:`ClientSession` cannot be used to run multiple operations
+    concurrently.
+
+    Should not be initialized directly by application developers - to create a
+    :class:`ClientSession`, call
+    :meth:`~pymongo.mongo_client.MongoClient.start_session`.
+    """
     def __init__(self, client, server_session, options, authset, implicit):
         # A MongoClient, a _ServerSession, a SessionOptions, and a set.
         self._client = client
@@ -459,6 +471,10 @@ class ClientSession(object):
         however, ``with_transaction`` will return without taking further
         action.
 
+        :class:`ClientSession` instances are **not thread-safe or fork-safe**.
+        Consequently, the ``callback`` must not attempt to execute multiple
+        operations concurrently.
+
         When ``callback`` raises an exception, ``with_transaction``
         automatically aborts the current transaction. When ``callback`` or
         :meth:`~ClientSession.commit_transaction` raises an exception that
@@ -576,7 +592,6 @@ class ClientSession(object):
         .. versionadded:: 3.7
         """
         self._check_ended()
-        retry = False
         state = self._transaction.state
         if state is _TxnState.NONE:
             raise InvalidOperation("No transaction started")
@@ -591,10 +606,9 @@ class ClientSession(object):
             # We're explicitly retrying the commit, move the state back to
             # "in progress" so that in_transaction returns true.
             self._transaction.state = _TxnState.IN_PROGRESS
-            retry = True
 
         try:
-            self._finish_transaction_with_retry("commitTransaction", retry)
+            self._finish_transaction_with_retry("commitTransaction")
         except ConnectionFailure as exc:
             # We do not know if the commit was successfully applied on the
             # server or if it satisfied the provided write concern, set the
@@ -637,44 +651,25 @@ class ClientSession(object):
                 "Cannot call abortTransaction after calling commitTransaction")
 
         try:
-            self._finish_transaction_with_retry("abortTransaction", False)
+            self._finish_transaction_with_retry("abortTransaction")
         except (OperationFailure, ConnectionFailure):
             # The transactions spec says to ignore abortTransaction errors.
             pass
         finally:
             self._transaction.state = _TxnState.ABORTED
 
-    def _finish_transaction_with_retry(self, command_name, explict_retry):
+    def _finish_transaction_with_retry(self, command_name):
         """Run commit or abort with one retry after any retryable error.
 
         :Parameters:
           - `command_name`: Either "commitTransaction" or "abortTransaction".
-          - `explict_retry`: True when this is an explict commit retry attempt,
-            ie the application called session.commit_transaction() twice.
         """
-        # This can be refactored with MongoClient._retry_with_session.
-        try:
-            return self._finish_transaction(command_name, explict_retry)
-        except ServerSelectionTimeoutError:
-            raise
-        except ConnectionFailure as exc:
-            try:
-                return self._finish_transaction(command_name, True)
-            except ServerSelectionTimeoutError:
-                # Raise the original error so the application can infer that
-                # an attempt was made.
-                raise exc
-        except OperationFailure as exc:
-            if exc.code not in _RETRYABLE_ERROR_CODES:
-                raise
-            try:
-                return self._finish_transaction(command_name, True)
-            except ServerSelectionTimeoutError:
-                # Raise the original error so the application can infer that
-                # an attempt was made.
-                raise exc
+        def func(session, sock_info, retryable):
+            return self._finish_transaction(sock_info, command_name)
+        return self._client._retry_internal(True, func, self, None)
 
-    def _finish_transaction(self, command_name, retrying):
+    def _finish_transaction(self, sock_info, command_name):
+        self._transaction.attempt += 1
         opts = self._transaction.opts
         wc = opts.write_concern
         cmd = SON([(command_name, 1)])
@@ -685,7 +680,7 @@ class ClientSession(object):
             # Transaction spec says that after the initial commit attempt,
             # subsequent commitTransaction commands should be upgraded to use
             # w:"majority" and set a default value of 10 seconds for wtimeout.
-            if retrying:
+            if self._transaction.attempt > 1:
                 wc_doc = wc.document
                 wc_doc["w"] = "majority"
                 wc_doc.setdefault("wtimeout", 10000)
@@ -694,13 +689,12 @@ class ClientSession(object):
         if self._transaction.recovery_token:
             cmd['recoveryToken'] = self._transaction.recovery_token
 
-        with self._client._socket_for_writes(self) as sock_info:
-            return self._client.admin._command(
-                sock_info,
-                cmd,
-                session=self,
-                write_concern=wc,
-                parse_write_concern_error=True)
+        return self._client.admin._command(
+            sock_info,
+            cmd,
+            session=self,
+            write_concern=wc,
+            parse_write_concern_error=True)
 
     def _advance_cluster_time(self, cluster_time):
         """Internal cluster time helper."""
@@ -835,13 +829,13 @@ class ClientSession(object):
 
 
 class _ServerSession(object):
-    def __init__(self, pool_id):
+    def __init__(self, generation):
         # Ensure id is type 4, regardless of CodecOptions.uuid_representation.
         self.session_id = {'id': Binary(uuid.uuid4().bytes, 4)}
         self.last_use = monotonic.time()
         self._transaction_id = 0
         self.dirty = False
-        self.pool_id = pool_id
+        self.generation = generation
 
     def mark_dirty(self):
         """Mark this session as dirty.
@@ -873,10 +867,10 @@ class _ServerSessionPool(collections.deque):
     """
     def __init__(self, *args, **kwargs):
         super(_ServerSessionPool, self).__init__(*args, **kwargs)
-        self.pool_id = 0
+        self.generation = 0
 
     def reset(self):
-        self.pool_id += 1
+        self.generation += 1
         self.clear()
 
     def pop_all(self):
@@ -899,7 +893,7 @@ class _ServerSessionPool(collections.deque):
             if not s.timed_out(session_timeout_minutes):
                 return s
 
-        return _ServerSession(self.pool_id)
+        return _ServerSession(self.generation)
 
     def return_server_session(self, server_session, session_timeout_minutes):
         self._clear_stale(session_timeout_minutes)
@@ -909,7 +903,8 @@ class _ServerSessionPool(collections.deque):
     def return_server_session_no_lock(self, server_session):
         # Discard sessions from an old pool to avoid duplicate sessions in the
         # child process after a fork.
-        if server_session.pool_id == self.pool_id and not server_session.dirty:
+        if (server_session.generation == self.generation and
+                not server_session.dirty):
             self.appendleft(server_session)
 
     def _clear_stale(self, session_timeout_minutes):
