@@ -15,7 +15,6 @@
 """Cursor class to iterate over Mongo query results."""
 
 import copy
-import datetime
 import warnings
 
 from collections import deque
@@ -29,18 +28,17 @@ from bson.son import SON
 from pymongo import helpers
 from pymongo.common import validate_boolean, validate_is_mapping
 from pymongo.collation import validate_collation_or_none
-from pymongo.errors import (AutoReconnect,
-                            ConnectionFailure,
+from pymongo.errors import (ConnectionFailure,
                             InvalidOperation,
                             NotMasterError,
                             OperationFailure)
-from pymongo.message import (_convert_exception,
-                             _CursorAddress,
+from pymongo.message import (_CursorAddress,
                              _GetMore,
                              _RawBatchGetMore,
                              _Query,
                              _RawBatchQuery)
-from pymongo.read_preferences import ReadPreference
+from pymongo.monitoring import ConnectionClosedReason
+
 
 _QUERY_OPTIONS = {
     "tailable_cursor": 2,
@@ -116,7 +114,8 @@ class Cursor(object):
                  modifiers=None, batch_size=0, manipulate=True,
                  collation=None, hint=None, max_scan=None, max_time_ms=None,
                  max=None, min=None, return_key=False, show_record_id=False,
-                 snapshot=False, comment=None, session=None):
+                 snapshot=False, comment=None, session=None,
+                 allow_disk_use=None):
         """Create a new cursor.
 
         Should not be called directly by application developers - see
@@ -161,6 +160,9 @@ class Cursor(object):
             raise TypeError("batch_size must be an integer")
         if batch_size < 0:
             raise ValueError("batch_size must be >= 0")
+        # Only set if allow_disk_use is provided by the user, else None.
+        if allow_disk_use is not None:
+            allow_disk_use = validate_boolean("allow_disk_use", allow_disk_use)
 
         if projection is not None:
             if not projection:
@@ -186,6 +188,7 @@ class Cursor(object):
         self.__collation = validate_collation_or_none(collation)
         self.__return_key = return_key
         self.__show_record_id = show_record_id
+        self.__allow_disk_use = allow_disk_use
         self.__snapshot = snapshot
         self.__set_hint(hint)
 
@@ -223,6 +226,10 @@ class Cursor(object):
         if oplog_replay:
             self.__query_flags |= _QUERY_OPTIONS["oplog_replay"]
 
+        # The namespace to use for find/getMore commands.
+        self.__dbname = collection.database.name
+        self.__collname = collection.name
+
     @property
     def collection(self):
         """The :class:`~pymongo.collection.Collection` that this
@@ -278,7 +285,9 @@ class Cursor(object):
                            "max_time_ms", "max_await_time_ms", "comment",
                            "max", "min", "ordering", "explain", "hint",
                            "batch_size", "max_scan", "manipulate",
-                           "query_flags", "modifiers", "collation")
+                           "query_flags", "modifiers", "collation", "empty",
+                           "show_record_id", "return_key", "allow_disk_use",
+                           "snapshot", "exhaust")
         data = dict((k, v) for k, v in iteritems(self.__dict__)
                     if k.startswith('_Cursor__') and k[9:] in values_to_clone)
         if deepcopy:
@@ -294,14 +303,20 @@ class Cursor(object):
     def __die(self, synchronous=False):
         """Closes this cursor.
         """
-        already_killed = self.__killed
+        try:
+            already_killed = self.__killed
+        except AttributeError:
+            # __init__ did not run to completion (or at all).
+            return
+
         self.__killed = True
         if self.__id and not already_killed:
             if self.__exhaust and self.__exhaust_mgr:
                 # If this is an exhaust cursor and we haven't completely
                 # exhausted the result set we *must* close the socket
                 # to stop the server from sending more data.
-                self.__exhaust_mgr.sock.close()
+                self.__exhaust_mgr.sock.close_socket(
+                    ConnectionClosedReason.ERROR)
             else:
                 address = _CursorAddress(
                     self.__address, self.__collection.full_name)
@@ -422,6 +437,28 @@ class Cursor(object):
         self.__query_flags &= ~mask
         return self
 
+    def allow_disk_use(self, allow_disk_use):
+        """Specifies whether MongoDB can use temporary disk files while
+        processing a blocking sort operation.
+
+        Raises :exc:`TypeError` if `allow_disk_use` is not a boolean.
+
+        .. note:: `allow_disk_use` requires server version **>= 4.4**
+
+        :Parameters:
+          - `allow_disk_use`: if True, MongoDB may use temporary
+            disk files to store data exceeding the system memory limit while
+            processing a blocking sort operation.
+
+        .. versionadded:: 3.11
+        """
+        if not isinstance(allow_disk_use, bool):
+            raise TypeError('allow_disk_use must be a bool')
+        self.__check_okay_to_chain()
+
+        self.__allow_disk_use = allow_disk_use
+        return self
+
     def limit(self, limit):
         """Limits the number of results to be returned by this cursor.
 
@@ -634,12 +671,19 @@ class Cursor(object):
         return self
 
     def max(self, spec):
-        """Adds `max` operator that specifies upper bound for specific index.
+        """Adds ``max`` operator that specifies upper bound for specific index.
+
+        When using ``max``, :meth:`~hint` should also be configured to ensure
+        the query uses the expected index and starting in MongoDB 4.2
+        :meth:`~hint` will be required.
 
         :Parameters:
           - `spec`: a list of field, limit pairs specifying the exclusive
             upper bound for all keys of a specific index in order.
 
+        .. versionchanged:: 3.8
+           Deprecated cursors that use ``max`` without a :meth:`~hint`.
+
         .. versionadded:: 2.7
         """
         if not isinstance(spec, (list, tuple)):
@@ -650,12 +694,19 @@ class Cursor(object):
         return self
 
     def min(self, spec):
-        """Adds `min` operator that specifies lower bound for specific index.
+        """Adds ``min`` operator that specifies lower bound for specific index.
+
+        When using ``min``, :meth:`~hint` should also be configured to ensure
+        the query uses the expected index and starting in MongoDB 4.2
+        :meth:`~hint` will be required.
 
         :Parameters:
           - `spec`: a list of field, limit pairs specifying the inclusive
             lower bound for all keys of a specific index in order.
 
+        .. versionchanged:: 3.8
+           Deprecated cursors that use ``min`` without a :meth:`~hint`.
+
         .. versionadded:: 2.7
         """
         if not isinstance(spec, (list, tuple)):
@@ -694,6 +745,9 @@ class Cursor(object):
             for doc in cursor:
                 print(doc)
 
+        For more advanced text search functionality, see MongoDB's
+        `Atlas Search <https://docs.atlas.mongodb.com/atlas-search/>`_.
+
         Raises :class:`~pymongo.errors.InvalidOperation` if this cursor has
         already been used. Only the last :meth:`sort` applied to this
         cursor has any effect.
@@ -754,7 +808,7 @@ class Cursor(object):
         if self.__max_time_ms is not None:
             cmd["maxTimeMS"] = self.__max_time_ms
         if self.__comment:
-            cmd["$comment"] = self.__comment
+            cmd["comment"] = self.__comment
 
         if self.__hint is not None:
             cmd["hint"] = self.__hint
@@ -791,7 +845,7 @@ class Cursor(object):
         if self.__max_time_ms is not None:
             options['maxTimeMS'] = self.__max_time_ms
         if self.__comment:
-            options['$comment'] = self.__comment
+            options['comment'] = self.__comment
         if self.__collation is not None:
             options['collation'] = self.__collation
 
@@ -801,6 +855,13 @@ class Cursor(object):
     def explain(self):
         """Returns an explain plan record for this cursor.
 
+        .. note:: Starting with MongoDB 3.2 :meth:`explain` uses
+          the default verbosity mode of the `explain command
+          <https://docs.mongodb.com/manual/reference/command/explain/>`_,
+          ``allPlansExecution``. To use a different verbosity use
+          :meth:`~pymongo.database.Database.command` to run the explain
+          command directly.
+
         .. mongodoc:: explain
         """
         c = self.clone()
@@ -827,9 +888,9 @@ class Cursor(object):
         Judicious use of hints can greatly improve query
         performance. When doing a query on multiple fields (at least
         one of which is indexed) pass the indexed field as a hint to
-        the query. Hinting will not do anything if the corresponding
-        index does not exist. Raises
-        :class:`~pymongo.errors.InvalidOperation` if this cursor has
+        the query. Raises :class:`~pymongo.errors.OperationFailure` if the
+        provided hint requires an index that does not exist on this collection,
+        and raises :class:`~pymongo.errors.InvalidOperation` if this cursor has
         already been used.
 
         `index` should be an index as passed to
@@ -855,7 +916,8 @@ class Cursor(object):
         http://docs.mongodb.org/manual/reference/operator/comment/
 
         :Parameters:
-          - `comment`: A string or document
+          - `comment`: A string to attach to the query to help interpret and
+            trace the operation in the server logs and in profile data.
 
         .. versionadded:: 2.7
         """
@@ -864,7 +926,7 @@ class Cursor(object):
         return self
 
     def where(self, code):
-        """Adds a $where clause to this query.
+        """Adds a `$where`_ clause to this query.
 
         The `code` argument must be an instance of :class:`basestring`
         (:class:`str` in python 3) or :class:`~bson.code.Code`
@@ -872,7 +934,11 @@ class Cursor(object):
         evaluated for each document scanned. Only those documents
         for which the expression evaluates to *true* will be returned
         as results. The keyword *this* refers to the object currently
-        being scanned.
+        being scanned. For example::
+
+            # Find all documents where field "a" is less than "b" plus "c".
+            for doc in db.test.find().where('this.a < (this.b + this.c)'):
+                print(doc)
 
         Raises :class:`TypeError` if `code` is not an instance of
         :class:`basestring` (:class:`str` in python 3). Raises
@@ -880,8 +946,14 @@ class Cursor(object):
         :class:`Cursor` has already been used. Only the last call to
         :meth:`where` applied to a :class:`Cursor` has any effect.
 
+        .. note:: MongoDB 4.4 drops support for :class:`~bson.code.Code`
+          with scope variables. Consider using `$expr`_ instead.
+
         :Parameters:
           - `code`: JavaScript expression to use as a filter
+
+        .. _$expr: https://docs.mongodb.com/manual/reference/operator/query/expr/
+        .. _$where: https://docs.mongodb.com/manual/reference/operator/query/where/
         """
         self.__check_okay_to_chain()
         if not isinstance(code, Code):
@@ -918,78 +990,21 @@ class Cursor(object):
         Can raise ConnectionFailure.
         """
         client = self.__collection.database.client
-        listeners = client._event_listeners
-        publish = listeners.enabled_for_commands
-        from_command = False
-        start = datetime.datetime.now()
-
-        def duration(): return datetime.datetime.now() - start
-
-        if operation:
-            try:
-                response = client._send_message_with_response(
-                    operation, exhaust=self.__exhaust, address=self.__address)
-                self.__address = response.address
-                if self.__exhaust:
-                    # 'response' is an ExhaustResponse.
-                    self.__exhaust_mgr = _SocketManager(response.socket_info,
-                                                        response.pool)
-
-                cmd_name = operation.name
-                reply = response.data
-                rqst_id = response.request_id
-                from_command = response.from_command
-            except AutoReconnect:
-                # Don't try to send kill cursors on another socket
-                # or to another server. It can cause a _pinValue
-                # assertion on some server releases if we get here
-                # due to a socket timeout.
-                self.__killed = True
-                self.__die()
-                raise
-        else:
-            # Exhaust cursor - no getMore message.
-            rqst_id = 0
-            cmd_name = 'getMore'
-            if publish:
-                # Fake a getMore command.
-                cmd = SON([('getMore', self.__id),
-                           ('collection', self.__collection.name)])
-                if self.__batch_size:
-                    cmd['batchSize'] = self.__batch_size
-                if self.__max_time_ms:
-                    cmd['maxTimeMS'] = self.__max_time_ms
-                listeners.publish_command_start(
-                    cmd, self.__collection.database.name, 0, self.__address)
-            try:
-                reply = self.__exhaust_mgr.sock.receive_message(None)
-            except Exception as exc:
-                if publish:
-                    listeners.publish_command_failure(
-                        duration(), _convert_exception(exc), cmd_name, rqst_id,
-                        self.__address)
-                if isinstance(exc, ConnectionFailure):
-                    self.__die()
-                raise
+        # OP_MSG is required to support exhaust cursors with encryption.
+        if client._encrypter and self.__exhaust:
+            raise InvalidOperation(
+                "exhaust cursors do not support auto encryption")
 
         try:
-            docs = self._unpack_response(response=reply,
-                                         cursor_id=self.__id,
-                                         codec_options=self.__codec_options)
-            if from_command:
-                first = docs[0]
-                client._receive_cluster_time(first, self.__session)
-                helpers._check_command_response(first)
-        except OperationFailure as exc:
+            response = client._run_operation_with_response(
+                operation, self._unpack_response, exhaust=self.__exhaust,
+                address=self.__address)
+        except OperationFailure:
             self.__killed = True
 
             # Make sure exhaust socket is returned immediately, if necessary.
             self.__die()
 
-            if publish:
-                listeners.publish_command_failure(
-                    duration(), exc.details, cmd_name, rqst_id, self.__address)
-
             # If this is a tailable cursor the error is likely
             # due to capped collection roll over. Setting
             # self.__killed to True ensures Cursor.alive will be
@@ -997,7 +1012,7 @@ class Cursor(object):
             if self.__query_flags & _QUERY_OPTIONS["tailable_cursor"]:
                 return
             raise
-        except NotMasterError as exc:
+        except NotMasterError:
             # Don't send kill cursors to another server after a "not master"
             # error. It's completely pointless.
             self.__killed = True
@@ -1005,42 +1020,38 @@ class Cursor(object):
             # Make sure exhaust socket is returned immediately, if necessary.
             self.__die()
 
-            if publish:
-                listeners.publish_command_failure(
-                    duration(), exc.details, cmd_name, rqst_id, self.__address)
-
-            client._reset_server_and_request_check(self.__address)
             raise
-        except Exception as exc:
-            if publish:
-                listeners.publish_command_failure(
-                    duration(), _convert_exception(exc), cmd_name, rqst_id,
-                    self.__address)
+        except ConnectionFailure:
+            # Don't try to send kill cursors on another socket
+            # or to another server. It can cause a _pinValue
+            # assertion on some server releases if we get here
+            # due to a socket timeout.
+            self.__killed = True
+            self.__die()
+            raise
+        except Exception:
+            # Close the cursor
+            self.__die()
             raise
 
-        if publish:
-            # Must publish in find / getMore / explain command response format.
-            if from_command:
-                res = docs[0]
-            elif cmd_name == "explain":
-                res = docs[0] if docs else {}
-            else:
-                res = {"cursor": {"id": reply.cursor_id,
-                                  "ns": self.__collection.full_name},
-                       "ok": 1}
-                if cmd_name == "find":
-                    res["cursor"]["firstBatch"] = docs
-                else:
-                    res["cursor"]["nextBatch"] = docs
-            listeners.publish_command_success(
-                duration(), res, cmd_name, rqst_id, self.__address)
+        self.__address = response.address
+        if self.__exhaust and not self.__exhaust_mgr:
+            # 'response' is an ExhaustResponse.
+            self.__exhaust_mgr = _SocketManager(response.socket_info,
+                                                response.pool)
 
-        if from_command:
+        cmd_name = operation.name
+        docs = response.docs
+        if response.from_command:
             if cmd_name != "explain":
                 cursor = docs[0]['cursor']
                 self.__id = cursor['id']
                 if cmd_name == 'find':
                     documents = cursor['firstBatch']
+                    # Update the namespace used for future getMore commands.
+                    ns = cursor.get('ns')
+                    if ns:
+                        self.__dbname, self.__collname = ns.split('.', 1)
                 else:
                     documents = cursor['nextBatch']
                 self.__data = deque(documents)
@@ -1050,9 +1061,9 @@ class Cursor(object):
                 self.__data = deque(docs)
                 self.__retrieved += len(docs)
         else:
-            self.__id = reply.cursor_id
+            self.__id = response.data.cursor_id
             self.__data = deque(docs)
-            self.__retrieved += reply.number_returned
+            self.__retrieved += response.data.number_returned
 
         if self.__id == 0:
             self.__killed = True
@@ -1063,8 +1074,10 @@ class Cursor(object):
         if self.__limit and self.__id and self.__limit <= self.__retrieved:
             self.__die()
 
-    def _unpack_response(self, response, cursor_id, codec_options):
-        return response.unpack_response(cursor_id, codec_options)
+    def _unpack_response(self, response, cursor_id, codec_options,
+                         user_fields=None, legacy_response=False):
+        return response.unpack_response(cursor_id, codec_options, user_fields,
+                                        legacy_response)
 
     def _read_preference(self):
         if self.__read_preference is None:
@@ -1087,6 +1100,12 @@ class Cursor(object):
             self.__session = self.__collection.database.client._ensure_session()
 
         if self.__id is None:  # Query
+            if (self.__min or self.__max) and not self.__hint:
+                warnings.warn("using a min/max query operator without "
+                              "specifying a Cursor.hint is deprecated. A "
+                              "hint will be required when using min/max in "
+                              "PyMongo 4.0",
+                              DeprecationWarning, stacklevel=3)
             q = self._query_class(self.__query_flags,
                                   self.__collection.database.name,
                                   self.__collection.name,
@@ -1100,7 +1119,8 @@ class Cursor(object):
                                   self.__read_concern,
                                   self.__collation,
                                   self.__session,
-                                  self.__collection.database.client)
+                                  self.__collection.database.client,
+                                  self.__allow_disk_use)
             self.__send_message(q)
         elif self.__id:  # Get More
             if self.__limit:
@@ -1111,19 +1131,17 @@ class Cursor(object):
                 limit = self.__batch_size
 
             # Exhaust cursors don't send getMore messages.
-            if self.__exhaust:
-                self.__send_message(None)
-            else:
-                g = self._getmore_class(self.__collection.database.name,
-                                        self.__collection.name,
-                                        limit,
-                                        self.__id,
-                                        self.__codec_options,
-                                        self._read_preference(),
-                                        self.__session,
-                                        self.__collection.database.client,
-                                        self.__max_await_time_ms)
-                self.__send_message(g)
+            g = self._getmore_class(self.__dbname,
+                                    self.__collname,
+                                    limit,
+                                    self.__id,
+                                    self.__codec_options,
+                                    self._read_preference(),
+                                    self.__session,
+                                    self.__collection.database.client,
+                                    self.__max_await_time_ms,
+                                    self.__exhaust_mgr)
+            self.__send_message(g)
 
         return len(self.__data)
 
@@ -1275,7 +1293,8 @@ class RawBatchCursor(Cursor):
             raise InvalidOperation(
                 "Cannot use RawBatchCursor with manipulate=True")
 
-    def _unpack_response(self, response, cursor_id, codec_options):
+    def _unpack_response(self, response, cursor_id, codec_options,
+                         user_fields=None, legacy_response=False):
         return response.raw_response(cursor_id)
 
     def explain(self):
