@@ -14,16 +14,52 @@
 
 """Exceptions raised by PyMongo."""
 
+import sys
+
 from bson.errors import *
 
 try:
-    from ssl import CertificateError
+    # CPython 3.7+
+    from ssl import SSLCertVerificationError as CertificateError
 except ImportError:
-    from pymongo.ssl_match_hostname import CertificateError
-
+    try:
+        from ssl import CertificateError
+    except ImportError:
+        class CertificateError(ValueError):
+            pass
 
 class PyMongoError(Exception):
     """Base class for all PyMongo exceptions."""
+    def __init__(self, message='', error_labels=None):
+        super(PyMongoError, self).__init__(message)
+        self._message = message
+        self._error_labels = set(error_labels or [])
+
+    def has_error_label(self, label):
+        """Return True if this error contains the given label.
+
+        .. versionadded:: 3.7
+        """
+        return label in self._error_labels
+
+    def _add_error_label(self, label):
+        """Add the given label to this error."""
+        self._error_labels.add(label)
+
+    def _remove_error_label(self, label):
+        """Remove the given label from this error."""
+        self._error_labels.discard(label)
+
+    if sys.version_info[0] == 2:
+        def __str__(self):
+            if isinstance(self._message, unicode):
+                return self._message.encode('utf-8', errors='replace')
+            return str(self._message)
+
+        def __unicode__(self):
+            if isinstance(self._message, unicode):
+                return self._message
+            return unicode(self._message, 'utf-8', errors='replace')
 
 
 class ProtocolError(PyMongoError):
@@ -47,8 +83,11 @@ class AutoReconnect(ConnectionFailure):
     Subclass of :exc:`~pymongo.errors.ConnectionFailure`.
     """
     def __init__(self, message='', errors=None):
+        error_labels = None
+        if errors is not None and isinstance(errors, dict):
+            error_labels = errors.get('errorLabels')
+        super(AutoReconnect, self).__init__(message, error_labels)
         self.errors = self.details = errors or []
-        ConnectionFailure.__init__(self, message)
 
 
 class NetworkTimeout(AutoReconnect):
@@ -61,6 +100,14 @@ class NetworkTimeout(AutoReconnect):
     """
 
 
+def _format_detailed_error(message, details):
+    if details is not None:
+        message = "%s, full error: %s" % (message, details)
+        if sys.version_info[0] == 2 and isinstance(message, unicode):
+            message = message.encode('utf-8', errors='replace')
+    return message
+
+
 class NotMasterError(AutoReconnect):
     """The server responded "not master" or "node is recovering".
 
@@ -74,6 +121,9 @@ class NotMasterError(AutoReconnect):
 
     Subclass of :exc:`~pymongo.errors.AutoReconnect`.
     """
+    def __init__(self, message='', errors=None):
+        super(NotMasterError, self).__init__(
+            _format_detailed_error(message, errors), errors=errors)
 
 
 class ServerSelectionTimeoutError(AutoReconnect):
@@ -101,10 +151,19 @@ class OperationFailure(PyMongoError):
        The :attr:`details` attribute.
     """
 
-    def __init__(self, error, code=None, details=None):
+    def __init__(self, error, code=None, details=None, max_wire_version=None):
+        error_labels = None
+        if details is not None:
+            error_labels = details.get('errorLabels')
+        super(OperationFailure, self).__init__(
+            _format_detailed_error(error, details), error_labels=error_labels)
         self.__code = code
         self.__details = details
-        PyMongoError.__init__(self, error)
+        self.__max_wire_version = max_wire_version
+
+    @property
+    def _max_wire_version(self):
+        return self.__max_wire_version
 
     @property
     def code(self):
@@ -125,6 +184,7 @@ class OperationFailure(PyMongoError):
         return self.__details
 
 
+
 class CursorNotFound(OperationFailure):
     """Raised while iterating query results if the cursor is
     invalidated on the server.
@@ -178,8 +238,8 @@ class BulkWriteError(OperationFailure):
     .. versionadded:: 2.7
     """
     def __init__(self, results):
-        OperationFailure.__init__(
-            self, "batch op errors occurred", 65, results)
+        super(BulkWriteError, self).__init__(
+            "batch op errors occurred", 65, results)
 
 
 class InvalidOperation(PyMongoError):
@@ -198,7 +258,7 @@ class InvalidURI(ConfigurationError):
     """Raised when trying to parse an invalid mongodb URI."""
 
 
-class ExceededMaxWaiters(Exception):
+class ExceededMaxWaiters(PyMongoError):
     """Raised when a thread tries to get a connection from a pool and
     ``maxPoolSize * waitQueueMultiple`` threads are already waiting.
 
@@ -211,3 +271,28 @@ class DocumentTooLarge(InvalidDocument):
     """Raised when an encoded document is too large for the connected server.
     """
     pass
+
+
+class EncryptionError(PyMongoError):
+    """Raised when encryption or decryption fails.
+
+    This error always wraps another exception which can be retrieved via the
+    :attr:`cause` property.
+
+    .. versionadded:: 3.9
+    """
+
+    def __init__(self, cause):
+        super(EncryptionError, self).__init__(str(cause))
+        self.__cause = cause
+
+    @property
+    def cause(self):
+        """The exception that caused this encryption or decryption error."""
+        return self.__cause
+
+
+class _OperationCancelled(AutoReconnect):
+    """Internal error raised when a socket operation is cancelled.
+    """
+    pass
