@@ -16,6 +16,10 @@
 
 .. versionadded:: 3.1
 
+.. attention:: Starting in PyMongo 3.11, the monitoring classes outlined below
+    are included in the PyMongo distribution under the
+    :mod:`~pymongo.event_loggers` submodule.
+
 Use :func:`register` to register global listeners for specific events.
 Listeners must inherit from one of the abstract classes below and implement
 the correct functions for that class.
@@ -561,6 +565,11 @@ class CommandStartedEvent(_CommandEvent):
         """The name of the database this command was run against."""
         return self.__db
 
+    def __repr__(self):
+        return "<%s %s db: %r, command: %r, operation_id: %s>" % (
+            self.__class__.__name__, self.connection_id, self.database_name,
+            self.command_name, self.operation_id)
+
 
 class CommandSucceededEvent(_CommandEvent):
     """Event published when a command succeeds.
@@ -596,6 +605,11 @@ class CommandSucceededEvent(_CommandEvent):
         """The server failure document for this operation."""
         return self.__reply
 
+    def __repr__(self):
+        return "<%s %s command: %r, operation_id: %s, duration_micros: %s>" % (
+            self.__class__.__name__, self.connection_id,
+            self.command_name, self.operation_id, self.duration_micros)
+
 
 class CommandFailedEvent(_CommandEvent):
     """Event published when a command fails.
@@ -626,6 +640,13 @@ class CommandFailedEvent(_CommandEvent):
         """The server failure document for this operation."""
         return self.__failure
 
+    def __repr__(self):
+        return (
+            "<%s %s command: %r, operation_id: %s, duration_micros: %s, "
+            "failure: %r>" % (
+                self.__class__.__name__, self.connection_id, self.command_name,
+                self.operation_id, self.duration_micros, self.failure))
+
 
 class _PoolEvent(object):
     """Base class for pool events."""
@@ -928,6 +949,10 @@ class _ServerEvent(object):
         """A unique identifier for the topology this server is a part of."""
         return self.__topology_id
 
+    def __repr__(self):
+        return "<%s %s topology_id: %s>" % (
+            self.__class__.__name__, self.server_address, self.topology_id)
+
 
 class ServerDescriptionChangedEvent(_ServerEvent):
     """Published when server description changes.
@@ -954,6 +979,11 @@ class ServerDescriptionChangedEvent(_ServerEvent):
         :class:`~pymongo.server_description.ServerDescription`."""
         return self.__new_description
 
+    def __repr__(self):
+        return "<%s %s changed from: %s, to: %s>" % (
+            self.__class__.__name__, self.server_address,
+            self.previous_description, self.new_description)
+
 
 class ServerOpeningEvent(_ServerEvent):
     """Published when server is initialized.
@@ -986,6 +1016,10 @@ class TopologyEvent(object):
         """A unique identifier for the topology this server is a part of."""
         return self.__topology_id
 
+    def __repr__(self):
+        return "<%s topology_id: %s>" % (
+            self.__class__.__name__, self.topology_id)
+
 
 class TopologyDescriptionChangedEvent(TopologyEvent):
     """Published when the topology description changes.
@@ -1012,6 +1046,11 @@ class TopologyDescriptionChangedEvent(TopologyEvent):
         :class:`~pymongo.topology_description.TopologyDescription`."""
         return self.__new_description
 
+    def __repr__(self):
+        return "<%s topology_id: %s changed from: %s, to: %s>" % (
+            self.__class__.__name__, self.topology_id,
+            self.previous_description, self.new_description)
+
 
 class TopologyOpenedEvent(TopologyEvent):
     """Published when the topology is initialized.
@@ -1045,6 +1084,9 @@ class _ServerHeartbeatEvent(object):
         to."""
         return self.__connection_id
 
+    def __repr__(self):
+        return "<%s %s>" % (self.__class__.__name__, self.connection_id)
+
 
 class ServerHeartbeatStartedEvent(_ServerHeartbeatEvent):
     """Published when a heartbeat is started.
@@ -1061,12 +1103,13 @@ class ServerHeartbeatSucceededEvent(_ServerHeartbeatEvent):
     .. versionadded:: 3.3
     """
 
-    __slots__ = ('__duration', '__reply')
+    __slots__ = ('__duration', '__reply', '__awaited')
 
-    def __init__(self, duration, reply, *args):
-        super(ServerHeartbeatSucceededEvent, self).__init__(*args)
+    def __init__(self, duration, reply, connection_id, awaited=False):
+        super(ServerHeartbeatSucceededEvent, self).__init__(connection_id)
         self.__duration = duration
         self.__reply = reply
+        self.__awaited = awaited
 
     @property
     def duration(self):
@@ -1078,6 +1121,21 @@ class ServerHeartbeatSucceededEvent(_ServerHeartbeatEvent):
         """An instance of :class:`~pymongo.ismaster.IsMaster`."""
         return self.__reply
 
+    @property
+    def awaited(self):
+        """Whether the heartbeat was awaited.
+
+        If true, then :meth:`duration` reflects the sum of the round trip time
+        to the server and the time that the server waited before sending a
+        response.
+        """
+        return self.__awaited
+
+    def __repr__(self):
+        return "<%s %s duration: %s, awaited: %s, reply: %s>" % (
+            self.__class__.__name__, self.connection_id,
+            self.duration, self.awaited, self.reply)
+
 
 class ServerHeartbeatFailedEvent(_ServerHeartbeatEvent):
     """Fired when the server heartbeat fails, either with an "ok: 0"
@@ -1086,12 +1144,13 @@ class ServerHeartbeatFailedEvent(_ServerHeartbeatEvent):
     .. versionadded:: 3.3
     """
 
-    __slots__ = ('__duration', '__reply')
+    __slots__ = ('__duration', '__reply', '__awaited')
 
-    def __init__(self, duration, reply, *args):
-        super(ServerHeartbeatFailedEvent, self).__init__(*args)
+    def __init__(self, duration, reply, connection_id, awaited=False):
+        super(ServerHeartbeatFailedEvent, self).__init__(connection_id)
         self.__duration = duration
         self.__reply = reply
+        self.__awaited = awaited
 
     @property
     def duration(self):
@@ -1103,6 +1162,21 @@ class ServerHeartbeatFailedEvent(_ServerHeartbeatEvent):
         """A subclass of :exc:`Exception`."""
         return self.__reply
 
+    @property
+    def awaited(self):
+        """Whether the heartbeat was awaited.
+
+        If true, then :meth:`duration` reflects the sum of the round trip time
+        to the server and the time that the server waited before sending a
+        response.
+        """
+        return self.__awaited
+
+    def __repr__(self):
+        return "<%s %s duration: %s, awaited: %s, reply: %r>" % (
+            self.__class__.__name__, self.connection_id,
+            self.duration, self.awaited, self.reply)
+
 
 class _EventListeners(object):
     """Configure event listeners for a client instance.
@@ -1255,7 +1329,7 @@ class _EventListeners(object):
                 _handle_exception()
 
     def publish_server_heartbeat_succeeded(self, connection_id, duration,
-                                           reply):
+                                           reply, awaited):
         """Publish a ServerHeartbeatSucceededEvent to all server heartbeat
         listeners.
 
@@ -1264,15 +1338,18 @@ class _EventListeners(object):
          - `duration`: The execution time of the event in the highest possible
             resolution for the platform.
          - `reply`: The command reply.
+         - `awaited`: True if the response was awaited.
          """
-        event = ServerHeartbeatSucceededEvent(duration, reply, connection_id)
+        event = ServerHeartbeatSucceededEvent(duration, reply, connection_id,
+                                              awaited)
         for subscriber in self.__server_heartbeat_listeners:
             try:
                 subscriber.succeeded(event)
             except Exception:
                 _handle_exception()
 
-    def publish_server_heartbeat_failed(self, connection_id, duration, reply):
+    def publish_server_heartbeat_failed(self, connection_id, duration, reply,
+                                        awaited):
         """Publish a ServerHeartbeatFailedEvent to all server heartbeat
         listeners.
 
@@ -1281,8 +1358,10 @@ class _EventListeners(object):
          - `duration`: The execution time of the event in the highest possible
             resolution for the platform.
          - `reply`: The command reply.
+         - `awaited`: True if the response was awaited.
          """
-        event = ServerHeartbeatFailedEvent(duration, reply, connection_id)
+        event = ServerHeartbeatFailedEvent(duration, reply, connection_id,
+                                           awaited)
         for subscriber in self.__server_heartbeat_listeners:
             try:
                 subscriber.failed(event)
