@@ -40,31 +40,29 @@ class Semaphore:
             raise ValueError("can't specify timeout for non-blocking acquire")
         rc = False
         endtime = None
-        self._cond.acquire()
-        while self._value == 0:
-            if not blocking:
-                break
-            if timeout is not None:
-                if endtime is None:
-                    endtime = _time() + timeout
-                else:
-                    timeout = endtime - _time()
-                    if timeout <= 0:
-                        break
-            self._cond.wait(timeout)
-        else:
-            self._value = self._value - 1
-            rc = True
-        self._cond.release()
+        with self._cond:
+            while self._value == 0:
+                if not blocking:
+                    break
+                if timeout is not None:
+                    if endtime is None:
+                        endtime = _time() + timeout
+                    else:
+                        timeout = endtime - _time()
+                        if timeout <= 0:
+                            break
+                self._cond.wait(timeout)
+            else:
+                self._value = self._value - 1
+                rc = True
         return rc
 
     __enter__ = acquire
 
     def release(self):
-        self._cond.acquire()
-        self._value = self._value + 1
-        self._cond.notify()
-        self._cond.release()
+        with self._cond:
+            self._value = self._value + 1
+            self._cond.notify()
 
     def __exit__(self, t, v, tb):
         self.release()
@@ -129,74 +127,3 @@ def create_semaphore(max_size, max_waiters):
             return BoundedSemaphore(max_size)
         else:
             return MaxWaitersBoundedSemaphoreThread(max_size, max_waiters)
-
-
-class Event(object):
-    """Copy of standard threading.Event, but uses a custom condition class.
-
-    Allows async frameworks to override monitors' synchronization behavior
-    with TopologySettings.condition_class.
-
-    Copied from CPython's threading.py at hash c7960cc9.
-    """
-
-    def __init__(self, condition_class):
-        self._cond = condition_class(threading.Lock())
-        self._flag = False
-
-    def is_set(self):
-        """Return true if and only if the internal flag is true."""
-        return self._flag
-
-    isSet = is_set
-
-    def set(self):
-        """Set the internal flag to true.
-
-        All threads waiting for it to become true are awakened. Threads
-        that call wait() once the flag is true will not block at all.
-
-        """
-        self._cond.acquire()
-        try:
-            self._flag = True
-            self._cond.notify_all()
-        finally:
-            self._cond.release()
-
-    def clear(self):
-        """Reset the internal flag to false.
-
-        Subsequently, threads calling wait() will block until set() is called to
-        set the internal flag to true again.
-
-        """
-        self._cond.acquire()
-        try:
-            self._flag = False
-        finally:
-            self._cond.release()
-
-    def wait(self, timeout=None):
-        """Block until the internal flag is true.
-
-        If the internal flag is true on entry, return immediately. Otherwise,
-        block until another thread calls set() to set the flag to true, or until
-        the optional timeout occurs.
-
-        When the timeout argument is present and not None, it should be a
-        floating point number specifying a timeout for the operation in seconds
-        (or fractions thereof).
-
-        This method returns the internal flag on exit, so it will always return
-        True except if a timeout is given and the operation times out.
-
-        """
-        self._cond.acquire()
-        try:
-            signaled = self._flag
-            if not signaled:
-                signaled = self._cond.wait(timeout)
-            return signaled
-        finally:
-            self._cond.release()
