@@ -15,6 +15,7 @@
 """Test suite for pymongo, bson, and gridfs.
 """
 
+import gc
 import os
 import socket
 import sys
@@ -23,6 +24,13 @@ import time
 import unittest
 import warnings
 
+try:
+    from xmlrunner import XMLTestRunner
+    HAVE_XML = True
+# ValueError is raised when version 3+ is installed on Jython 2.7.
+except (ImportError, ValueError):
+    HAVE_XML = False
+
 try:
     import ipaddress
     HAVE_IPADDRESS = True
@@ -53,6 +61,13 @@ try:
 except ImportError:
     pass
 
+# Enable debug output for uncollectable objects. PyPy does not have set_debug.
+if hasattr(gc, 'set_debug'):
+    gc.set_debug(
+        gc.DEBUG_UNCOLLECTABLE |
+        getattr(gc, 'DEBUG_OBJECTS', 0) |
+        getattr(gc, 'DEBUG_INSTANCES', 0))
+
 # The host and port of a single mongod or mongos, or the seed host
 # for a replica set.
 host = os.environ.get("DB_IP", 'localhost')
@@ -66,15 +81,12 @@ CERT_PATH = os.path.join(os.path.dirname(os.path.realpath(__file__)),
 CLIENT_PEM = os.environ.get('CLIENT_PEM',
                             os.path.join(CERT_PATH, 'client.pem'))
 CA_PEM = os.environ.get('CA_PEM', os.path.join(CERT_PATH, 'ca.pem'))
-CERT_REQS = validate_cert_reqs('CERT_REQS', os.environ.get('CERT_REQS'))
 
-_SSL_OPTIONS = dict(ssl=True)
+TLS_OPTIONS = dict(tls=True)
 if CLIENT_PEM:
-    _SSL_OPTIONS['ssl_certfile'] = CLIENT_PEM
+    TLS_OPTIONS['tlsCertificateKeyFile'] = CLIENT_PEM
 if CA_PEM:
-    _SSL_OPTIONS['ssl_ca_certs'] = CA_PEM
-if CERT_REQS is not None:
-    _SSL_OPTIONS['ssl_cert_reqs'] = CERT_REQS
+    TLS_OPTIONS['tlsCAFile'] = CA_PEM
 
 COMPRESSORS = os.environ.get("COMPRESSORS")
 
@@ -172,8 +184,7 @@ class ClientContext(object):
         self.mongoses = []
         self.is_rs = False
         self.has_ipv6 = False
-        self.ssl = False
-        self.ssl_cert_none = False
+        self.tls = False
         self.ssl_certfile = False
         self.server_is_resolvable = is_server_resolvable()
         self.default_client_options = {}
@@ -215,18 +226,18 @@ class ClientContext(object):
             self.connection_attempts.append(
                 'failed to connect client %r: %s' % (client, exc))
             return None
+        finally:
+            client.close()
 
     def _init_client(self):
         self.client = self._connect(host, port)
         if HAVE_SSL and not self.client:
             # Is MongoDB configured for SSL?
-            self.client = self._connect(host, port, **_SSL_OPTIONS)
+            self.client = self._connect(host, port, **TLS_OPTIONS)
             if self.client:
-                self.ssl = True
-                self.default_client_options.update(_SSL_OPTIONS)
+                self.tls = True
+                self.default_client_options.update(TLS_OPTIONS)
                 self.ssl_certfile = True
-                if _SSL_OPTIONS.get('ssl_cert_reqs') == ssl.CERT_NONE:
-                    self.ssl_cert_none = True
 
         if self.client:
             self.connected = True
@@ -593,22 +604,24 @@ class ClientContext(object):
                              "failCommand fail point must be supported",
                              func=func)
 
-    def require_ssl(self, func):
-        """Run a test only if the client can connect over SSL."""
-        return self._require(lambda: self.ssl,
-                             "Must be able to connect via SSL",
+    def require_failCommand_appName(self, func):
+        """Run a test only if the server supports the failCommand appName."""
+        # SERVER-47195
+        return self._require(lambda: (self.test_commands_enabled and
+                                      self.version >= (4, 4, -1)),
+                             "failCommand appName must be supported",
                              func=func)
 
-    def require_no_ssl(self, func):
-        """Run a test only if the client can connect over SSL."""
-        return self._require(lambda: not self.ssl,
-                             "Must be able to connect without SSL",
+    def require_tls(self, func):
+        """Run a test only if the client can connect over TLS."""
+        return self._require(lambda: self.tls,
+                             "Must be able to connect via TLS",
                              func=func)
 
-    def require_ssl_cert_none(self, func):
-        """Run a test only if the client can connect with ssl.CERT_NONE."""
-        return self._require(lambda: self.ssl_cert_none,
-                             "Must be able to connect with ssl.CERT_NONE",
+    def require_no_tls(self, func):
+        """Run a test only if the client can connect over TLS."""
+        return self._require(lambda: not self.tls,
+                             "Must be able to connect without TLS",
                              func=func)
 
     def require_ssl_certfile(self, func):
@@ -657,7 +670,9 @@ class ClientContext(object):
     @property
     def supports_reindex(self):
         """Does the connected server support reindex?"""
-        return not (self.version.at_least(4, 1, 0) and self.is_mongos)
+        return not ((self.version.at_least(4, 1, 0) and self.is_mongos) or
+                    (self.version.at_least(4, 5, 0) and (
+                            self.is_mongos or self.is_rs)))
 
     @property
     def supports_getpreverror(self):
@@ -718,6 +733,17 @@ class PyMongoTestCase(unittest.TestCase):
     def assertEqualReply(self, expected, actual, msg=None):
         self.assertEqual(sanitize_reply(expected), sanitize_reply(actual), msg)
 
+    @contextmanager
+    def fail_point(self, command_args):
+        cmd_on = SON([('configureFailPoint', 'failCommand')])
+        cmd_on.update(command_args)
+        client_context.client.admin.command(cmd_on)
+        try:
+            yield
+        finally:
+            client_context.client.admin.command(
+                'configureFailPoint', cmd_on['configureFailPoint'], mode='off')
+
 
 class IntegrationTest(PyMongoTestCase):
     """Base class for TestCases that need a connection to MongoDB to pass."""
@@ -732,16 +758,6 @@ class IntegrationTest(PyMongoTestCase):
         else:
             cls.credentials = {}
 
-    @contextmanager
-    def fail_point(self, command_args):
-        cmd_on = SON([('configureFailPoint', 'failCommand')])
-        cmd_on.update(command_args)
-        self.client.admin.command(cmd_on)
-        try:
-            yield
-        finally:
-            self.client.admin.command(
-                'configureFailPoint', cmd_on['configureFailPoint'], mode='off')
 
 # Use assertRaisesRegex if available, otherwise use Python 2.7's
 # deprecated assertRaisesRegexp, with a 'p'.
@@ -778,23 +794,85 @@ def setup():
     warnings.simplefilter("always")
 
 
+def _get_executors(topology):
+    executors = []
+    for server in topology._servers.values():
+        # Some MockMonitor do not have an _executor.
+        if hasattr(server._monitor, '_executor'):
+            executors.append(server._monitor._executor)
+        if hasattr(server._monitor, '_rtt_monitor'):
+            executors.append(server._monitor._rtt_monitor._executor)
+    executors.append(topology._Topology__events_executor)
+    if topology._srv_monitor:
+        executors.append(topology._srv_monitor._executor)
+
+    return [e for e in executors if e is not None]
+
+
+def all_executors_stopped(topology):
+    running = [e for e in _get_executors(topology) if not e._stopped]
+    if running:
+        print('  Topology %s has THREADS RUNNING: %s, created at: %s' % (
+            topology, running, topology._settings._stack))
+        return False
+    return True
+
+
+def print_unclosed_clients():
+    from pymongo.topology import Topology
+    processed = set()
+    # Call collect to manually cleanup any would-be gc'd clients to avoid
+    # false positives.
+    gc.collect()
+    for obj in gc.get_objects():
+        try:
+            if isinstance(obj, Topology):
+                # Avoid printing the same Topology multiple times.
+                if obj._topology_id in processed:
+                    continue
+                all_executors_stopped(obj)
+                processed.add(obj._topology_id)
+        except ReferenceError:
+            pass
+
+
 def teardown():
+    garbage = []
+    for g in gc.garbage:
+        garbage.append('GARBAGE: %r' % (g,))
+        garbage.append('  gc.get_referents: %r' % (gc.get_referents(g),))
+        garbage.append('  gc.get_referrers: %r' % (gc.get_referrers(g),))
+    if garbage:
+        assert False, '\n'.join(garbage)
     c = client_context.client
-    c.drop_database("pymongo-pooling-tests")
-    c.drop_database("pymongo_test")
-    c.drop_database("pymongo_test1")
-    c.drop_database("pymongo_test2")
-    c.drop_database("pymongo_test_mike")
-    c.drop_database("pymongo_test_bernie")
+    if c:
+        c.drop_database("pymongo-pooling-tests")
+        c.drop_database("pymongo_test")
+        c.drop_database("pymongo_test1")
+        c.drop_database("pymongo_test2")
+        c.drop_database("pymongo_test_mike")
+        c.drop_database("pymongo_test_bernie")
+        c.close()
+
+    # Jython does not support gc.get_objects.
+    if not sys.platform.startswith('java'):
+        print_unclosed_clients()
 
 
 class PymongoTestRunner(unittest.TextTestRunner):
     def run(self, test):
         setup()
         result = super(PymongoTestRunner, self).run(test)
-        try:
+        teardown()
+        return result
+
+
+if HAVE_XML:
+    class PymongoXMLTestRunner(XMLTestRunner):
+        def run(self, test):
+            setup()
+            result = super(PymongoXMLTestRunner, self).run(test)
             teardown()
-        finally:
             return result
 
 
