@@ -51,7 +51,7 @@ class MockPool(Pool):
             + client.mock_members
             + client.mock_mongoses), "bad host: %s" % host_and_port
 
-        with Pool.get_socket(self, all_credentials) as sock_info:
+        with Pool.get_socket(self, all_credentials, checkout) as sock_info:
             sock_info.mock_host = self.mock_host
             sock_info.mock_port = self.mock_port
             yield sock_info
@@ -65,8 +65,9 @@ class MockMonitor(Monitor):
             topology,
             pool,
             topology_settings):
-        # MockMonitor gets a 'client' arg, regular monitors don't.
-        self.client = client
+        # MockMonitor gets a 'client' arg, regular monitors don't. Weakref it
+        # to avoid cycles.
+        self.client = weakref.proxy(client)
         Monitor.__init__(
             self,
             server_description,
@@ -74,9 +75,10 @@ class MockMonitor(Monitor):
             pool,
             topology_settings)
 
-    def _check_once(self, metadata=None, cluster_time=None):
+    def _check_once(self):
+        client = self.client
         address = self._server_description.address
-        response, rtt = self.client.mock_is_master('%s:%d' % address)
+        response, rtt = client.mock_is_master('%s:%d' % address)
         return ServerDescription(address, IsMaster(response), rtt)
 
 
