@@ -409,6 +409,33 @@ class TestSCRAM(unittest.TestCase):
         client_context.client.testscram.command("dropAllUsersFromDatabase")
         client_context.client.drop_database("testscram")
 
+    def test_scram_skip_empty_exchange(self):
+        listener = WhiteListEventListener("saslStart", "saslContinue")
+        client_context.create_user(
+            'testscram', 'sha256', 'pwd', roles=['dbOwner'],
+            mechanisms=['SCRAM-SHA-256'])
+
+        client = rs_or_single_client_noauth(
+            username='sha256', password='pwd', authSource='testscram',
+            event_listeners=[listener])
+        client.testscram.command('dbstats')
+
+        if client_context.version < (4, 4, -1):
+            # Assert we sent the skipEmptyExchange option.
+            first_event = listener.results['started'][0]
+            self.assertEqual(first_event.command_name, 'saslStart')
+            self.assertEqual(
+                first_event.command['options'], {'skipEmptyExchange': True})
+
+        # Assert the third exchange was skipped on servers that support it.
+        # Note that the first exchange occurs on the connection handshake.
+        started = listener.started_command_names()
+        if client_context.version.at_least(4, 4, -1):
+            self.assertEqual(started, ['saslContinue'])
+        else:
+            self.assertEqual(
+                started, ['saslStart', 'saslContinue', 'saslContinue'])
+
     @ignore_deprecations
     def test_scram(self):
         host, port = client_context.host, client_context.port
@@ -553,8 +580,13 @@ class TestSCRAM(unittest.TestCase):
             'mongodb://both:pwd@%s:%d/testscram' % (host, port),
             event_listeners=[self.listener])
         client.testscram.command('dbstats')
-        started = self.listener.results['started'][0]
-        self.assertEqual(started.command.get('mechanism'), 'SCRAM-SHA-256')
+        if client_context.version.at_least(4, 4, -1):
+            # Speculative authentication in 4.4+ sends saslStart with the
+            # handshake.
+            self.assertEqual(self.listener.results['started'], [])
+        else:
+            started = self.listener.results['started'][0]
+            self.assertEqual(started.command.get('mechanism'), 'SCRAM-SHA-256')
 
         client = rs_or_single_client_noauth(
             'mongodb://both:pwd@%s:%d/testscram?authMechanism=SCRAM-SHA-1'
@@ -663,8 +695,6 @@ class TestAuthURIOptions(unittest.TestCase):
         client_context.create_user('admin', 'admin', 'pass')
         client_context.create_user(
             'pymongo_test', 'user', 'pass', ['userAdmin', 'readWrite'])
-        self.client = rs_or_single_client_noauth(
-            username='admin', password='pass')
 
     def tearDown(self):
         client_context.drop_user('pymongo_test', 'user')
