@@ -48,6 +48,7 @@ from pymongo.errors import (AutoReconnect,
                             InvalidURI,
                             NetworkTimeout,
                             OperationFailure,
+                            ServerSelectionTimeoutError,
                             WriteConcernError)
 from pymongo.monitoring import (ServerHeartbeatListener,
                                 ServerHeartbeatStartedEvent)
@@ -59,6 +60,7 @@ from pymongo.read_preferences import ReadPreference
 from pymongo.server_selectors import (any_server_selector,
                                       writable_server_selector)
 from pymongo.server_type import SERVER_TYPE
+from pymongo.settings import TOPOLOGY_TYPE
 from pymongo.srv_resolver import _HAVE_DNSPYTHON
 from pymongo.write_concern import WriteConcern
 from test import (client_context,
@@ -100,6 +102,10 @@ class ClientUnitTest(unittest.TestCase):
         cls.client = rs_or_single_client(connect=False,
                                          serverSelectionTimeoutMS=100)
 
+    @classmethod
+    def tearDownClass(cls):
+        cls.client.close()
+
     def test_keyword_arg_defaults(self):
         client = MongoClient(socketTimeoutMS=None,
                              connectTimeoutMS=20000,
@@ -128,6 +134,24 @@ class ClientUnitTest(unittest.TestCase):
         self.assertEqual(ReadPreference.PRIMARY, client.read_preference)
         self.assertAlmostEqual(12, client.server_selection_timeout)
 
+    def test_connect_timeout(self):
+        client = MongoClient(connect=False, connectTimeoutMS=None,
+                             socketTimeoutMS=None)
+        pool_opts = client._MongoClient__options.pool_options
+        self.assertEqual(None, pool_opts.socket_timeout)
+        self.assertEqual(None, pool_opts.connect_timeout)
+        client = MongoClient(connect=False, connectTimeoutMS=0,
+                             socketTimeoutMS=0)
+        pool_opts = client._MongoClient__options.pool_options
+        self.assertEqual(None, pool_opts.socket_timeout)
+        self.assertEqual(None, pool_opts.connect_timeout)
+        client = MongoClient(
+            'mongodb://localhost/?connectTimeoutMS=0&socketTimeoutMS=0',
+            connect=False)
+        pool_opts = client._MongoClient__options.pool_options
+        self.assertEqual(None, pool_opts.socket_timeout)
+        self.assertEqual(None, pool_opts.connect_timeout)
+
     def test_types(self):
         self.assertRaises(TypeError, MongoClient, 1)
         self.assertRaises(TypeError, MongoClient, 1.14)
@@ -785,6 +809,24 @@ class TestClient(IntegrationTest):
         client.close()
         self.assertTrue(client._kill_cursors_executor._stopped)
 
+    def test_uri_connect_option(self):
+        # Ensure that topology is not opened if connect=False.
+        client = rs_client(connect=False)
+        self.assertFalse(client._topology._opened)
+
+        # Ensure kill cursors thread has not been started.
+        kc_thread = client._kill_cursors_executor._thread
+        self.assertFalse(kc_thread and kc_thread.is_alive())
+
+        # Using the client should open topology and start the thread.
+        client.admin.command('isMaster')
+        self.assertTrue(client._topology._opened)
+        kc_thread = client._kill_cursors_executor._thread
+        self.assertTrue(kc_thread and kc_thread.is_alive())
+
+        # Tear down.
+        client.close()
+
     def test_close_does_not_open_servers(self):
         client = rs_client(connect=False)
         topology = client._topology
@@ -792,6 +834,19 @@ class TestClient(IntegrationTest):
         client.close()
         self.assertEqual(topology._servers, {})
 
+    def test_close_closes_sockets(self):
+        client = rs_client()
+        self.addCleanup(client.close)
+        client.test.test.find_one()
+        topology = client._topology
+        client.close()
+        for server in topology._servers.values():
+            self.assertFalse(server._pool.sockets)
+            self.assertTrue(server._monitor._executor._stopped)
+            self.assertTrue(server._monitor._rtt_monitor._executor._stopped)
+            self.assertFalse(server._monitor._pool.sockets)
+            self.assertFalse(server._monitor._rtt_monitor._pool.sockets)
+
     def test_bad_uri(self):
         with self.assertRaises(InvalidURI):
             MongoClient("http://localhost")
@@ -895,7 +950,7 @@ class TestClient(IntegrationTest):
         assertRaisesExactly(
             OperationFailure, lazy_client.test.collection.find_one)
 
-    @client_context.require_no_ssl
+    @client_context.require_no_tls
     def test_unix_socket(self):
         if not hasattr(socket, "AF_UNIX"):
             raise SkipTest("UNIX-sockets are not supported on this system")
@@ -959,8 +1014,8 @@ class TestClient(IntegrationTest):
         c = connected(rs_or_single_client(socketTimeoutMS=None))
         self.assertEqual(None, get_pool(c).opts.socket_timeout)
 
-        self.assertRaises(ValueError,
-                          rs_or_single_client, socketTimeoutMS=0)
+        c = connected(rs_or_single_client(socketTimeoutMS=0))
+        self.assertEqual(None, get_pool(c).opts.socket_timeout)
 
         self.assertRaises(ValueError,
                           rs_or_single_client, socketTimeoutMS=-1)
@@ -1062,7 +1117,7 @@ class TestClient(IntegrationTest):
 
     @client_context.require_ipv6
     def test_ipv6(self):
-        if client_context.ssl:
+        if client_context.tls:
             if not HAVE_IPADDRESS:
                 raise SkipTest("Need the ipaddress module to test with SSL")
 
@@ -1083,6 +1138,7 @@ class TestClient(IntegrationTest):
         self.assertTrue("pymongo_test" in dbs)
         self.assertTrue("pymongo_test_bernie" in dbs)
 
+    @ignore_deprecations
     @client_context.require_no_mongos
     def test_fsync_lock_unlock(self):
         if server_is_master_with_slave(client_context.client):
@@ -1106,13 +1162,19 @@ class TestClient(IntegrationTest):
             time.sleep(1)
         self.assertFalse(locked)
 
-    def test_is_locked_does_not_raise_warning(self):
-        client = rs_or_single_client()
-        with warnings.catch_warnings(record=True) as ctx:
-            warnings.simplefilter("always")
-            _ = client.is_locked
-            self.assertFalse(
-                any(issubclass(w.category, DeprecationWarning) for w in ctx))
+    def test_deprecated_methods(self):
+        with warnings.catch_warnings():
+            warnings.simplefilter("error", DeprecationWarning)
+            with self.assertRaisesRegex(DeprecationWarning,
+                                        'is_locked is deprecated'):
+                _ = self.client.is_locked
+            if not client_context.is_mongos:
+                with self.assertRaisesRegex(DeprecationWarning,
+                                            'fsync is deprecated'):
+                    self.client.fsync(lock=True)
+                with self.assertRaisesRegex(DeprecationWarning,
+                                            'unlock is deprecated'):
+                    self.client.unlock()
 
     def test_contextlib(self):
         client = rs_or_single_client()
@@ -1462,7 +1524,7 @@ class TestClient(IntegrationTest):
         self.addCleanup(client.close)
         client.admin.command('ping')
         pool = get_pool(client)
-        pool_id = pool.pool_id
+        generation = pool.generation
 
         # Continuously reset the pool.
         class ResetPoolThread(threading.Thread):
@@ -1487,8 +1549,9 @@ class TestClient(IntegrationTest):
         try:
             while True:
                 for _ in range(10):
-                    client._topology.update_pool()
-                if pool_id != pool.pool_id:
+                    client._topology.update_pool(
+                        client._MongoClient__all_credentials)
+                if generation != pool.generation:
                     break
         finally:
             t.stop()
@@ -1514,6 +1577,8 @@ class TestClient(IntegrationTest):
             return original_connect(*args, **kwargs)
 
         pool.connect = stall_connect
+        # Un-patch Pool.connect to break the cyclic reference.
+        self.addCleanup(delattr, pool, 'connect')
 
         # Wait for the background thread to start creating connections
         wait_until(lambda: len(pool.sockets) > 1, 'start creating connections')
@@ -1526,6 +1591,29 @@ class TestClient(IntegrationTest):
             # Each ping command should not take more than 2 seconds
             self.assertLess(total, 2)
 
+    @client_context.require_replica_set
+    def test_direct_connection(self):
+        # direct_connection=True should result in Single topology.
+        client = rs_or_single_client(directConnection=True)
+        client.admin.command('ping')
+        self.assertEqual(len(client.nodes), 1)
+        self.assertEqual(client._topology_settings.get_topology_type(),
+                         TOPOLOGY_TYPE.Single)
+        client.close()
+
+        # direct_connection=False should result in RS topology.
+        client = rs_or_single_client(directConnection=False)
+        client.admin.command('ping')
+        self.assertGreaterEqual(len(client.nodes), 1)
+        self.assertIn(client._topology_settings.get_topology_type(),
+                      [TOPOLOGY_TYPE.ReplicaSetNoPrimary,
+                       TOPOLOGY_TYPE.ReplicaSetWithPrimary])
+        client.close()
+
+        # directConnection=True, should error with multiple hosts as a list.
+        with self.assertRaises(ConfigurationError):
+            MongoClient(['host1', 'host2'], directConnection=True)
+
 
 class TestExhaustCursor(IntegrationTest):
     """Test that clients properly handle errors from exhaust cursors."""
@@ -1586,12 +1674,12 @@ class TestExhaustCursor(IntegrationTest):
             msg += encode({'$err': 'mock err', 'code': 0})
             return message._OpReply.unpack(msg)
 
-        saved = sock_info.receive_message
         sock_info.receive_message = receive_message
         self.assertRaises(OperationFailure, list, cursor)
-        sock_info.receive_message = saved
+        # Unpatch the instance.
+        del sock_info.receive_message
 
-        # The socket is returned the pool and it still works.
+        # The socket is returned to the pool and it still works.
         self.assertEqual(200, collection.count_documents({}))
         self.assertIn(sock_info, pool.sockets)
 
@@ -1728,6 +1816,7 @@ class TestMongoClientFailover(MockClientTest):
                 mongoses=[],
                 host='b:2',  # Pass a secondary.
                 replicaSet='rs')
+            self.addCleanup(c.close)
 
             wait_until(lambda: len(c.nodes) == 3, 'connect')
             self.assertEqual(c.address, ('a', 1))
@@ -1757,7 +1846,10 @@ class TestMongoClientFailover(MockClientTest):
             mongoses=[],
             host='b:2',  # Pass a secondary.
             replicaSet='rs',
-            retryReads=False)
+            retryReads=False,
+            serverSelectionTimeoutMS=1000,
+        )
+        self.addCleanup(c.close)
 
         wait_until(lambda: len(c.nodes) == 3, 'connect')
 
@@ -1766,7 +1858,9 @@ class TestMongoClientFailover(MockClientTest):
         c.kill_host('b:2')
         c.kill_host('c:3')
 
-        # MongoClient discovers it's alone.
+        # MongoClient discovers it's alone. The first attempt raises either
+        # ServerSelectionTimeoutError or AutoReconnect (from
+        # MockPool.get_socket).
         self.assertRaises(AutoReconnect, c.db.collection.find_one)
 
         # But it can reconnect.
@@ -1786,7 +1880,9 @@ class TestMongoClientFailover(MockClientTest):
                 host='a:1',
                 replicaSet='rs',
                 connect=False,
-                retryReads=False)
+                retryReads=False,
+                serverSelectionTimeoutMS=1000)
+            self.addCleanup(c.close)
 
             # Set host-specific information so we can test whether it is reset.
             c.set_wire_version_range('a:1', 2, 6)
@@ -1796,7 +1892,9 @@ class TestMongoClientFailover(MockClientTest):
 
             c.kill_host('a:1')
 
-            # MongoClient is disconnected from the primary.
+            # MongoClient is disconnected from the primary. This raises either
+            # ServerSelectionTimeoutError or AutoReconnect (from
+            # MockPool.get_socket).
             self.assertRaises(AutoReconnect, operation_callback, c)
 
             # The primary's description is reset.
@@ -1846,6 +1944,36 @@ class TestMongoClientFailover(MockClientTest):
         task.kill()
         self.assertTrue(task.dead)
 
+    def test_gevent_timeout(self):
+        if not gevent_monkey_patched():
+            raise SkipTest("Must be running monkey patched by gevent")
+        from gevent import spawn, Timeout
+        client = rs_or_single_client(maxPoolSize=1)
+        coll = client.pymongo_test.test
+        coll.insert_one({})
+
+        def contentious_task():
+            # The 10 second timeout causes this test to fail without blocking
+            # forever if a bug like PYTHON-2334 is reintroduced.
+            with Timeout(10):
+                coll.find_one({'$where': delay(1)})
+
+        def timeout_task():
+            with Timeout(.5):
+                try:
+                    coll.find_one({})
+                except Timeout:
+                    pass
+
+        ct = spawn(contentious_task)
+        tt = spawn(timeout_task)
+        tt.join(15)
+        ct.join(15)
+        self.assertTrue(tt.dead)
+        self.assertTrue(ct.dead)
+        self.assertIsNone(tt.get())
+        self.assertIsNone(ct.get())
+
 
 if __name__ == "__main__":
     unittest.main()
