@@ -234,6 +234,20 @@ class TestCollection(IntegrationTest):
         with self.write_concern_collection() as coll:
             coll.create_indexes([IndexModel('hello')])
 
+    @client_context.require_version_max(4, 3, -1)
+    def test_create_indexes_commitQuorum_requires_44(self):
+        db = self.db
+        with self.assertRaisesRegex(
+                ConfigurationError,
+                'Must be connected to MongoDB 4\.4\+ to use the commitQuorum '
+                'option for createIndexes'):
+            db.coll.create_indexes([IndexModel('a')], commitQuorum="majority")
+
+    @client_context.require_no_standalone
+    @client_context.require_version_min(4, 4, -1)
+    def test_create_indexes_commitQuorum(self):
+        self.db.coll.create_indexes([IndexModel('a')], commitQuorum="majority")
+
     def test_create_index(self):
         db = self.db
 
@@ -619,11 +633,6 @@ class TestCollection(IntegrationTest):
                           partialFilterExpression={"x": {"$asdasd": 3}})
         self.assertRaises(OperationFailure, db.test.create_index, "x",
                           partialFilterExpression={"$and": 5})
-        self.assertRaises(OperationFailure, db.test.create_index, "x",
-                          partialFilterExpression={
-                              "$and": [{"$and":  [{"x": {"$lt": 2}},
-                                                  {"x": {"$gt": 0}}]},
-                                       {"x": {"$exists": True}}]})
 
         self.assertEqual("x_1", db.test.create_index(
             [('x', ASCENDING)], partialFilterExpression={"a": {"$lte": 1.5}}))
