@@ -25,7 +25,7 @@ from bson.codec_options import CodecOptions
 from bson.objectid import ObjectId
 from pymongo.errors import OperationFailure
 from pymongo.write_concern import WriteConcern
-from test import client_context, IntegrationTest
+from test import client_context, unittest, IntegrationTest
 from test.utils import connected, rs_or_single_client, single_client
 
 
@@ -55,21 +55,21 @@ class TestCommon(IntegrationTest):
         self.assertEqual(None, coll.find_one({'uu': uu}))
         self.assertEqual(uu, coll.find_one({'uu': UUIDLegacy(uu)})['uu'])
 
-        # Test Cursor.count
-        self.assertEqual(0, coll.find({'uu': uu}).count())
+        # Test count_documents
+        self.assertEqual(0, coll.count_documents({'uu': uu}))
         coll = self.db.get_collection(
             "uuid", CodecOptions(uuid_representation=PYTHON_LEGACY))
-        self.assertEqual(1, coll.find({'uu': uu}).count())
+        self.assertEqual(1, coll.count_documents({'uu': uu}))
 
         # Test delete
         coll = self.db.get_collection(
             "uuid", CodecOptions(uuid_representation=STANDARD))
         coll.delete_one({'uu': uu})
-        self.assertEqual(1, coll.count())
+        self.assertEqual(1, coll.count_documents({}))
         coll = self.db.get_collection(
             "uuid", CodecOptions(uuid_representation=PYTHON_LEGACY))
         coll.delete_one({'uu': uu})
-        self.assertEqual(0, coll.count())
+        self.assertEqual(0, coll.count_documents({}))
 
         # Test update_one
         coll.insert_one({'_id': uu, 'i': 1})
@@ -133,7 +133,7 @@ class TestCommon(IntegrationTest):
         self.assertEqual([], result)
 
         result = coll.map_reduce(map, reduce, "results", query=q)
-        self.assertEqual(0, self.db.results.count())
+        self.assertEqual(0, self.db.results.count_documents({}))
 
         coll = self.db.get_collection(
             "uuid", CodecOptions(uuid_representation=PYTHON_LEGACY))
@@ -142,7 +142,7 @@ class TestCommon(IntegrationTest):
         self.assertEqual(2, len(result))
 
         result = coll.map_reduce(map, reduce, "results", query=q)
-        self.assertEqual(2, self.db.results.count())
+        self.assertEqual(2, self.db.results.count_documents({}))
 
         self.db.drop_collection("result")
         coll.drop()
@@ -155,6 +155,10 @@ class TestCommon(IntegrationTest):
         wc = WriteConcern(w=2, wtimeout=1000)
         self.assertEqual(wc, c.write_concern)
 
+        # Can we override back to the server default?
+        db = c.get_database('pymongo_test', write_concern=WriteConcern())
+        self.assertEqual(db.write_concern, WriteConcern())
+
         db = c.pymongo_test
         self.assertEqual(wc, db.write_concern)
         coll = db.test
