@@ -146,6 +146,18 @@ class TestCursor(IntegrationTest):
             self.assertEqual(0, cursor._Cursor__query_flags)
             self.assertFalse(cursor._Cursor__exhaust)
 
+    def test_allow_disk_use(self):
+        db = self.db
+        db.pymongo_test.drop()
+        coll = db.pymongo_test
+
+        self.assertRaises(TypeError, coll.find().allow_disk_use, 'baz')
+
+        cursor = coll.find().allow_disk_use(True)
+        self.assertEqual(True, cursor._Cursor__allow_disk_use)
+        cursor = coll.find().allow_disk_use(False)
+        self.assertEqual(False, cursor._Cursor__allow_disk_use)
+
     def test_max_time_ms(self):
         db = self.db
         db.pymongo_test.drop()
@@ -832,8 +844,20 @@ class TestCursor(IntegrationTest):
         self.assertEqual(3, len(list(db.test.find().where('this.x < 3'))))
         self.assertEqual(3,
                          len(list(db.test.find().where(Code('this.x < 3')))))
-        self.assertEqual(3, len(list(db.test.find().where(Code('this.x < i',
-                                                               {"i": 3})))))
+
+        code_with_scope = Code('this.x < i', {"i": 3})
+        if client_context.version.at_least(4, 3, 3):
+            # MongoDB 4.4 removed support for Code with scope.
+            with self.assertRaises(OperationFailure):
+                list(db.test.find().where(code_with_scope))
+
+            code_with_empty_scope = Code('this.x < 3', {})
+            with self.assertRaises(OperationFailure):
+                list(db.test.find().where(code_with_empty_scope))
+        else:
+            self.assertEqual(
+                3, len(list(db.test.find().where(code_with_scope))))
+
         self.assertEqual(10, len(list(db.test.find())))
 
         self.assertEqual(3, db.test.find().where('this.x < 3').count())
@@ -890,6 +914,8 @@ class TestCursor(IntegrationTest):
 
         self.assertEqual(cursor, cursor.rewind())
 
+    # manipulate, oplog_reply, and snapshot are all deprecated.
+    @ignore_deprecations
     def test_clone(self):
         self.db.test.insert_many([{"x": i} for i in range(1, 4)])
 
@@ -928,31 +954,30 @@ class TestCursor(IntegrationTest):
 
         # Just test attributes
         cursor = self.db.test.find({"x": re.compile("^hello.*")},
+                                   projection={'_id': False},
                                    skip=1,
                                    no_cursor_timeout=True,
                                    cursor_type=CursorType.TAILABLE_AWAIT,
+                                   sort=[("x", 1)],
                                    allow_partial_results=True,
+                                   oplog_replay=True,
+                                   batch_size=123,
                                    manipulate=False,
-                                   projection={'_id': False}).limit(2)
+                                   collation={'locale': 'en_US'},
+                                   hint=[("_id", 1)],
+                                   max_scan=100,
+                                   max_time_ms=1000,
+                                   return_key=True,
+                                   show_record_id=True,
+                                   snapshot=True,
+                                   allow_disk_use=True).limit(2)
         cursor.min([('a', 1)]).max([('b', 3)])
         cursor.add_option(128)
         cursor.comment('hi!')
 
+        # Every attribute should be the same.
         cursor2 = cursor.clone()
-        self.assertEqual(cursor._Cursor__skip, cursor2._Cursor__skip)
-        self.assertEqual(cursor._Cursor__limit, cursor2._Cursor__limit)
-        self.assertEqual(type(cursor._Cursor__codec_options),
-                         type(cursor2._Cursor__codec_options))
-        self.assertEqual(cursor._Cursor__manipulate,
-                         cursor2._Cursor__manipulate)
-        self.assertEqual(cursor._Cursor__query_flags,
-                         cursor2._Cursor__query_flags)
-        self.assertEqual(cursor._Cursor__comment,
-                         cursor2._Cursor__comment)
-        self.assertEqual(cursor._Cursor__min,
-                         cursor2._Cursor__min)
-        self.assertEqual(cursor._Cursor__max,
-                         cursor2._Cursor__max)
+        self.assertEqual(cursor.__dict__, cursor2.__dict__)
 
         # Shallow copies can so can mutate
         cursor2 = copy.copy(cursor)
@@ -987,6 +1012,14 @@ class TestCursor(IntegrationTest):
         self.assertTrue(isinstance(cursor2._Cursor__hint, SON))
         self.assertEqual(cursor._Cursor__hint, cursor2._Cursor__hint)
 
+    def test_clone_empty(self):
+        self.db.test.delete_many({})
+        self.db.test.insert_many([{"x": i} for i in range(1, 4)])
+        cursor = self.db.test.find()[2:2]
+        cursor2 = cursor.clone()
+        self.assertRaises(StopIteration, cursor.next)
+        self.assertRaises(StopIteration, cursor2.next)
+
     @ignore_deprecations
     def test_count_with_fields(self):
         self.db.test.drop()
