@@ -51,11 +51,13 @@ from test import (client_context,
                   SkipTest,
                   unittest,
                   IntegrationTest)
-from test.utils import (ignore_deprecations,
+from test.utils import (EventListener,
+                        ignore_deprecations,
                         remove_all_users,
                         rs_or_single_client_noauth,
                         rs_or_single_client,
                         server_started_with_auth,
+                        wait_until,
                         IMPOSSIBLE_WRITE_CONCERN,
                         OvertCommandListener)
 from test.test_custom_types import DECIMAL_CODECOPTS
@@ -367,6 +369,23 @@ class TestDatabase(IntegrationTest):
         self.assertTrue(db.validate_collection(db.test, scandata=True, full=True))
         self.assertTrue(db.validate_collection(db.test, True, True))
 
+    @client_context.require_version_min(4, 3, 3)
+    def test_validate_collection_background(self):
+        db = self.client.pymongo_test
+        db.test.insert_one({"dummy": u"object"})
+        coll = db.test
+        self.assertTrue(db.validate_collection(coll, background=False))
+        # The inMemory storage engine does not support background=True.
+        if client_context.storage_engine != 'inMemory':
+            self.assertTrue(db.validate_collection(coll, background=True))
+            self.assertTrue(
+                db.validate_collection(coll, scandata=True, background=True))
+            # The server does not support background=True with full=True.
+            # Assert that we actually send the background option by checking
+            # that this combination fails.
+            with self.assertRaises(OperationFailure):
+                db.validate_collection(coll, full=True, background=True)
+
     @client_context.require_no_mongos
     def test_profiling_levels(self):
         db = self.client.pymongo_test
@@ -659,14 +678,6 @@ class TestDatabase(IntegrationTest):
         admin_db_auth = self.client.admin
         users_db_auth = self.client.pymongo_test
 
-        # Non-root client.
-        client = rs_or_single_client_noauth()
-        admin_db = client.admin
-        users_db = client.pymongo_test
-        other_db = client.pymongo_test1
-
-        self.assertRaises(OperationFailure, users_db.test.find_one)
-
         admin_db_auth.add_user(
             'ro-admin',
             'pass',
@@ -677,15 +688,36 @@ class TestDatabase(IntegrationTest):
             'user', 'pass', roles=["userAdmin", "readWrite"])
         self.addCleanup(remove_all_users, users_db_auth)
 
+        # Non-root client.
+        listener = EventListener()
+        client = rs_or_single_client_noauth(event_listeners=[listener])
+        admin_db = client.admin
+        users_db = client.pymongo_test
+        other_db = client.pymongo_test1
+
+        self.assertRaises(OperationFailure, users_db.test.find_one)
+        self.assertEqual(listener.started_command_names(), ['find'])
+        listener.reset()
+
         # Regular user should be able to query its own db, but
         # no other.
         users_db.authenticate('user', 'pass')
+        if client_context.version.at_least(3, 0):
+            self.assertEqual(listener.started_command_names()[0], 'saslStart')
+        else:
+            self.assertEqual(listener.started_command_names()[0], 'getnonce')
+
         self.assertEqual(0, users_db.test.count_documents({}))
         self.assertRaises(OperationFailure, other_db.test.find_one)
 
+        listener.reset()
         # Admin read-only user should be able to query any db,
         # but not write.
         admin_db.authenticate('ro-admin', 'pass')
+        if client_context.version.at_least(3, 0):
+            self.assertEqual(listener.started_command_names()[0], 'saslStart')
+        else:
+            self.assertEqual(listener.started_command_names()[0], 'getnonce')
         self.assertEqual(None, other_db.test.find_one())
         self.assertRaises(OperationFailure,
                           other_db.test.insert_one, {})
@@ -693,8 +725,23 @@ class TestDatabase(IntegrationTest):
         # Close all sockets.
         client.close()
 
+        listener.reset()
         # We should still be able to write to the regular user's db.
         self.assertTrue(users_db.test.delete_many({}))
+        names = listener.started_command_names()
+        if client_context.version.at_least(4, 4, -1):
+            # No speculation with multiple users (but we do skipEmptyExchange).
+            self.assertEqual(
+                names, ['saslStart', 'saslContinue', 'saslStart',
+                        'saslContinue', 'delete'])
+        elif client_context.version.at_least(3, 0):
+            self.assertEqual(
+                names, ['saslStart', 'saslContinue', 'saslContinue',
+                        'saslStart', 'saslContinue', 'saslContinue', 'delete'])
+        else:
+            self.assertEqual(
+                names, ['getnonce', 'authenticate',
+                        'getnonce', 'authenticate', 'delete'])
 
         # And read from other dbs...
         self.assertEqual(0, other_db.test.count_documents({}))
@@ -907,12 +954,12 @@ class TestDatabase(IntegrationTest):
         # command document will have no 'ok' field. We should raise
         # OperationFailure instead of KeyError.
         self.assertRaises(OperationFailure,
-                          helpers._check_command_response, {})
+                          helpers._check_command_response, {}, None)
 
         try:
-            helpers._check_command_response({'$err': 'foo'})
+            helpers._check_command_response({'$err': 'foo'}, None)
         except OperationFailure as e:
-            self.assertEqual(e.args[0], 'foo')
+            self.assertEqual(e.args[0], "foo, full error: {'$err': 'foo'}")
         else:
             self.fail("_check_command_response didn't raise OperationFailure")
 
@@ -923,9 +970,9 @@ class TestDatabase(IntegrationTest):
             'raw': {'shard0/host0,host1': {'ok': 0, 'errmsg': 'inner'}}}
 
         with self.assertRaises(OperationFailure) as context:
-            helpers._check_command_response(error_document)
+            helpers._check_command_response(error_document, None)
 
-        self.assertEqual('inner', str(context.exception))
+        self.assertIn('inner', str(context.exception))
 
         # If a shard has no primary and you run a command like dbstats, which
         # cannot be run on a secondary, mongos's response includes empty "raw"
@@ -936,9 +983,9 @@ class TestDatabase(IntegrationTest):
             'raw': {'shard0/host0,host1': {}}}
 
         with self.assertRaises(OperationFailure) as context:
-            helpers._check_command_response(error_document)
+            helpers._check_command_response(error_document, None)
 
-        self.assertEqual('outer', str(context.exception))
+        self.assertIn('outer', str(context.exception))
 
         # Raw error has ok: 0 but no errmsg. Not a known case, but test it.
         error_document = {
@@ -947,9 +994,9 @@ class TestDatabase(IntegrationTest):
             'raw': {'shard0/host0,host1': {'ok': 0}}}
 
         with self.assertRaises(OperationFailure) as context:
-            helpers._check_command_response(error_document)
+            helpers._check_command_response(error_document, None)
 
-        self.assertEqual('outer', str(context.exception))
+        self.assertIn('outer', str(context.exception))
 
     @client_context.require_test_commands
     @client_context.require_no_mongos
@@ -1017,5 +1064,55 @@ class TestDatabase(IntegrationTest):
         self.assertIsInstance(current_op, MySON)
 
 
+class TestDatabaseAggregation(IntegrationTest):
+    def setUp(self):
+        self.pipeline = [{"$listLocalSessions": {}},
+                         {"$limit": 1},
+                         {"$addFields": {"dummy": "dummy field"}},
+                         {"$project": {"_id": 0, "dummy": 1}}]
+        self.result = {"dummy": "dummy field"}
+        self.admin = self.client.admin
+
+    @client_context.require_version_min(3, 6, 0)
+    def test_database_aggregation(self):
+        with self.admin.aggregate(self.pipeline) as cursor:
+            result = next(cursor)
+            self.assertEqual(result, self.result)
+
+    @client_context.require_version_min(3, 6, 0)
+    @client_context.require_no_mongos
+    def test_database_aggregation_fake_cursor(self):
+        coll_name = "test_output"
+        if client_context.version < (4, 3):
+            db_name = "admin"
+            write_stage = {"$out": coll_name}
+        else:
+            # SERVER-43287 disallows writing with $out to the admin db, use
+            # $merge instead.
+            db_name = "pymongo_test"
+            write_stage = {
+                "$merge": {"into": {"db": db_name, "coll": coll_name}}}
+        output_coll = self.client[db_name][coll_name]
+        output_coll.drop()
+        self.addCleanup(output_coll.drop)
+
+        admin = self.admin.with_options(write_concern=WriteConcern(w=0))
+        pipeline = self.pipeline[:]
+        pipeline.append(write_stage)
+        with admin.aggregate(pipeline) as cursor:
+            with self.assertRaises(StopIteration):
+                next(cursor)
+
+        result = wait_until(output_coll.find_one, "read unacknowledged write")
+        self.assertEqual(result["dummy"], self.result["dummy"])
+
+    @client_context.require_version_max(3, 6, 0, -1)
+    def test_database_aggregation_unsupported(self):
+        err_msg = r"Database.aggregate\(\) is only supported on MongoDB 3.6\+."
+        with self.assertRaisesRegex(ConfigurationError, err_msg):
+            with self.admin.aggregate(self.pipeline) as _:
+                pass
+
+
 if __name__ == "__main__":
     unittest.main()
