@@ -19,6 +19,7 @@
 
 import datetime
 import sys
+import zipfile
 sys.path[0:0] = [""]
 
 from bson.objectid import ObjectId
@@ -33,10 +34,11 @@ from gridfs.grid_file import (DEFAULT_CHUNK_SIZE,
 from gridfs.errors import NoFile
 from pymongo import MongoClient
 from pymongo.errors import ConfigurationError, ServerSelectionTimeoutError
+from pymongo.message import _CursorAddress
 from test import (IntegrationTest,
                   unittest,
                   qcheck)
-from test.utils import rs_or_single_client
+from test.utils import rs_or_single_client, EventListener
 
 
 class TestGridFileNoConnect(unittest.TestCase):
@@ -616,6 +618,51 @@ Bye"""))
         with self.assertRaises(ConfigurationError):
             GridIn(rs_or_single_client(w=0).pymongo_test.fs)
 
+    def test_survive_cursor_not_found(self):
+        # By default the find command returns 101 documents in the first batch.
+        # Use 102 batches to cause a single getMore.
+        chunk_size = 1024
+        data = b'd' * (102 * chunk_size)
+        listener = EventListener()
+        client = rs_or_single_client(event_listeners=[listener])
+        db = client.pymongo_test
+        with GridIn(db.fs, chunk_size=chunk_size) as infile:
+            infile.write(data)
+
+        with GridOut(db.fs, infile._id) as outfile:
+            self.assertEqual(len(outfile.readchunk()), chunk_size)
+
+            # Kill the cursor to simulate the cursor timing out on the server
+            # when an application spends a long time between two calls to
+            # readchunk().
+            client._close_cursor_now(
+                outfile._GridOut__chunk_iter._cursor.cursor_id,
+                _CursorAddress(client.address, db.fs.chunks.full_name))
+
+            # Read the rest of the file without error.
+            self.assertEqual(len(outfile.read()), len(data) - chunk_size)
+
+        # Paranoid, ensure that a getMore was actually sent.
+        self.assertIn("getMore", listener.started_command_names())
+
+    def test_zip(self):
+        zf = StringIO()
+        z = zipfile.ZipFile(zf, "w")
+        z.writestr("test.txt", b"hello world")
+        z.close()
+        zf.seek(0)
+
+        f = GridIn(self.db.fs, filename="test.zip")
+        f.write(zf)
+        f.close()
+        self.assertEqual(1, self.db.fs.files.count_documents({}))
+        self.assertEqual(1, self.db.fs.chunks.count_documents({}))
+
+        g = GridOut(self.db.fs, f._id)
+        z = zipfile.ZipFile(g)
+        self.assertSequenceEqual(z.namelist(), ["test.txt"])
+        self.assertEqual(z.read("test.txt"), b"hello world")
+
 
 if __name__ == "__main__":
     unittest.main()
