@@ -19,55 +19,15 @@ import threading
 
 sys.path[0:0] = [""]
 
-from pymongo import monitoring
 from pymongo.errors import ConnectionFailure
 from pymongo.ismaster import IsMaster
 from pymongo.monitor import Monitor
-from pymongo.pool import PoolOptions
 from test import unittest, client_knobs
-from test.utils import HeartbeatEventListener, single_client, wait_until
-
-
-class MockSocketInfo(object):
-    def close(self):
-        pass
-
-    def __enter__(self):
-        return self
-
-    def __exit__(self, exc_type, exc_val, exc_tb):
-        pass
-
-
-class MockPool(object):
-    def __init__(self, *args, **kwargs):
-        self.pool_id = 0
-        self._lock = threading.Lock()
-        self.opts = PoolOptions()
-
-    def get_socket(self, all_credentials):
-        return MockSocketInfo()
-
-    def return_socket(self, _):
-        pass
-
-    def reset(self):
-        with self._lock:
-            self.pool_id += 1
-
-    def remove_stale_sockets(self):
-        pass
+from test.utils import (HeartbeatEventListener, MockPool, single_client,
+                        wait_until)
 
 
 class TestHeartbeatMonitoring(unittest.TestCase):
-    @classmethod
-    def setUpClass(cls):
-        cls.saved_listeners = monitoring._LISTENERS
-        monitoring._LISTENERS = monitoring._Listeners([], [], [], [])
-
-    @classmethod
-    def tearDownClass(cls):
-        monitoring._LISTENERS = cls.saved_listeners
 
     def create_mock_monitor(self, responses, uri, expected_results):
         listener = HeartbeatEventListener()
