@@ -29,7 +29,7 @@ from bson.codec_options import CodecOptions
 from bson.objectid import ObjectId
 from bson.py3compat import string_type
 from bson.son import SON
-from pymongo import ASCENDING, DESCENDING
+from pymongo import ASCENDING, DESCENDING, GEOHAYSTACK
 from pymongo.database import Database
 from pymongo.common import partition_node
 from pymongo.errors import (BulkWriteError,
@@ -43,6 +43,7 @@ from pymongo.errors import (BulkWriteError,
                             WriteConcernError,
                             WTimeoutError)
 from pymongo.message import _CursorAddress
+from pymongo.operations import IndexModel
 from pymongo.son_manipulator import (AutoReference,
                                      NamespaceInjector,
                                      ObjectIdShuffler,
@@ -105,6 +106,18 @@ class TestDeprecations(IntegrationTest):
         finally:
             self.db.test.drop()
 
+    def test_reindex_deprecation(self):
+        self.assertRaises(DeprecationWarning, lambda: self.db.test.reindex())
+
+    def test_geoHaystack_deprecation(self):
+        self.addCleanup(self.db.test.drop)
+        keys = [("pos", GEOHAYSTACK), ("type", ASCENDING)]
+        self.assertRaises(
+            DeprecationWarning, self.db.test.create_index, keys, bucketSize=1)
+        indexes = [IndexModel(keys, bucketSize=1)]
+        self.assertRaises(
+            DeprecationWarning, self.db.test.create_indexes, indexes)
+
 
 class TestLegacy(IntegrationTest):
 
@@ -2293,6 +2306,8 @@ class TestLegacyBulkWriteConcern(BulkTestBase):
     @classmethod
     def tearDownClass(cls):
         cls.deprecation_filter.stop()
+        if cls.secondary:
+            cls.secondary.close()
 
     def cause_wtimeout(self, batch):
         if self.need_replication_stopped:
