@@ -189,6 +189,12 @@ class TestObjectId(unittest.TestCase):
 
         for tstamp, exp_datetime_args in TEST_DATA.items():
             oid = generate_objectid_with_timestamp(tstamp)
+            # 32-bit platforms may overflow in datetime.fromtimestamp.
+            if tstamp > 0x7FFFFFFF and sys.maxsize < 2**32:
+                try:
+                    oid.generation_time
+                except (OverflowError, ValueError):
+                    continue
             self.assertEqual(
                 oid.generation_time,
                 datetime.datetime(*exp_datetime_args, tzinfo=utc))
