@@ -15,14 +15,17 @@
 import datetime
 import uuid
 
-from bson import BSON
-from bson.binary import JAVA_LEGACY
+from bson import decode, encode
+from bson.binary import Binary, JAVA_LEGACY
 from bson.codec_options import CodecOptions
-from bson.raw_bson import RawBSONDocument
+from bson.errors import InvalidBSON
+from bson.raw_bson import RawBSONDocument, DEFAULT_RAW_BSON_OPTIONS
+from bson.son import SON
 from test import client_context, unittest
+from test.test_client import IntegrationTest
 
 
-class TestRawBSONDocument(unittest.TestCase):
+class TestRawBSONDocument(IntegrationTest):
 
     # {u'_id': ObjectId('556df68b6e32ab21a95e0785'),
     #  u'name': u'Sherlock',
@@ -36,6 +39,7 @@ class TestRawBSONDocument(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls):
+        super(TestRawBSONDocument, cls).setUpClass()
         cls.client = client_context.client
 
     def tearDown(self):
@@ -51,6 +55,21 @@ class TestRawBSONDocument(unittest.TestCase):
     def test_raw(self):
         self.assertEqual(self.bson_string, self.document.raw)
 
+    def test_empty_doc(self):
+        doc = RawBSONDocument(encode({}))
+        with self.assertRaises(KeyError):
+            doc['does-not-exist']
+
+    def test_invalid_bson_sequence(self):
+        bson_byte_sequence = encode({'a': 1})+encode({})
+        with self.assertRaisesRegex(InvalidBSON, 'invalid object length'):
+            RawBSONDocument(bson_byte_sequence)
+
+    def test_invalid_bson_eoo(self):
+        invalid_bson_eoo = encode({'a': 1})[:-1] + b'\x01'
+        with self.assertRaisesRegex(InvalidBSON, 'bad eoo'):
+            RawBSONDocument(invalid_bson_eoo)
+
     @client_context.require_connection
     def test_round_trip(self):
         db = self.client.get_database(
@@ -61,6 +80,21 @@ class TestRawBSONDocument(unittest.TestCase):
         self.assertIsInstance(result, RawBSONDocument)
         self.assertEqual(dict(self.document.items()), dict(result.items()))
 
+    @client_context.require_connection
+    def test_round_trip_raw_uuid(self):
+        coll = self.client.get_database('pymongo_test').test_raw
+        uid = uuid.uuid4()
+        doc = {'_id': 1,
+               'bin4': Binary(uid.bytes, 4),
+               'bin3': Binary(uid.bytes, 3)}
+        raw = RawBSONDocument(encode(doc))
+        coll.insert_one(raw)
+        self.assertEqual(coll.find_one(), {'_id': 1, 'bin4': uid, 'bin3': uid})
+
+        # Test that the raw bytes haven't changed.
+        raw_coll = coll.with_options(codec_options=DEFAULT_RAW_BSON_OPTIONS)
+        self.assertEqual(raw_coll.find_one(), raw)
+
     def test_with_codec_options(self):
         # {u'date': datetime.datetime(2015, 6, 3, 18, 40, 50, 826000),
         #  u'_id': UUID('026fab8f-975f-4965-9fbf-85ad874c60ff')}
@@ -93,7 +127,7 @@ class TestRawBSONDocument(unittest.TestCase):
                                        document_class=RawBSONDocument)
         coll = db.get_collection('test_raw', codec_options=raw_java_legacy)
         self.assertEqual(
-            RawBSONDocument(BSON.encode(doc, codec_options=raw_java_legacy)),
+            RawBSONDocument(encode(doc, codec_options=raw_java_legacy)),
             coll.find_one())
 
     @client_context.require_connection
@@ -102,7 +136,7 @@ class TestRawBSONDocument(unittest.TestCase):
         db = self.client.pymongo_test
         db.test_raw.insert_one(doc)
         result = db.test_raw.find_one()
-        self.assertEqual(BSON(self.document.raw).decode(), result['embedded'])
+        self.assertEqual(decode(self.document.raw), result['embedded'])
 
         # Make sure that CodecOptions are preserved.
         # {'embedded': [
@@ -141,3 +175,10 @@ class TestRawBSONDocument(unittest.TestCase):
         coll.delete_many(self.document)
         coll.update_one(self.document, {'$set': {'a': 'b'}}, upsert=True)
         coll.update_many(self.document, {'$set': {'b': 'c'}})
+
+    def test_preserve_key_ordering(self):
+        keyvaluepairs = [('a', 1), ('b', 2), ('c', 3),]
+        rawdoc = RawBSONDocument(encode(SON(keyvaluepairs)))
+
+        for rkey, elt in zip(rawdoc, keyvaluepairs):
+            self.assertEqual(rkey, elt[0])
