@@ -30,21 +30,23 @@ from pymongo.mongo_client import MongoClient
 from pymongo.operations import IndexModel, InsertOne
 from pymongo.read_concern import ReadConcern
 from pymongo.write_concern import WriteConcern
-from test import client_context, unittest
-from test.utils import (IMPOSSIBLE_WRITE_CONCERN,
-                        EventListener,
+from test import (client_context,
+                  IntegrationTest,
+                  unittest)
+from test.utils import (EventListener,
                         disable_replication,
                         enable_replication,
-                        rs_or_single_client)
+                        rs_or_single_client,
+                        TestCreator)
+from test.utils_spec_runner import SpecRunner
 
 
 _TEST_PATH = os.path.join(
     os.path.dirname(os.path.realpath(__file__)), 'read_write_concern')
 
 
-class TestReadWriteConcernSpec(unittest.TestCase):
+class TestReadWriteConcernSpec(IntegrationTest):
 
-    @client_context.require_connection
     def test_omit_default_read_write_concern(self):
         listener = EventListener()
         # Client with default readConcern and writeConcern
@@ -98,7 +100,10 @@ class TestReadWriteConcernSpec(unittest.TestCase):
                         name, event.command_name))
 
     def assertWriteOpsRaise(self, write_concern, expected_exception):
-        client = rs_or_single_client(**write_concern.document)
+        wc = write_concern.document
+        # Set socket timeout to avoid indefinite stalls
+        client = rs_or_single_client(
+            w=wc['w'], wTimeoutMS=wc['wtimeout'], socketTimeoutMS=30000)
         db = client.get_database('pymongo_test')
         coll = db.test
 
@@ -120,6 +125,9 @@ class TestReadWriteConcernSpec(unittest.TestCase):
         ]
         ops_require_34 = [
             ('aggregate', lambda: coll.aggregate([{'$out': 'out'}])),
+            # SERVER-46668 Delete all the documents in the collection to
+            # workaround a hang in createIndexes.
+            ('delete_many', lambda: coll.delete_many({})),
             ('create_index', lambda: coll.create_index([('a', DESCENDING)])),
             ('create_indexes', lambda: coll.create_indexes([IndexModel('b')])),
             ('drop_index', lambda: coll.drop_index([('a', DESCENDING)])),
@@ -129,8 +137,8 @@ class TestReadWriteConcernSpec(unittest.TestCase):
         ]
         if client_context.version > (3, 4):
             ops.extend(ops_require_34)
-            # SERVER-34776: Drop database does not respect wtimeout in 4.0.
-            if client_context.version <= (3, 6):
+            # SERVER-47194: dropDatabase does not respect wtimeout in 3.6.
+            if client_context.version[:2] != (3, 6):
                 ops.append(('drop_database', lambda: client.drop_database(db)))
 
         for name, f in ops:
@@ -164,6 +172,44 @@ class TestReadWriteConcernSpec(unittest.TestCase):
         self.assertWriteOpsRaise(WriteConcern(w=client_context.w, wtimeout=1),
                                  WTimeoutError)
 
+    @client_context.require_failCommand_fail_point
+    def test_error_includes_errInfo(self):
+        expected_wce = {
+            "code": 100,
+            "codeName": "UnsatisfiableWriteConcern",
+            "errmsg": "Not enough data-bearing nodes",
+            "errInfo":  {
+                "writeConcern": {
+                    "w": 2,
+                    "wtimeout": 0,
+                    "provenance": "clientSupplied"
+                }
+            }
+        }
+        cause_wce = {
+            "configureFailPoint": "failCommand",
+            "mode": {"times": 2},
+            "data": {
+                "failCommands": ["insert"],
+                "writeConcernError": expected_wce
+            },
+        }
+        with self.fail_point(cause_wce):
+            # Write concern error on insert includes errInfo.
+            with self.assertRaises(WriteConcernError) as ctx:
+                self.db.test.insert_one({})
+            self.assertEqual(ctx.exception.details, expected_wce)
+
+            # Test bulk_write as well.
+            with self.assertRaises(BulkWriteError) as ctx:
+                self.db.test.bulk_write([InsertOne({})])
+            expected_details = {
+                'writeErrors': [],
+                'writeConcernErrors': [expected_wce],
+                'nInserted': 1, 'nUpserted': 0, 'nMatched': 0, 'nModified': 0,
+                'nRemoved': 0, 'upserted': []}
+            self.assertEqual(ctx.exception.details, expected_details)
+
 
 def normalize_write_concern(concern):
     result = {}
@@ -247,7 +293,10 @@ def create_tests():
     for dirpath, _, filenames in os.walk(_TEST_PATH):
         dirname = os.path.split(dirpath)[-1]
 
-        if dirname == 'connection-string':
+        if dirname == 'operation':
+            # This directory is tested by TestOperations.
+            continue
+        elif dirname == 'connection-string':
             create_test = create_connection_string_test
         else:
             create_test = create_document_test
@@ -271,5 +320,27 @@ def create_tests():
 create_tests()
 
 
+class TestOperation(SpecRunner):
+    # Location of JSON test specifications.
+    TEST_PATH = os.path.join(_TEST_PATH, 'operation')
+
+    def get_outcome_coll_name(self, outcome, collection):
+        """Spec says outcome has an optional 'collection.name'."""
+        return outcome['collection'].get('name', collection.name)
+
+
+def create_operation_test(scenario_def, test, name):
+    @client_context.require_test_commands
+    def run_scenario(self):
+        self.run_scenario(scenario_def, test)
+
+    return run_scenario
+
+
+test_creator = TestCreator(
+    create_operation_test, TestOperation, TestOperation.TEST_PATH)
+test_creator.create_tests()
+
+
 if __name__ == '__main__':
     unittest.main()
