@@ -166,7 +166,7 @@ class TestReplicaSetClient(TestReplicaSetClientBase):
 
         # Disable background refresh.
         with client_knobs(heartbeat_frequency=999999):
-            c = rs_client(socketTimeoutMS=3000, w=self.w)
+            c = rs_client(socketTimeoutMS=1000, w=self.w)
             collection = c.pymongo_test.test
             collection.insert_one({})
 
@@ -174,7 +174,7 @@ class TestReplicaSetClient(TestReplicaSetClientBase):
             self.assertRaises(
                 NetworkTimeout,
                 collection.find_one,
-                {'$where': delay(5)})
+                {'$where': delay(1.5)})
 
             self.assertTrue(c.primary)
             collection.find_one()  # No error.
@@ -186,7 +186,7 @@ class TestReplicaSetClient(TestReplicaSetClientBase):
             self.assertRaises(
                 NetworkTimeout,
                 coll.find_one,
-                {'$where': delay(5)})
+                {'$where': delay(1.5)})
 
             self.assertTrue(c.secondaries)
 
@@ -195,7 +195,7 @@ class TestReplicaSetClient(TestReplicaSetClientBase):
 
     @client_context.require_ipv6
     def test_ipv6(self):
-        if client_context.ssl:
+        if client_context.tls:
             if not HAVE_IPADDRESS:
                 raise SkipTest("Need the ipaddress module to test with SSL")
 
@@ -300,6 +300,7 @@ class TestReplicaSetWireVersion(MockClientTest):
             host='a:1',
             replicaSet='rs',
             connect=False)
+        self.addCleanup(c.close)
 
         c.set_wire_version_range('a:1', 3, 7)
         c.set_wire_version_range('b:2', 2, 3)
@@ -330,18 +331,19 @@ class TestReplicaSetClientInternalIPs(MockClientTest):
     def test_connect_with_internal_ips(self):
         # Client is passed an IP it can reach, 'a:1', but the RS config
         # only contains unreachable IPs like 'internal-ip'. PYTHON-608.
+        client = MockClient(
+            standalones=[],
+            members=['a:1'],
+            mongoses=[],
+            ismaster_hosts=['internal-ip:27017'],
+            host='a:1',
+            replicaSet='rs',
+            serverSelectionTimeoutMS=100)
+        self.addCleanup(client.close)
         with self.assertRaises(AutoReconnect) as context:
-            connected(MockClient(
-                standalones=[],
-                members=['a:1'],
-                mongoses=[],
-                ismaster_hosts=['internal-ip:27017'],
-                host='a:1',
-                replicaSet='rs',
-                serverSelectionTimeoutMS=100))
+            connected(client)
 
-        self.assertEqual(
-            "Could not reach any servers in [('internal-ip', 27017)]."
+        self.assertIn("Could not reach any servers in [('internal-ip', 27017)]."
             " Replica set is configured with internal hostnames or IPs?",
             str(context.exception))
 
@@ -356,6 +358,7 @@ class TestReplicaSetClientMaxWriteBatchSize(MockClientTest):
             host='a:1',
             replicaSet='rs',
             connect=False)
+        self.addCleanup(c.close)
 
         c.set_max_write_batch_size('a:1', 1)
         c.set_max_write_batch_size('b:2', 2)
