@@ -17,22 +17,24 @@
 import json
 import os
 import sys
-import weakref
+import time
 
 sys.path[0:0] = [""]
 
 from bson.json_util import object_hook
 from pymongo import monitoring
-from pymongo import periodic_executor
+from pymongo.common import clean_node
+from pymongo.errors import (ConnectionFailure,
+                            NotMasterError)
 from pymongo.ismaster import IsMaster
 from pymongo.monitor import Monitor
-from pymongo.read_preferences import MovingAverage
 from pymongo.server_description import ServerDescription
-from pymongo.server_type import SERVER_TYPE
 from pymongo.topology_description import TOPOLOGY_TYPE
-from test import unittest, client_context, client_knobs
+from test import unittest, client_context, client_knobs, IntegrationTest
 from test.utils import (ServerAndTopologyEventListener,
                         single_client,
+                        server_name_to_type,
+                        rs_or_single_client,
                         wait_until)
 
 # Location of JSON test specifications.
@@ -43,7 +45,7 @@ _TEST_PATH = os.path.join(
 
 def compare_server_descriptions(expected, actual):
     if ((not expected['address'] == "%s:%s" % actual.address) or
-            (not SERVER_TYPE.__getattribute__(expected['type']) ==
+            (not server_name_to_type(expected['type']) ==
                 actual.server_type)):
         return False
     expected_hosts = set(
@@ -176,90 +178,62 @@ class TestAllScenarios(unittest.TestCase):
 
 def create_test(scenario_def):
     def run_scenario(self):
-        responses = (r for r in scenario_def['phases'][0]['responses'])
-
         with client_knobs(events_queue_frequency=0.1):
-            class MockMonitor(Monitor):
-                def __init__(self, server_description, topology, pool,
-                             topology_settings):
-                    """Have to copy entire constructor from Monitor so that we
-                    can override _run and change the periodic executor's
-                     interval."""
+            _run_scenario(self)
 
-                    self._server_description = server_description
-                    self._pool = pool
-                    self._settings = topology_settings
-                    self._avg_round_trip_time = MovingAverage()
-                    options = self._settings._pool_options
-                    self._listeners = options.event_listeners
-                    self._publish = self._listeners is not None
+    def _run_scenario(self):
+        class NoopMonitor(Monitor):
+            """Override the _run method to do nothing."""
+            def _run(self):
+                time.sleep(0.05)
 
-                    def target():
-                        monitor = self_ref()
-                        if monitor is None:
-                            return False
-                        MockMonitor._run(monitor)  # Change target to subclass
-                        return True
-
-                    # Shorten interval
-                    executor = periodic_executor.PeriodicExecutor(
-                        interval=0.1,
-                        min_interval=0.1,
-                        target=target,
-                        name="pymongo_server_monitor_thread")
-                    self._executor = executor
-                    self_ref = weakref.ref(self, executor.close)
-                    self._topology = weakref.proxy(topology, executor.close)
-
-                def _run(self):
-                    try:
-                        if self._server_description.address != ('a', 27017):
-                            # Because PyMongo doesn't keep information about
-                            # the order of addresses, we might accidentally
-                            # start a MockMonitor on the wrong server first,
-                            # so we need to only mock responses for the server
-                            # the test's response is supposed to come from.
-                            return
-                        response = next(responses)[1]
-                        isMaster = IsMaster(response)
-                        self._server_description = ServerDescription(
-                            address=self._server_description.address,
-                            ismaster=isMaster)
-                        self._topology.on_change(self._server_description)
-                    except (ReferenceError, StopIteration):
-                        # Topology was garbage-collected.
-                        self.close()
-
-            m = single_client(h=scenario_def['uri'], p=27017,
-                              event_listeners=(self.all_listener,),
-                              _monitor_class=MockMonitor)
-
-            expected_results = scenario_def['phases'][0]['outcome']['events']
-
-            expected_len = len(expected_results)
-            wait_until(lambda: len(self.all_listener.results) >= expected_len,
-                       "publish all events", timeout=15)
+        m = single_client(h=scenario_def['uri'], p=27017,
+                          event_listeners=[self.all_listener],
+                          _monitor_class=NoopMonitor)
+        topology = m._get_topology()
 
         try:
-            i = 0
-            while i < expected_len:
-                result = self.all_listener.results[i] if len(
-                    self.all_listener.results) > i else None
-                # The order of ServerOpening/ClosedEvents doesn't matter
-                if (isinstance(result,
-                               monitoring.ServerOpeningEvent) or
-                        isinstance(result,
-                                   monitoring.ServerClosedEvent)):
-                    i, passed, message = compare_multiple_events(
-                        i, expected_results, self.all_listener.results)
-                    self.assertTrue(passed, message)
-                else:
-                    self.assertTrue(
-                        *compare_events(expected_results[i], result))
-                    i += 1
+            for phase in scenario_def['phases']:
+                for (source, response) in phase['responses']:
+                    source_address = clean_node(source)
+                    topology.on_change(ServerDescription(
+                        address=source_address,
+                        ismaster=IsMaster(response),
+                        round_trip_time=0))
 
+                expected_results = phase['outcome']['events']
+                expected_len = len(expected_results)
+                wait_until(
+                    lambda: len(self.all_listener.results) >= expected_len,
+                    "publish all events", timeout=15)
+
+                # Wait some time to catch possible lagging extra events.
+                time.sleep(0.5)
+
+                i = 0
+                while i < expected_len:
+                    result = self.all_listener.results[i] if len(
+                        self.all_listener.results) > i else None
+                    # The order of ServerOpening/ClosedEvents doesn't matter
+                    if isinstance(result, (monitoring.ServerOpeningEvent,
+                                           monitoring.ServerClosedEvent)):
+                        i, passed, message = compare_multiple_events(
+                            i, expected_results, self.all_listener.results)
+                        self.assertTrue(passed, message)
+                    else:
+                        self.assertTrue(
+                            *compare_events(expected_results[i], result))
+                        i += 1
+
+                # Assert no extra events.
+                extra_events = self.all_listener.results[expected_len:]
+                if extra_events:
+                    self.fail('Extra events %r' % (extra_events,))
+
+                self.all_listener.reset()
         finally:
             m.close()
+
     return run_scenario
 
 
@@ -278,5 +252,94 @@ def create_tests():
 
 create_tests()
 
+
+class TestSdamMonitoring(IntegrationTest):
+
+    @classmethod
+    @client_context.require_failCommand_fail_point
+    def setUpClass(cls):
+        super(TestSdamMonitoring, cls).setUpClass()
+        # Speed up the tests by decreasing the event publish frequency.
+        cls.knobs = client_knobs(events_queue_frequency=0.1)
+        cls.knobs.enable()
+        cls.listener = ServerAndTopologyEventListener()
+        retry_writes = client_context.supports_transactions()
+        cls.test_client = rs_or_single_client(
+            event_listeners=[cls.listener], retryWrites=retry_writes)
+        cls.coll = cls.test_client[cls.client.db.name].test
+        cls.coll.insert_one({})
+
+    @classmethod
+    def tearDownClass(cls):
+        cls.test_client.close()
+        cls.knobs.disable()
+        super(TestSdamMonitoring, cls).tearDownClass()
+
+    def setUp(self):
+        self.listener.reset()
+
+    def _test_app_error(self, fail_command_opts, expected_error):
+        address = self.test_client.address
+
+        # Test that an application error causes a ServerDescriptionChangedEvent
+        # to be published.
+        data = {'failCommands': ['insert']}
+        data.update(fail_command_opts)
+        fail_insert = {
+            'configureFailPoint': 'failCommand',
+            'mode': {'times': 1},
+            'data': data,
+        }
+        with self.fail_point(fail_insert):
+            if self.test_client.retry_writes:
+                self.coll.insert_one({})
+            else:
+                with self.assertRaises(expected_error):
+                    self.coll.insert_one({})
+                self.coll.insert_one({})
+
+        def marked_unknown(event):
+            return (
+                isinstance(event, monitoring.ServerDescriptionChangedEvent)
+                and event.server_address == address
+                and not event.new_description.is_server_type_known)
+
+        def discovered_node(event):
+            return (
+                isinstance(event, monitoring.ServerDescriptionChangedEvent)
+                and event.server_address == address
+                and not event.previous_description.is_server_type_known
+                and event.new_description.is_server_type_known)
+
+        def marked_unknown_and_rediscovered():
+            return (len(self.listener.matching(marked_unknown)) >= 1 and
+                    len(self.listener.matching(discovered_node)) >= 1)
+
+        # Topology events are published asynchronously
+        wait_until(marked_unknown_and_rediscovered, 'rediscover node')
+
+        # Expect a single ServerDescriptionChangedEvent for the network error.
+        marked_unknown_events = self.listener.matching(marked_unknown)
+        self.assertEqual(len(marked_unknown_events), 1, marked_unknown_events)
+        self.assertIsInstance(
+            marked_unknown_events[0].new_description.error, expected_error)
+
+    def test_network_error_publishes_events(self):
+        self._test_app_error({'closeConnection': True}, ConnectionFailure)
+
+    # In 4.4+, NotMaster errors from failCommand don't cause SDAM state
+    # changes because topologyVersion is not incremented.
+    @client_context.require_version_max(4, 3)
+    def test_not_master_error_publishes_events(self):
+        self._test_app_error({'errorCode': 10107, 'closeConnection': False,
+                              'errorLabels': ['RetryableWriteError']},
+                             NotMasterError)
+
+    def test_shutdown_error_publishes_events(self):
+        self._test_app_error({'errorCode': 91, 'closeConnection': False,
+                              'errorLabels': ['RetryableWriteError']},
+                             NotMasterError)
+
+
 if __name__ == "__main__":
     unittest.main()
