@@ -32,7 +32,8 @@ from test import IntegrationTest, client_context, db_user, db_pwd, unittest, Ski
 from test.utils import (ignore_deprecations,
                         rs_or_single_client,
                         EventListener,
-                        TestCreator)
+                        TestCreator,
+                        wait_until)
 from test.utils_spec_runner import SpecRunner
 
 # Ignore auth commands like saslStart, so we can assert lsid is in all commands.
@@ -77,6 +78,7 @@ class TestSession(IntegrationTest):
     @classmethod
     def tearDownClass(cls):
         monitoring._SENSITIVE_COMMANDS.update(cls.sensitive_commands)
+        cls.client2.close()
         super(TestSession, cls).tearDownClass()
 
     def setUp(self):
@@ -84,6 +86,7 @@ class TestSession(IntegrationTest):
         self.session_checker_listener = SessionTestListener()
         self.client = rs_or_single_client(
             event_listeners=[self.listener, self.session_checker_listener])
+        self.addCleanup(self.client.close)
         self.db = self.client.pymongo_test
         self.initial_lsids = set(s['id'] for s in session_ids(self.client))
 
@@ -224,6 +227,7 @@ class TestSession(IntegrationTest):
         client.close()
         self.assertEqual(len(listener.results['started']), 0)
 
+    @ignore_deprecations  # fsync and unlock
     def test_client(self):
         client = self.client
 
@@ -751,6 +755,7 @@ class TestSession(IntegrationTest):
         # Ensure the collection exists.
         self.client.pymongo_test.test_unacked_writes.insert_one({})
         client = rs_or_single_client(w=0, event_listeners=[self.listener])
+        self.addCleanup(client.close)
         db = client.pymongo_test
         coll = db.test_unacked_writes
         ops = [
@@ -761,6 +766,19 @@ class TestSession(IntegrationTest):
         ops.extend(self.collection_write_ops(coll))
         self._test_unacknowledged_ops(client, *ops)
 
+        def drop_db():
+            try:
+                self.client.drop_database(db.name)
+                return True
+            except OperationFailure as exc:
+                # Try again on BackgroundOperationInProgressForDatabase and
+                # BackgroundOperationInProgressForNamespace.
+                if exc.code in (12586, 12587):
+                    return False
+                raise
+
+        wait_until(drop_db, 'dropped database after w=0 writes')
+
 
 class TestCausalConsistency(unittest.TestCase):
 
@@ -769,6 +787,10 @@ class TestCausalConsistency(unittest.TestCase):
         cls.listener = SessionTestListener()
         cls.client = rs_or_single_client(event_listeners=[cls.listener])
 
+    @classmethod
+    def tearDownClass(cls):
+        cls.client.close()
+
     @client_context.require_sessions
     def setUp(self):
         super(TestCausalConsistency, self).setUp()
@@ -1227,6 +1249,7 @@ class TestClusterTime(IntegrationTest):
         # Prevent heartbeats from updating $clusterTime between operations.
         client = rs_or_single_client(event_listeners=[listener],
                                      heartbeatFrequencyMS=999999)
+        self.addCleanup(client.close)
         collection = client.pymongo_test.collection
         # Prepare for tests of find() and aggregate().
         collection.insert_many([{} for _ in range(10)])
