@@ -30,7 +30,7 @@ from pymongo import MongoClient, ssl_support
 from pymongo.errors import (ConfigurationError,
                             ConnectionFailure,
                             OperationFailure)
-from pymongo.ssl_support import HAVE_SSL, get_ssl_context, validate_cert_reqs
+from pymongo.ssl_support import HAVE_SSL, get_ssl_context, validate_cert_reqs, _ssl
 from pymongo.write_concern import WriteConcern
 from test import (IntegrationTest,
                   client_context,
@@ -39,7 +39,26 @@ from test import (IntegrationTest,
                   SkipTest,
                   unittest,
                   HAVE_IPADDRESS)
-from test.utils import remove_all_users, connected
+from test.utils import (EventListener,
+                        cat_files,
+                        connected,
+                        remove_all_users)
+
+
+_HAVE_PYOPENSSL = False
+try:
+    # All of these must be available to use PyOpenSSL
+    import OpenSSL
+    import requests
+    import service_identity
+    _HAVE_PYOPENSSL = True
+except ImportError:
+    pass
+
+if _HAVE_PYOPENSSL:
+    from pymongo.ocsp_support import _load_trusted_ca_certs
+else:
+    _load_trusted_ca_certs = None
 
 if HAVE_SSL:
     import ssl
@@ -49,6 +68,7 @@ CERT_PATH = os.path.join(os.path.dirname(os.path.realpath(__file__)),
 CLIENT_PEM = os.path.join(CERT_PATH, 'client.pem')
 CLIENT_ENCRYPTED_PEM = os.path.join(CERT_PATH, 'password_protected.pem')
 CA_PEM = os.path.join(CERT_PATH, 'ca.pem')
+CA_BUNDLE_PEM = os.path.join(CERT_PATH, 'trusted-ca.pem')
 CRL_PEM = os.path.join(CERT_PATH, 'crl.pem')
 MONGODB_X509_USERNAME = (
     "C=US,ST=New York,L=New York City,O=MDB,OU=Drivers,CN=client")
@@ -143,6 +163,15 @@ class TestClientSSL(unittest.TestCase):
             validate_cert_reqs('ssl_cert_reqs', 'CERT_REQUIRED'),
             ssl.CERT_REQUIRED)
 
+    @unittest.skipUnless(_HAVE_PYOPENSSL, "PyOpenSSL is not available.")
+    def test_use_openssl_when_available(self):
+        self.assertTrue(_ssl.IS_PYOPENSSL)
+
+    @unittest.skipUnless(_HAVE_PYOPENSSL, "Cannot test without PyOpenSSL")
+    def test_load_trusted_ca_certs(self):
+        trusted_ca_certs = _load_trusted_ca_certs(CA_BUNDLE_PEM)
+        self.assertEqual(2, len(trusted_ca_certs))
+
 
 class TestSSL(IntegrationTest):
 
@@ -167,7 +196,7 @@ class TestSSL(IntegrationTest):
         MongoClient.PORT = cls.saved_port
         super(TestSSL, cls).tearDownClass()
 
-    @client_context.require_ssl
+    @client_context.require_tls
     def test_simple_ssl(self):
         # Expects the server to be running with ssl and with
         # no --sslPEMKeyFile or with --sslWeakCertificateValidation
@@ -179,8 +208,7 @@ class TestSSL(IntegrationTest):
         #
         #   --sslPEMKeyFile=/path/to/pymongo/test/certificates/server.pem
         #   --sslCAFile=/path/to/pymongo/test/certificates/ca.pem
-        vi = sys.version_info
-        if vi[0] == 2 and vi < (2, 7, 9) or vi[0] == 3 and vi < (3, 3):
+        if not hasattr(ssl, 'SSLContext') and not _ssl.IS_PYOPENSSL:
             self.assertRaises(
                 ConfigurationError,
                 MongoClient,
@@ -319,17 +347,17 @@ class TestSSL(IntegrationTest):
         # Python > 2.7.9. If SSLContext doesn't have load_default_certs
         # it also doesn't have check_hostname.
         ctx = get_ssl_context(
-            None, None, None, None, ssl.CERT_NONE, None, False)
+            None, None, None, None, ssl.CERT_NONE, None, False, True)
         if hasattr(ctx, 'load_default_certs'):
             self.assertFalse(ctx.check_hostname)
             ctx = get_ssl_context(
-                None, None, None, None, ssl.CERT_NONE, None, True)
+                None, None, None, None, ssl.CERT_NONE, None, True, True)
             self.assertFalse(ctx.check_hostname)
             ctx = get_ssl_context(
-                None, None, None, None, ssl.CERT_REQUIRED, None, False)
+                None, None, None, None, ssl.CERT_REQUIRED, None, False, True)
             self.assertFalse(ctx.check_hostname)
             ctx = get_ssl_context(
-                None, None, None, None, ssl.CERT_REQUIRED, None, True)
+                None, None, None, None, ssl.CERT_REQUIRED, None, True, True)
             if _PY37PLUS:
                 self.assertTrue(ctx.check_hostname)
             else:
@@ -378,7 +406,7 @@ class TestSSL(IntegrationTest):
 
     @client_context.require_ssl_certfile
     def test_ssl_crlfile_support(self):
-        if not hasattr(ssl, 'VERIFY_CRL_CHECK_LEAF'):
+        if not hasattr(ssl, 'VERIFY_CRL_CHECK_LEAF') or _ssl.IS_PYOPENSSL:
             self.assertRaises(
                 ConfigurationError,
                 MongoClient,
@@ -428,6 +456,12 @@ class TestSSL(IntegrationTest):
         if sys.version_info < (2, 7, 9):
             raise SkipTest("Can't load system CA certificates.")
 
+        if (ssl.OPENSSL_VERSION.lower().startswith('libressl') and
+                sys.platform == 'darwin' and not _ssl.IS_PYOPENSSL):
+            raise SkipTest(
+                "LibreSSL on OSX doesn't support setting CA certificates "
+                "using SSL_CERT_FILE environment variable.")
+
         # Tell OpenSSL where CA certificates live.
         os.environ['SSL_CERT_FILE'] = CA_PEM
         try:
@@ -461,7 +495,7 @@ class TestSSL(IntegrationTest):
 
     def test_system_certs_config_error(self):
         ctx = get_ssl_context(
-            None, None, None, None, ssl.CERT_NONE, None, False)
+            None, None, None, None, ssl.CERT_NONE, None, False, True)
         if ((sys.platform != "win32"
              and hasattr(ctx, "set_default_verify_paths"))
                 or hasattr(ctx, "load_default_certs")):
@@ -494,11 +528,11 @@ class TestSSL(IntegrationTest):
         ssl_support.HAVE_WINCERTSTORE = False
         try:
             ctx = get_ssl_context(
-                None, None, None, CA_PEM, ssl.CERT_REQUIRED, None, True)
+                None, None, None, CA_PEM, ssl.CERT_REQUIRED, None, True, True)
             ssl_sock = ctx.wrap_socket(socket.socket())
             self.assertEqual(ssl_sock.ca_certs, CA_PEM)
 
-            ctx = get_ssl_context(None, None, None, None, None, None, True)
+            ctx = get_ssl_context(None, None, None, None, None, None, True, True)
             ssl_sock = ctx.wrap_socket(socket.socket())
             self.assertEqual(ssl_sock.ca_certs, ssl_support.certifi.where())
         finally:
@@ -516,11 +550,11 @@ class TestSSL(IntegrationTest):
             raise SkipTest("Need wincertstore to test wincertstore.")
 
         ctx = get_ssl_context(
-            None, None, None, CA_PEM, ssl.CERT_REQUIRED, None, True)
+            None, None, None, CA_PEM, ssl.CERT_REQUIRED, None, True, True)
         ssl_sock = ctx.wrap_socket(socket.socket())
         self.assertEqual(ssl_sock.ca_certs, CA_PEM)
 
-        ctx = get_ssl_context(None, None, None, None, None, None, True)
+        ctx = get_ssl_context(None, None, None, None, None, None, True, True)
         ssl_sock = ctx.wrap_socket(socket.socket())
         self.assertEqual(ssl_sock.ca_certs, ssl_support._WINCERTS.name)
 
@@ -550,16 +584,24 @@ class TestSSL(IntegrationTest):
 
         self.assertRaises(OperationFailure, noauth.pymongo_test.test.count)
 
+        listener = EventListener()
         auth = MongoClient(
             client_context.pair,
             authMechanism='MONGODB-X509',
             ssl=True,
             ssl_cert_reqs=ssl.CERT_NONE,
-            ssl_certfile=CLIENT_PEM)
+            ssl_certfile=CLIENT_PEM,
+            event_listeners=[listener])
 
         if client_context.version.at_least(3, 3, 12):
             # No error
             auth.pymongo_test.test.find_one()
+            names = listener.started_command_names()
+            if client_context.version.at_least(4, 4, -1):
+                # Speculative auth skips the authenticate command.
+                self.assertEqual(names, ['find'])
+            else:
+                self.assertEqual(names, ['authenticate', 'find'])
         else:
             # Should require a username
             with self.assertRaises(ConfigurationError):
@@ -625,6 +667,24 @@ class TestSSL(IntegrationTest):
         else:
             self.fail("Invalid certificate accepted.")
 
+    @client_context.require_ssl_certfile
+    def test_connect_with_ca_bundle(self):
+        def remove(path):
+            try:
+                os.remove(path)
+            except OSError:
+                pass
+
+        temp_ca_bundle = os.path.join(CERT_PATH, 'trusted-ca-bundle.pem')
+        self.addCleanup(remove, temp_ca_bundle)
+        # Add the CA cert file to the bundle.
+        cat_files(temp_ca_bundle, CA_BUNDLE_PEM, CA_PEM)
+        with MongoClient('localhost',
+                         tls=True,
+                         tlsCertificateKeyFile=CLIENT_PEM,
+                         tlsCAFile=temp_ca_bundle) as client:
+            self.assertTrue(client.admin.command('ismaster'))
+
 
 if __name__ == "__main__":
     unittest.main()
