@@ -18,13 +18,12 @@ import sys
 
 sys.path[0:0] = [""]
 
-import threading
-
 from bson.py3compat import imap
 from pymongo import common
 from pymongo.read_preferences import ReadPreference, Secondary
 from pymongo.server_type import SERVER_TYPE
-from pymongo.topology import Topology
+from pymongo.topology import (_ErrorContext,
+                              Topology)
 from pymongo.topology_description import TOPOLOGY_TYPE
 from pymongo.errors import (AutoReconnect,
                             ConfigurationError,
@@ -37,46 +36,17 @@ from pymongo.server_selectors import (any_server_selector,
                                       writable_server_selector)
 from pymongo.settings import TopologySettings
 from test import client_knobs, unittest
-from test.utils import wait_until
-
-
-class MockSocketInfo(object):
-    def close(self):
-        pass
-
-    def __enter__(self):
-        return self
-
-    def __exit__(self, exc_type, exc_val, exc_tb):
-        pass
-
-
-class MockPool(object):
-    def __init__(self, *args, **kwargs):
-        self.pool_id = 0
-        self._lock = threading.Lock()
-        self.opts = PoolOptions()
-
-    def get_socket(self, all_credentials):
-        return MockSocketInfo()
-
-    def return_socket(self, _):
-        pass
-
-    def reset(self):
-        with self._lock:
-            self.pool_id += 1
-
-    def remove_stale_sockets(self):
-        pass
+from test.utils import MockPool, wait_until
 
 
 class MockMonitor(object):
     def __init__(self, server_description, topology, pool, topology_settings):
         self._server_description = server_description
-        self._topology = topology
         self.opened = False
 
+    def cancel_check(self):
+        pass
+
     def open(self):
         self.opened = True
 
@@ -145,12 +115,6 @@ class TopologyTest(unittest.TestCase):
         self.addCleanup(self.client_knobs.disable)
 
 
-# Use assertRaisesRegex if available, otherwise use Python 2.7's
-# deprecated assertRaisesRegexp, with a 'p'.
-if not hasattr(unittest.TestCase, 'assertRaisesRegex'):
-    TopologyTest.assertRaisesRegex = unittest.TestCase.assertRaisesRegexp
-
-
 class TestTopologyConfiguration(TopologyTest):
     def test_timeout_configuration(self):
         pool_options = PoolOptions(connect_timeout=1, socket_timeout=2)
@@ -270,6 +234,7 @@ class TestSingleServerTopology(TopologyTest):
                     raise AutoReconnect('mock monitor error')
 
         t = create_mock_topology(monitor_class=TestMonitor)
+        self.addCleanup(t.close)
         s = t.select_server(writable_server_selector)
         self.assertEqual(125, s.description.round_trip_time)
 
@@ -439,7 +404,7 @@ class TestMultiServerTopology(TopologyTest):
         self.assertEqual(TOPOLOGY_TYPE.ReplicaSetNoPrimary,
                          t.description.topology_type)
 
-    def test_reset_server(self):
+    def test_handle_error(self):
         t = create_mock_topology(replica_set_name='rs')
         got_ismaster(t, ('a', 27017), {
             'ok': 1,
@@ -454,7 +419,8 @@ class TestMultiServerTopology(TopologyTest):
             'setName': 'rs',
             'hosts': ['a', 'b']})
 
-        t.reset_server(('a', 27017))
+        errctx = _ErrorContext(AutoReconnect('mock'), 0, 0, True)
+        t.handle_error(('a', 27017), errctx)
         self.assertEqual(SERVER_TYPE.Unknown, get_type(t, 'a'))
         self.assertEqual(SERVER_TYPE.RSSecondary, get_type(t, 'b'))
         self.assertEqual('rs', t.description.replica_set_name)
@@ -471,18 +437,60 @@ class TestMultiServerTopology(TopologyTest):
         self.assertEqual(TOPOLOGY_TYPE.ReplicaSetWithPrimary,
                          t.description.topology_type)
 
-        t.reset_server(('b', 27017))
+        t.handle_error(('b', 27017), errctx)
         self.assertEqual(SERVER_TYPE.RSPrimary, get_type(t, 'a'))
         self.assertEqual(SERVER_TYPE.Unknown, get_type(t, 'b'))
         self.assertEqual('rs', t.description.replica_set_name)
         self.assertEqual(TOPOLOGY_TYPE.ReplicaSetWithPrimary,
                          t.description.topology_type)
 
-    def test_reset_removed_server(self):
+    def test_handle_getlasterror(self):
+        t = create_mock_topology(replica_set_name='rs')
+        got_ismaster(t, ('a', 27017), {
+            'ok': 1,
+            'ismaster': True,
+            'setName': 'rs',
+            'hosts': ['a', 'b']})
+
+        got_ismaster(t, ('b', 27017), {
+            'ok': 1,
+            'ismaster': False,
+            'secondary': True,
+            'setName': 'rs',
+            'hosts': ['a', 'b']})
+
+        t.handle_getlasterror(('a', 27017), 'not master')
+        self.assertEqual(SERVER_TYPE.Unknown, get_type(t, 'a'))
+        self.assertEqual(SERVER_TYPE.RSSecondary, get_type(t, 'b'))
+        self.assertEqual('rs', t.description.replica_set_name)
+        self.assertEqual(TOPOLOGY_TYPE.ReplicaSetNoPrimary,
+                         t.description.topology_type)
+
+        got_ismaster(t, ('a', 27017), {
+            'ok': 1,
+            'ismaster': True,
+            'setName': 'rs',
+            'hosts': ['a', 'b']})
+
+        self.assertEqual(SERVER_TYPE.RSPrimary, get_type(t, 'a'))
+        self.assertEqual(TOPOLOGY_TYPE.ReplicaSetWithPrimary,
+                         t.description.topology_type)
+
+    def test_handle_error_removed_server(self):
         t = create_mock_topology(replica_set_name='rs')
 
         # No error resetting a server not in the TopologyDescription.
-        t.reset_server(('b', 27017))
+        errctx = _ErrorContext(AutoReconnect('mock'), 0, 0, True)
+        t.handle_error(('b', 27017), errctx)
+
+        # Server was *not* added as type Unknown.
+        self.assertFalse(t.has_server(('b', 27017)))
+
+    def test_handle_getlasterror_removed_server(self):
+        t = create_mock_topology(replica_set_name='rs')
+
+        # No error resetting a server not in the TopologyDescription.
+        t.handle_getlasterror(('b', 27017), 'not master')
 
         # Server was *not* added as type Unknown.
         self.assertFalse(t.has_server(('b', 27017)))
@@ -654,6 +662,24 @@ class TestMultiServerTopology(TopologyTest):
 
         self.assertEqual(2, write_batch_size())
 
+    def test_topology_repr(self):
+        t = create_mock_topology(replica_set_name='rs')
+        self.addCleanup(t.close)
+        got_ismaster(t, ('a', 27017), {
+            'ok': 1,
+            'ismaster': True,
+            'setName': 'rs',
+            'hosts': ['a', 'c', 'b']})
+        self.assertEqual(
+            repr(t.description),
+            "<TopologyDescription id: %s, "
+            "topology_type: ReplicaSetWithPrimary, servers: ["
+            "<ServerDescription ('a', 27017) server_type: RSPrimary, rtt: 0>, "
+            "<ServerDescription ('b', 27017) server_type: Unknown,"
+            " rtt: None>, "
+            "<ServerDescription ('c', 27017) server_type: Unknown,"
+            " rtt: None>]>" % (t._topology_id,))
+
 
 def wait_for_master(topology):
     """Wait for a Topology to discover a writable server.
@@ -689,13 +715,14 @@ class TestTopologyErrors(TopologyTest):
                     raise AutoReconnect('mock monitor error')
 
         t = create_mock_topology(monitor_class=TestMonitor)
+        self.addCleanup(t.close)
         server = wait_for_master(t)
         self.assertEqual(1, ismaster_count[0])
-        pool_id = server.pool.pool_id
+        generation = server.pool.generation
 
         # Pool is reset by ismaster failure.
         t.request_check_all()
-        self.assertNotEqual(pool_id, server.pool.pool_id)
+        self.assertNotEqual(generation, server.pool.generation)
 
     def test_ismaster_retry(self):
         # ismaster succeeds at first, then raises socket error, then succeeds.
@@ -707,18 +734,26 @@ class TestTopologyErrors(TopologyTest):
                 if ismaster_count[0] in (1, 3):
                     return IsMaster({'ok': 1, 'maxWireVersion': 6}), 0
                 else:
-                    raise AutoReconnect('mock monitor error')
+                    raise AutoReconnect(
+                        'mock monitor error #%s' % (ismaster_count[0],))
 
         t = create_mock_topology(monitor_class=TestMonitor)
+        self.addCleanup(t.close)
         server = wait_for_master(t)
         self.assertEqual(1, ismaster_count[0])
         self.assertEqual(SERVER_TYPE.Standalone,
                          server.description.server_type)
 
-        # Second ismaster call, then immediately the third.
+        # Second ismaster call, server is marked Unknown, then the monitor
+        # immediately runs a retry (third ismaster).
         t.request_check_all()
+        # The third ismaster call (the immediate retry) happens sometime soon
+        # after the failed check triggered by request_check_all. Wait until
+        # the server becomes known again.
+        server = t.select_server(writable_server_selector, 0.250)
+        self.assertEqual(SERVER_TYPE.Standalone,
+                         server.description.server_type)
         self.assertEqual(3, ismaster_count[0])
-        self.assertEqual(SERVER_TYPE.Standalone, get_type(t, 'a'))
 
     def test_internal_monitor_error(self):
         exception = AssertionError('internal error')
@@ -728,6 +763,7 @@ class TestTopologyErrors(TopologyTest):
                 raise exception
 
         t = create_mock_topology(monitor_class=TestMonitor)
+        self.addCleanup(t.close)
         with self.assertRaisesRegex(ConnectionFailure, 'internal error'):
             t.select_server(any_server_selector,
                             server_selection_timeout=0.5)
@@ -738,7 +774,7 @@ class TestServerSelectionErrors(TopologyTest):
         with self.assertRaises(ConnectionFailure) as context:
             topology.select_server(selector, server_selection_timeout=0)
 
-        self.assertEqual(message, str(context.exception))
+        self.assertIn(message, str(context.exception))
 
     def test_no_primary(self):
         t = create_mock_topology(replica_set_name='rs')
@@ -765,12 +801,13 @@ class TestServerSelectionErrors(TopologyTest):
 
         self.assertMessage(
             'No replica set members match selector'
-            ' "Secondary(tag_sets=None, max_staleness=-1)"',
+            ' "Secondary(tag_sets=None, max_staleness=-1, hedge=None)"',
             t, ReadPreference.SECONDARY)
 
         self.assertMessage(
             "No replica set members match selector"
-            " \"Secondary(tag_sets=[{'dc': 'ny'}], max_staleness=-1)\"",
+            " \"Secondary(tag_sets=[{'dc': 'ny'}], max_staleness=-1, "
+            "hedge=None)\"",
             t, Secondary(tag_sets=[{'dc': 'ny'}]))
 
     def test_bad_replica_set_name(self):
