@@ -18,17 +18,21 @@ import copy
 import sys
 import warnings
 
+try:
+    from ssl import CERT_NONE
+except ImportError:
+    CERT_NONE = 0
+
 sys.path[0:0] = [""]
 
+from bson.binary import JAVA_LEGACY
+from bson.py3compat import string_type, _unicode
+from pymongo import ReadPreference
+from pymongo.errors import ConfigurationError, InvalidURI
 from pymongo.uri_parser import (parse_userinfo,
                                 split_hosts,
                                 split_options,
                                 parse_uri)
-from pymongo.errors import ConfigurationError, InvalidURI
-from pymongo.ssl_support import ssl
-from pymongo import ReadPreference
-from bson.binary import JAVA_LEGACY
-from bson.py3compat import string_type, _unicode
 from test import unittest
 
 
@@ -465,8 +469,8 @@ class TestURI(unittest.TestCase):
         # check that tlsInsecure is expanded correctly.
         uri = "mongodb://example.com/?tlsInsecure=true"
         res = {
-            "ssl_match_hostname": False, "ssl_cert_reqs": ssl.CERT_NONE,
-            "tlsinsecure": True}
+            "ssl_match_hostname": False, "ssl_cert_reqs": CERT_NONE,
+            "tlsinsecure": True, 'ssl_check_ocsp_endpoint': False}
         self.assertEqual(res, parse_uri(uri)["options"])
 
     def test_tlsinsecure_legacy_conflict(self):
@@ -494,6 +498,43 @@ class TestURI(unittest.TestCase):
         self.assertEqual(len(ctx), 1)
         self.assertTrue(issubclass(ctx[0].category, DeprecationWarning))
 
+    def test_unquote_after_parsing(self):
+        quoted_val = "val%21%40%23%24%25%5E%26%2A%28%29_%2B%2C%3A+etc"
+        unquoted_val = "val!@#$%^&*()_+,: etc"
+        uri = ("mongodb://user:password@localhost/?authMechanism=MONGODB-AWS"
+               "&authMechanismProperties=AWS_SESSION_TOKEN:"+quoted_val)
+        res = parse_uri(uri)
+        options = {
+            'authmechanism': 'MONGODB-AWS',
+            'authmechanismproperties': {
+                'AWS_SESSION_TOKEN': unquoted_val}}
+        self.assertEqual(options, res['options'])
+
+        uri = (("mongodb://localhost/foo?readpreference=secondary&"
+                "readpreferencetags=dc:west,"+quoted_val+":"+quoted_val+"&"
+                "readpreferencetags=dc:east,use:"+quoted_val))
+        res = parse_uri(uri)
+        options = {
+            'readpreference': ReadPreference.SECONDARY.mongos_mode,
+            'readpreferencetags': [
+                {'dc': 'west', unquoted_val: unquoted_val},
+                {'dc': 'east', 'use': unquoted_val}
+            ]
+        }
+        self.assertEqual(options, res['options'])
+
+    def test_redact_AWS_SESSION_TOKEN(self):
+        unquoted_colon = "token:"
+        uri = ("mongodb://user:password@localhost/?authMechanism=MONGODB-AWS"
+               "&authMechanismProperties=AWS_SESSION_TOKEN:"+unquoted_colon)
+        with self.assertRaisesRegex(
+                ValueError,
+                'auth mechanism properties must be key:value pairs like '
+                'SERVICE_NAME:mongodb, not AWS_SESSION_TOKEN:<redacted token>'
+                ', did you forget to percent-escape the token with '
+                'quote_plus?'):
+            parse_uri(uri)
+
 
 if __name__ == "__main__":
     unittest.main()
