@@ -20,6 +20,7 @@ import contextlib
 import functools
 import os
 import re
+import shutil
 import sys
 import threading
 import time
@@ -30,42 +31,33 @@ from functools import partial
 
 from bson import json_util, py3compat
 from bson.objectid import ObjectId
+from bson.son import SON
 
 from pymongo import (MongoClient,
                      monitoring, read_preferences)
 from pymongo.errors import ConfigurationError, OperationFailure
 from pymongo.monitoring import _SENSITIVE_COMMANDS, ConnectionPoolListener
-from pymongo.pool import PoolOptions
+from pymongo.pool import (_CancellationContext,
+                          PoolOptions)
 from pymongo.read_concern import ReadConcern
 from pymongo.read_preferences import ReadPreference
 from pymongo.server_selectors import (any_server_selector,
                                       writable_server_selector)
+from pymongo.server_type import SERVER_TYPE
 from pymongo.write_concern import WriteConcern
 
 from test import (client_context,
                   db_user,
                   db_pwd)
 
-IMPOSSIBLE_WRITE_CONCERN = WriteConcern(w=1000)
+if sys.version_info[0] < 3:
+    # Python 2.7, use our backport.
+    from test.barrier import Barrier
+else:
+    from threading import Barrier
 
 
-class WhiteListEventListener(monitoring.CommandListener):
-
-    def __init__(self, *commands):
-        self.commands = set(commands)
-        self.results = defaultdict(list)
-
-    def started(self, event):
-        if event.command_name in self.commands:
-            self.results['started'].append(event)
-
-    def succeeded(self, event):
-        if event.command_name in self.commands:
-            self.results['succeeded'].append(event)
-
-    def failed(self, event):
-        if event.command_name in self.commands:
-            self.results['failed'].append(event)
+IMPOSSIBLE_WRITE_CONCERN = WriteConcern(w=50)
 
 
 class CMAPListener(ConnectionPoolListener):
@@ -136,6 +128,25 @@ class EventListener(monitoring.CommandListener):
         self.results.clear()
 
 
+class WhiteListEventListener(EventListener):
+
+    def __init__(self, *commands):
+        self.commands = set(commands)
+        super(WhiteListEventListener, self).__init__()
+
+    def started(self, event):
+        if event.command_name in self.commands:
+            super(WhiteListEventListener, self).started(event)
+
+    def succeeded(self, event):
+        if event.command_name in self.commands:
+            super(WhiteListEventListener, self).succeeded(event)
+
+    def failed(self, event):
+        if event.command_name in self.commands:
+            super(WhiteListEventListener, self).failed(event)
+
+
 class OvertCommandListener(EventListener):
     """A CommandListener that ignores sensitive commands."""
     def started(self, event):
@@ -151,8 +162,7 @@ class OvertCommandListener(EventListener):
             super(OvertCommandListener, self).failed(event)
 
 
-class ServerAndTopologyEventListener(monitoring.ServerListener,
-                                     monitoring.TopologyListener):
+class _ServerEventListener(object):
     """Listens to all events."""
 
     def __init__(self):
@@ -167,6 +177,24 @@ class ServerAndTopologyEventListener(monitoring.ServerListener,
     def closed(self, event):
         self.results.append(event)
 
+    def matching(self, matcher):
+        """Return the matching events."""
+        results = self.results[:]
+        return [event for event in results if matcher(event)]
+
+    def reset(self):
+        self.results = []
+
+
+class ServerEventListener(_ServerEventListener,
+                          monitoring.ServerListener):
+    """Listens to Server events."""
+
+
+class ServerAndTopologyEventListener(ServerEventListener,
+                                     monitoring.TopologyListener):
+    """Listens to Server and Topology events."""
+
 
 class HeartbeatEventListener(monitoring.ServerHeartbeatListener):
     """Listens to only server heartbeat events."""
@@ -183,9 +211,18 @@ class HeartbeatEventListener(monitoring.ServerHeartbeatListener):
     def failed(self, event):
         self.results.append(event)
 
+    def matching(self, matcher):
+        """Return the matching events."""
+        results = self.results[:]
+        return [event for event in results if matcher(event)]
+
 
 class MockSocketInfo(object):
-    def close(self):
+    def __init__(self):
+        self.cancel_context = _CancellationContext()
+        self.more_to_come = False
+
+    def close_socket(self, reason):
         pass
 
     def __enter__(self):
@@ -197,11 +234,11 @@ class MockSocketInfo(object):
 
 class MockPool(object):
     def __init__(self, *args, **kwargs):
-        self.pool_id = 0
+        self.generation = 0
         self._lock = threading.Lock()
         self.opts = PoolOptions()
 
-    def get_socket(self, all_credentials):
+    def get_socket(self, all_credentials, checkout=False):
         return MockSocketInfo()
 
     def return_socket(self, *args, **kwargs):
@@ -209,7 +246,7 @@ class MockPool(object):
 
     def _reset(self):
         with self._lock:
-            self.pool_id += 1
+            self.generation += 1
 
     def reset(self):
         self._reset()
@@ -220,7 +257,7 @@ class MockPool(object):
     def update_is_writable(self, is_writable):
         pass
 
-    def remove_stale_sockets(self, reference_pool_id):
+    def remove_stale_sockets(self, *args, **kwargs):
         pass
 
 
@@ -410,12 +447,13 @@ def _connection_string(h, authenticate):
         return "mongodb://%s" % (str(h),)
 
 
-def _mongo_client(host, port, authenticate=True, direct=False, **kwargs):
+def _mongo_client(host, port, authenticate=True, directConnection=False,
+                  **kwargs):
     """Create a new client over SSL/TLS if necessary."""
     host = host or client_context.host
     port = port or client_context.port
     client_options = client_context.default_client_options.copy()
-    if client_context.replica_set_name and not direct:
+    if client_context.replica_set_name and not directConnection:
         client_options['replicaSet'] = client_context.replica_set_name
     client_options.update(kwargs)
 
@@ -427,12 +465,13 @@ def _mongo_client(host, port, authenticate=True, direct=False, **kwargs):
 
 def single_client_noauth(h=None, p=None, **kwargs):
     """Make a direct connection. Don't authenticate."""
-    return _mongo_client(h, p, authenticate=False, direct=True, **kwargs)
+    return _mongo_client(h, p, authenticate=False,
+                         directConnection=True, **kwargs)
 
 
 def single_client(h=None, p=None, **kwargs):
     """Make a direct connection, and authenticate if necessary."""
-    return _mongo_client(h, p, direct=True, **kwargs)
+    return _mongo_client(h, p, directConnection=True, **kwargs)
 
 
 def rs_client_noauth(h=None, p=None, **kwargs):
@@ -658,6 +697,16 @@ def wait_until(predicate, success_description, timeout=10):
         time.sleep(interval)
 
 
+def repl_set_step_down(client, **kwargs):
+    """Run replSetStepDown, first unfreezing a secondary with replSetFreeze."""
+    cmd = SON([('replSetStepDown', 1)])
+    cmd.update(kwargs)
+
+    # Unfreeze a secondary to ensure a speedy election.
+    client.admin.command(
+        'replSetFreeze', 0, read_preference=ReadPreference.SECONDARY)
+    client.admin.command(cmd)
+
 def is_mongos(client):
     res = client.admin.command('ismaster')
     return res.get('msg', '') == 'isdbgrid'
@@ -860,4 +909,32 @@ def parse_read_preference(pref):
     max_staleness = pref.get('maxStalenessSeconds', -1)
     tag_sets = pref.get('tag_sets')
     return read_preferences.make_read_preference(
-        mode, tag_sets=tag_sets, max_staleness=max_staleness)
\ No newline at end of file
+        mode, tag_sets=tag_sets, max_staleness=max_staleness)
+
+
+def server_name_to_type(name):
+    """Convert a ServerType name to the corresponding value. For SDAM tests."""
+    # Special case, some tests in the spec include the PossiblePrimary
+    # type, but only single-threaded drivers need that type. We call
+    # possible primaries Unknown.
+    if name == 'PossiblePrimary':
+        return SERVER_TYPE.Unknown
+    return getattr(SERVER_TYPE, name)
+
+
+def cat_files(dest, *sources):
+    """Cat multiple files into dest."""
+    with open(dest, 'wb') as fdst:
+        for src in sources:
+            with open(src, 'rb') as fsrc:
+                shutil.copyfileobj(fsrc, fdst)
+
+
+@contextlib.contextmanager
+def assertion_context(msg):
+    """A context manager that adds info to an assertion failure."""
+    try:
+        yield
+    except AssertionError as exc:
+        msg = '%s (%s)' % (exc, msg)
+        py3compat.reraise(type(exc), msg, sys.exc_info()[2])
