@@ -15,23 +15,26 @@
 """Utilities for testing driver specs."""
 
 import copy
-import sys
+import threading
 
 
 from bson import decode, encode
 from bson.binary import Binary, STANDARD
 from bson.codec_options import CodecOptions
 from bson.int64 import Int64
-from bson.py3compat import iteritems, abc, text_type
+from bson.py3compat import iteritems, abc, string_type, text_type
 from bson.son import SON
 
 from gridfs import GridFSBucket
 
 from pymongo import (client_session,
+                     helpers,
                      operations)
 from pymongo.command_cursor import CommandCursor
 from pymongo.cursor import Cursor
-from pymongo.errors import (OperationFailure, PyMongoError)
+from pymongo.errors import (BulkWriteError,
+                            OperationFailure,
+                            PyMongoError)
 from pymongo.read_concern import ReadConcern
 from pymongo.read_preferences import ReadPreference
 from pymongo.results import _WriteResult, BulkWriteResult
@@ -45,8 +48,46 @@ from test.utils import (camel_to_snake,
                         camel_to_snake_args,
                         camel_to_upper_camel,
                         CompareType,
+                        CMAPListener,
                         OvertCommandListener,
-                        rs_client, parse_read_preference)
+                        parse_read_preference,
+                        rs_client,
+                        ServerAndTopologyEventListener,
+                        HeartbeatEventListener)
+
+
+class SpecRunnerThread(threading.Thread):
+    def __init__(self, name):
+        super(SpecRunnerThread, self).__init__()
+        self.name = name
+        self.exc = None
+        self.setDaemon(True)
+        self.cond = threading.Condition()
+        self.ops = []
+        self.stopped = False
+
+    def schedule(self, work):
+        self.ops.append(work)
+        with self.cond:
+            self.cond.notify()
+
+    def stop(self):
+        self.stopped = True
+        with self.cond:
+            self.cond.notify()
+
+    def run(self):
+        while not self.stopped or self.ops:
+            if not self. ops:
+                with self.cond:
+                    self.cond.wait(10)
+            if self.ops:
+                try:
+                    work = self.ops.pop(0)
+                    work()
+                except Exception as exc:
+                    self.exc = exc
+                    self.stop()
 
 
 class SpecRunner(IntegrationTest):
@@ -57,7 +98,8 @@ class SpecRunner(IntegrationTest):
         cls.mongos_clients = []
 
         # Speed up the tests by decreasing the heartbeat frequency.
-        cls.knobs = client_knobs(min_heartbeat_interval=0.1)
+        cls.knobs = client_knobs(heartbeat_frequency=0.1,
+                                 min_heartbeat_interval=0.1)
         cls.knobs.enable()
 
     @classmethod
@@ -67,7 +109,10 @@ class SpecRunner(IntegrationTest):
 
     def setUp(self):
         super(SpecRunner, self).setUp()
+        self.targets = {}
         self.listener = None
+        self.pool_listener = None
+        self.server_listener = None
         self.maxDiff = None
 
     def _set_fail_point(self, client, command_args):
@@ -107,6 +152,26 @@ class SpecRunner(IntegrationTest):
         self.assertIsNone(session._pinned_address)
         self.assertIsNone(session._transaction.pinned_address)
 
+    def assert_collection_exists(self, database, collection):
+        """Run the assertCollectionExists test operation."""
+        db = self.client[database]
+        self.assertIn(collection, db.list_collection_names())
+
+    def assert_collection_not_exists(self, database, collection):
+        """Run the assertCollectionNotExists test operation."""
+        db = self.client[database]
+        self.assertNotIn(collection, db.list_collection_names())
+
+    def assert_index_exists(self, database, collection, index):
+        """Run the assertIndexExists test operation."""
+        coll = self.client[database][collection]
+        self.assertIn(index, [doc['name'] for doc in coll.list_indexes()])
+
+    def assert_index_not_exists(self, database, collection, index):
+        """Run the assertIndexNotExists test operation."""
+        coll = self.client[database][collection]
+        self.assertNotIn(index, [doc['name'] for doc in coll.list_indexes()])
+
     def assertErrorLabelsContain(self, exc, expected_labels):
         labels = [l for l in expected_labels if exc.has_error_label(l)]
         self.assertEqual(labels, expected_labels)
@@ -204,6 +269,25 @@ class SpecRunner(IntegrationTest):
         if 'maxCommitTimeMS' in opts:
             opts['max_commit_time_ms'] = opts.pop('maxCommitTimeMS')
 
+        if 'hint' in opts:
+            hint = opts.pop('hint')
+            if not isinstance(hint, string_type):
+                hint = list(iteritems(hint))
+            opts['hint'] = hint
+
+        # Properly format 'hint' arguments for the Bulk API tests.
+        if 'requests' in opts:
+            reqs = opts.pop('requests')
+            for req in reqs:
+                args = req.pop('arguments')
+                if 'hint' in args:
+                    hint = args.pop('hint')
+                    if not isinstance(hint, string_type):
+                        hint = list(iteritems(hint))
+                    args['hint'] = hint
+                req['arguments'] = args
+            opts['requests'] = reqs
+
         return dict(opts)
 
     def run_operation(self, sessions, collection, operation):
@@ -273,7 +357,8 @@ class SpecRunner(IntegrationTest):
                 arguments["requests"] = requests
             elif arg_name == "session":
                 arguments['session'] = sessions[arguments['session']]
-            elif name == 'command' and arg_name == 'command':
+            elif (name in ('command', 'run_admin_command') and
+                  arg_name == 'command'):
                 # Ensure the first key is the command name.
                 ordered_command = SON([(operation['command_name'], 1)])
                 ordered_command.update(arguments['command'])
@@ -290,9 +375,21 @@ class SpecRunner(IntegrationTest):
                 arguments['callback'] = lambda _: self.run_operations(
                     sessions, original_collection, copy.deepcopy(callback_ops),
                     in_with_transaction=True)
+            elif name == 'drop_collection' and arg_name == 'collection':
+                arguments['name_or_collection'] = arguments.pop(arg_name)
+            elif name == 'create_collection' and arg_name == 'collection':
+                arguments['name'] = arguments.pop(arg_name)
+            elif name == 'create_index' and arg_name == 'keys':
+                arguments['keys'] = list(arguments.pop(arg_name).items())
+            elif name == 'drop_index' and arg_name == 'name':
+                arguments['index_or_name'] = arguments.pop(arg_name)
             else:
                 arguments[c2s] = arguments.pop(arg_name)
 
+        if name == 'run_on_thread':
+            args = {'sessions': sessions, 'collection': collection}
+            args.update(arguments)
+            arguments = args
         result = cmd(**dict(arguments))
 
         if name == "aggregate":
@@ -317,41 +414,48 @@ class SpecRunner(IntegrationTest):
         """Allow encryption spec to override expected error classes."""
         return (PyMongoError,)
 
+    def _run_op(self, sessions, collection, op, in_with_transaction):
+        expected_result = op.get('result')
+        if expect_error(op):
+            with self.assertRaises(self.allowable_errors(op),
+                                   msg=op['name']) as context:
+                self.run_operation(sessions, collection, op.copy())
+
+            if expect_error_message(expected_result):
+                if isinstance(context.exception, BulkWriteError):
+                    errmsg = str(context.exception.details).lower()
+                else:
+                    errmsg = str(context.exception).lower()
+                self.assertIn(expected_result['errorContains'].lower(),
+                              errmsg)
+            if expect_error_code(expected_result):
+                self.assertEqual(expected_result['errorCodeName'],
+                                 context.exception.details.get('codeName'))
+            if expect_error_labels_contain(expected_result):
+                self.assertErrorLabelsContain(
+                    context.exception,
+                    expected_result['errorLabelsContain'])
+            if expect_error_labels_omit(expected_result):
+                self.assertErrorLabelsOmit(
+                    context.exception,
+                    expected_result['errorLabelsOmit'])
+
+            # Reraise the exception if we're in the with_transaction
+            # callback.
+            if in_with_transaction:
+                raise context.exception
+        else:
+            result = self.run_operation(sessions, collection, op.copy())
+            if 'result' in op:
+                if op['name'] == 'runCommand':
+                    self.check_command_result(expected_result, result)
+                else:
+                    self.check_result(expected_result, result)
+
     def run_operations(self, sessions, collection, ops,
                        in_with_transaction=False):
         for op in ops:
-            expected_result = op.get('result')
-            if expect_error(op):
-                with self.assertRaises(self.allowable_errors(op),
-                                       msg=op['name']) as context:
-                    self.run_operation(sessions, collection, op.copy())
-
-                if expect_error_message(expected_result):
-                    self.assertIn(expected_result['errorContains'].lower(),
-                                  str(context.exception).lower())
-                if expect_error_code(expected_result):
-                    self.assertEqual(expected_result['errorCodeName'],
-                                     context.exception.details.get('codeName'))
-                if expect_error_labels_contain(expected_result):
-                    self.assertErrorLabelsContain(
-                        context.exception,
-                        expected_result['errorLabelsContain'])
-                if expect_error_labels_omit(expected_result):
-                    self.assertErrorLabelsOmit(
-                        context.exception,
-                        expected_result['errorLabelsOmit'])
-
-                # Reraise the exception if we're in the with_transaction
-                # callback.
-                if in_with_transaction:
-                    raise context.exception
-            else:
-                result = self.run_operation(sessions, collection, op.copy())
-                if 'result' in op:
-                    if op['name'] == 'runCommand':
-                        self.check_command_result(expected_result, result)
-                    else:
-                        self.check_result(expected_result, result)
+            self._run_op(sessions, collection, op, in_with_transaction)
 
     # TODO: factor with test_command_monitoring.py
     def check_events(self, test, listener, session_ids):
@@ -463,7 +567,29 @@ class SpecRunner(IntegrationTest):
 
     def run_scenario(self, scenario_def, test):
         self.maybe_skip_scenario(test)
+
+        # Kill all sessions before and after each test to prevent an open
+        # transaction (from a test failure) from blocking collection/database
+        # operations during test set up and tear down.
+        self.kill_all_sessions()
+        self.addCleanup(self.kill_all_sessions)
+        self.setup_scenario(scenario_def)
+        database_name = self.get_scenario_db_name(scenario_def)
+        collection_name = self.get_scenario_coll_name(scenario_def)
+        # SPEC-1245 workaround StaleDbVersion on distinct
+        for c in self.mongos_clients:
+            c[database_name][collection_name].distinct("x")
+
+        # Configure the fail point before creating the client.
+        if 'failPoint' in test:
+            fp = test['failPoint']
+            self.set_fail_point(fp)
+            self.addCleanup(self.set_fail_point, {
+                'configureFailPoint': fp['configureFailPoint'], 'mode': 'off'})
+
         listener = OvertCommandListener()
+        pool_listener = CMAPListener()
+        server_listener = ServerAndTopologyEventListener()
         # Create a new client, to avoid interference from pooled sessions.
         client_options = self.parse_client_options(test['clientOptions'])
         # MMAPv1 does not support retryable writes.
@@ -472,32 +598,29 @@ class SpecRunner(IntegrationTest):
             self.skipTest("MMAPv1 does not support retryWrites=True")
         use_multi_mongos = test['useMultipleMongoses']
         if client_context.is_mongos and use_multi_mongos:
-            client = rs_client(client_context.mongos_seeds(),
-                               event_listeners=[listener], **client_options)
+            client = rs_client(
+                client_context.mongos_seeds(),
+                event_listeners=[listener, pool_listener, server_listener],
+                **client_options)
         else:
-            client = rs_client(event_listeners=[listener], **client_options)
+            client = rs_client(
+                event_listeners=[listener, pool_listener, server_listener],
+                **client_options)
+        self.scenario_client = client
         self.listener = listener
+        self.pool_listener = pool_listener
+        self.server_listener = server_listener
         # Close the client explicitly to avoid having too many threads open.
         self.addCleanup(client.close)
 
-        # Kill all sessions before and after each test to prevent an open
-        # transaction (from a test failure) from blocking collection/database
-        # operations during test set up and tear down.
-        self.kill_all_sessions()
-        self.addCleanup(self.kill_all_sessions)
-
-        database_name = self.get_scenario_db_name(scenario_def)
-        collection_name = self.get_scenario_coll_name(scenario_def)
-        self.setup_scenario(scenario_def)
-
-        # SPEC-1245 workaround StaleDbVersion on distinct
-        for c in self.mongos_clients:
-            c[database_name][collection_name].distinct("x")
-
         # Create session0 and session1.
         sessions = {}
         session_ids = {}
         for i in range(2):
+            # Don't attempt to create sessions if they are not supported by
+            # the running server version.
+            if not client_context.sessions_enabled:
+                break
             session_name = 'session%d' % i
             opts = camel_to_snake_args(test['sessionOptions'][session_name])
             if 'default_transaction_options' in opts:
@@ -514,14 +637,6 @@ class SpecRunner(IntegrationTest):
 
         self.addCleanup(end_sessions, sessions)
 
-        if 'failPoint' in test:
-            fp = test['failPoint']
-            self.set_fail_point(fp)
-            self.addCleanup(self.set_fail_point, {
-                'configureFailPoint': fp['configureFailPoint'], 'mode': 'off'})
-
-        listener.results.clear()
-
         collection = client[database_name][collection_name]
         self.run_test_ops(sessions, collection, test)
 
@@ -549,11 +664,11 @@ class SpecRunner(IntegrationTest):
                 outcome_coll_name,
                 read_preference=ReadPreference.PRIMARY,
                 read_concern=ReadConcern('local'))
+            actual_data = list(outcome_coll.find(sort=[('_id', 1)]))
 
             # The expected data needs to be the left hand side here otherwise
             # CompareType(Binary) doesn't work.
-            self.assertEqual(
-                wrap_types(expected_c['data']), list(outcome_coll.find()))
+            self.assertEqual(wrap_types(expected_c['data']), actual_data)
 
 
 def expect_any_error(op):
